% Detect onset from EMG signal using Morphological operation
% See: Dapeng Yang, Huajie Zhang, Yikun Gu, Hong Liu, Accurate EMG onset
% detection in pathological, weak and noisy myoelectric signals. 
% Biomedical Signal Processing and Control, Volume 33, 2017, Pages 306-315

% th_scale    % Scale of the threshold of TKEO
% len_close   % Window length of the kernel of closing operator
% len_open    % Window length of the kernel of opening operator


% TIPS:
% If detected onsets are few, or onsets are delayed
%   -> Decrease th_scale to make threshold lower
% If an onset is separated into several onsets
%   -> Increase len_close to merge them
% If short onsets exist
%   -> Increase len_open to remove them

% len_close ensures minimum between onsets term
% len_open eunsures minimum onset duration


function [trigger, phi, phi_bin, t0_mco, thr] = detect_onset(x, th_scale, len_close, len_open)
disp('Start trigger onset detection ...')

% Demean signal
x = x - mean(x);

% Compute the Taeger-Kaiser energy operator
matver = version('-release'); % Detect matlab ver because circshift changed after 2016b
if verLessThan('matlab', '9.0') % 9.0 is R2016a
    [sx1,sx2] = size(x);
    [~,sdim] = max([sx1, sx2]);
    if sdim==1
        phi = x.^2 - circshift(x,1).*circshift(x,-1);
    else
        phi = x'.^2 - circshift(x',1).*circshift(x',-1);
        phi = phi';
    end
else
    [sx1,sx2] = size(x);
    [~,sdim] = max([sx1, sx2]);
    phi = x.^2 - circshift(x,1,sdim).*circshift(x,-1,sdim);
end
phi(1) = 0; phi(end) = 0; % first and last elements are not defined

% Compute threshold (mean + scale*std)
thr = mean(phi(2:end-1)) + th_scale*std(phi(2:end-1));

% Binarize TKEO
phi_bin = zeros(size(phi));
phi_bin(phi > thr) = 1;

% Morphological close operation to concat continuous signals
% SE = strel('rectangle',[1 len_close]); % It requires ImageProcToolbox
% t0_mc = imclose(phi_bin,SE); % It requires ImageProcToolbox
t0_mc = vb_closing_2d(phi_bin, len_close);

% Morphological open operation to remove noisy signals
% SE = strel('rectangle',[1 len_open]);% It requires ImageProcToolbox
% t0_mco = imopen(t0_mc,SE); % It requires ImageProcToolbox
t0_mco = vb_opening_2d(t0_mc, len_open);

% t0_mco is square wave, so extract onset of them
trigger = zeros(size(x));
trigger((t0_mco(1:end-1) - t0_mco(2:end))<0) = 1;