function import_eog_squid(p, input_dirname, eeg_dirname)
% Import EOG data from EEG to SQUID file
% EOG data is registered as 'EOG1' and 'EOG2' in Extra Channel of EEG file
% (Except for Motor task of s093)
%
% Copyright (C) 2011, ATR All Rights Reserved.
% License : New BSD License(see VBMEG_LICENSE.txt)

disp(mfilename);

set_fig_property(4, 2, 10, 10);

% Set input directory
input_dir_meg = fullfile(p.proj_root, p.dirname.modality, input_dirname, p.task);
input_dir_eeg = fullfile(p.proj_root, eeg_dirname, input_dirname, p.task);

% Set sync channel of MEG
switch p.task
    case 'Somatosensory'
        ext_chname = {'432'};
    case 'Auditory'
        ext_chname = {'433'};
    case {'Motor', 'Rest'}
        ext_chname = {'435'};
end

% Set sync channel of EEG
ext_chname_eeg = {'EXT1'};

% EOG channels of EEG
eog_ch_name = {'EOG1', 'EOG2'};

% EOG channels will be registered in MEG as
new_eog_ch_name = {'EOG1', 'EOG2'};
new_eog_ch_type = [-2, -2];

for run = 1:p.num_run
    file_name_meg = [sprintf('run%02d', run), '.meg.mat'];
    file_name_eeg = [sprintf('run%02d', run), '.eeg.mat'];
    meg_file = fullfile(input_dir_meg, file_name_meg);
    eog_file = fullfile(input_dir_eeg, file_name_eeg);

    %--- In case of s093, EOG was recorded in MEG file ----------
    if strcmp(p.sub, '093')
        eog_file = meg_file;
        eog_ch_name = {'416', '417'};
        % Load EOG data
        loadspec = [];
        loadspec.ActiveChannel = false;
        loadspec.ChannelType = 'EXTRA';
        loadspec.ChannelName = eog_ch_name;
        [eog, channel_info_eog, time_info_eog] = vb_load_meg_data(eog_file, loadspec);
        eog_new = eog;
        % Append EOG ch to input file
        append_ch = [];
        append_ch.data = eog_new;
        append_ch.name = new_eog_ch_name;
        append_ch.type = new_eog_ch_type;
        append_extra_ch(meg_file, append_ch)
        % Below process is unnecessary.
        continue
    end
    %-----------------------------------------------------------

    %% Load data
    % Load sync signal
    loadspec = [];
    loadspec.ActiveChannel = false;
    loadspec.ChannelType = 'EXTRA';
    loadspec.ChannelName = ext_chname;
    [sync_meg, channel_info, time_info] = vb_load_meg_data(meg_file, loadspec);
    time = time_info.time;
    freq = time_info.sample_frequency;
    
    % Load EOG data
    loadspec = [];
    loadspec.ActiveChannel = false;
    loadspec.ChannelType = 'EXTRA';
    loadspec.ChannelName = eog_ch_name;
    [eog, channel_info_eog, time_info_eog] = vb_load_meg_data(eog_file, loadspec);
    time_eog = time_info_eog.time;
    freq_eog = time_info_eog.sample_frequency;

    % Load trigger or sync signal
    loadspec = [];
    loadspec.ActiveChannel = false;
    loadspec.ChannelType = 'EXTRA';
    loadspec.ChannelName = ext_chname_eeg;
    [sync_eeg, ~, ~] = vb_load_meg_data(eog_file, loadspec);

    %% Import 
    [eog_new, sync_onset_new, sync_onset_src, sync_onset_dst] =import_signal_using_sync(sync_eeg, sync_meg, eog);

    % Compare onset index of dst and new (imported src index)
    ix_onset_new = find(sync_onset_new);
    ix_onset_dst = find(sync_onset_dst);
    dif = abs(ix_onset_dst-ix_onset_new);

    dif_ms = dif ./ freq*1000;
    max_diff_ms = num2str(max(dif)/freq*1000);
    
    %% Check result
    h=figure;
    subplot(2,1,1), plot(time, sync_onset_dst - sync_onset_new)
    xlim([-inf inf])
    xlabel('Time [sec]')
    ylabel('Differences [-1/0/1]')
    title('sync onset (MEG) - sync onset (imported EOG)')
    subplot(2,1,2), plot(dif_ms), 
    xlim([-inf inf])
    xlabel('#Pulse')
    ylabel('Differences [ms]')
    title(['Diff of pulse timing between EOG and MEG (max ' num2str(max_diff_ms) ' ms)'])
    fig_file = fullfile(p.fig_root, mfilename, [p.sub '_' p.task '_' num2str(run)]);
    vb_savefig_as_shown(h, fig_file)
    disp([fig_file '.png was saved.'])

    % Append EOG ch to input file
    append_ch = [];
    append_ch.data = eog_new;
    append_ch.name = new_eog_ch_name;
    append_ch.type = new_eog_ch_type;
    append_extra_ch(meg_file, append_ch)
end



