function import_trigger(p, input_dirname, ref_modality)
% Import trigger from different modality (ref)
% This is used for importing Motor trigger from SQUID to EEG
% In case of Somatosensory of s093, trigger of EEG was mis-recorded, 
% hence import trigger from SQUID.
%
% Copyright (C) 2011, ATR All Rights Reserved.
% License : New BSD License(see VBMEG_LICENSE.txt)

disp(mfilename);

set_fig_property(4, 2, 10, 10);

% Set input directory
input_dir = fullfile(p.proj_root, p.dirname.modality, input_dirname, p.task);
input_dir_ref = fullfile(p.proj_root, ref_modality, input_dirname, p.task);

% Set sync channel of reference and target modality
if strcmp(p.task, 'Motor') || strcmp(p.task, 'Rest')
    sync_chname_ref = {'435'}; % SQUID
    sync_chname = {'EXT1'}; % EEG
    ignore_mismatch = false;
elseif strcmp(p.task, 'Somatosensory') && strcmp(p.sub, '093')
    sync_chname_ref = {'435'}; % SQUID
    sync_chname = {'EXT2'}; % EEG
    ignore_mismatch = 'end'; % Ignore mis-matched sync signal
else
    warning('No need to conduct importing trigger.')
    return
end

% Trigger channels of ref
trg_ch_name = {'trigger_pulse'};

% Trigger channel will be registered as
new_trg_ch_name = trg_ch_name;
new_trg_ch_type = [1];

for run = 1:p.num_run
    file_name_ref = [sprintf('run%02d', run), '.meg.mat'];
    file_name = [sprintf('run%02d', run), '.eeg.mat'];
    meg_file = fullfile(input_dir_ref, file_name_ref);
    eeg_file = fullfile(input_dir, file_name);

    %% Load data
    % Load sync signal from MEG
    loadspec = [];
    loadspec.ActiveChannel = false;
    loadspec.ChannelType = 'EXTRA';
    loadspec.ChannelName = sync_chname_ref;
    [sync_meg, ~, ~] = vb_load_meg_data(meg_file, loadspec);

    % Load sync signal from EEG
    loadspec = [];
    loadspec.ActiveChannel = false;
    loadspec.ChannelType = 'EXTRA';
    loadspec.ChannelName = sync_chname;
    [sync_eeg, ~, time_info] = vb_load_meg_data(eeg_file, loadspec);
    time_eeg = time_info.time;
    freq_eeg = time_info.sample_frequency;
    
    % Load trigger data from MEG
    loadspec = [];
    loadspec.ActiveChannel = false;
    loadspec.ChannelType = 'EXTRA';
    loadspec.ChannelName = trg_ch_name;
    [trigger, ~, ~] = vb_load_meg_data(meg_file, loadspec);

    %% Import 
    [trigger_new, sync_onset_new, sync_onset_src, sync_onset_dst] =import_signal_using_sync(sync_meg, sync_eeg, trigger, ignore_mismatch);

    % Compare onset index of dst and new (imported src index)
    ix_onset_new = find(sync_onset_new);
    ix_onset_dst = find(sync_onset_dst);
    dif = abs(ix_onset_dst-ix_onset_new);

    dif_ms = dif ./ freq_eeg*1000;
    max_diff_ms = num2str(max(dif)/freq_eeg*1000);
    
    %% Check result
    h=figure;
    subplot(2,1,1), plot(time_eeg, sync_onset_dst - sync_onset_new)
    xlim([-inf inf])
    xlabel('Time [sec]')
    ylabel('Differences [-1/0/1]')
    title('sync onset (EEG) - sync onset (imported MEG)')
    subplot(2,1,2), plot(dif_ms), 
    xlim([-inf inf])
    xlabel('#Pulse')
    ylabel('Differences [ms]')
    title(['Diff of pulse timing between EEG and MEG (max ' num2str(max_diff_ms) ' ms)'])
    fig_file = fullfile(p.fig_root, mfilename, [p.sub '_' p.task '_' num2str(run)]);
    vb_savefig_as_shown(h, fig_file)
    disp([fig_file '.png was saved.'])

    % Append EOG ch to input file
    append_ch = [];
    append_ch.data = trigger_new;
    append_ch.name = new_trg_ch_name;
    append_ch.type = new_trg_ch_type;
    append_extra_ch(eeg_file, append_ch)
end



