function estimate_source_current_SOLCO(p, prefix_in, METHOD)
% Estimate source currents from MEG/EEG data using SOLCO
% METHOD : Method can be chosen from 
%          {'wmn', 'lcmv', 'mc', 'loreta'}
%
% Copyright (C) 2011, ATR All Rights Reserved.
% License : New BSD License(see VBMEG_LICENSE.txt)

disp(mfilename);

% Set default method
if ~exist('METHOD', 'var') || isempty(METHOD)
    METHOD = {'lcmv'};
end
if ~iscell(METHOD)
    METHOD = {METHOD};
end

% Set prefix of input file
if isempty(prefix_in)
    prefix_in_ = [];
else
    prefix_in_ = [prefix_in '_'];
end

%% Set common parameters
time_noise_sec = p.time_noise_sec; % Period for baseline [s]

% Set input files (abs path)
brainfile = fullfile(p.proj_root, p.dirname.brain, [p.struct_name, '.brain.mat']);
areafile  = fullfile(p.proj_root, p.dirname.brain, [p.struct_name, '.area.mat']);
actfile   = fullfile(p.proj_root, p.dirname.brain, [p.struct_name, '.act.mat']);
datafile  = fullfile(p.proj_root, p.dirname.modality, p.dirname.trial, [prefix_in_ p.task '.info.mat']);
basisfile = fullfile(p.proj_root, p.dirname.modality, p.dirname.leadfield, [p.task '.basis.mat']); % Use modified basis

% Load temporal peaks detected in find_temporal_peak.m
load(datafile, 'time_peak_sec', 'time_peak_win_sec')
if ~exist('time_peak_win_sec','var') || isempty(time_peak_win_sec)
    % If window of temporal peak is not specified, use time of interest
    time_peak_win_sec = p.time_of_interest_sec;
end
num_peak = size(time_peak_win_sec, 1);

% Load time information
[~, ~, time_info] = vb_load_meg_data(datafile);
time_sec = time_info.time;

% Set index of the time window for baseline
[~, from_ix] = min(abs(time_sec-time_noise_sec(1)));
[~, to_ix] = min(abs(time_sec-time_noise_sec(2)));
twin_noise_ix = [from_ix to_ix];

% Set output directory
outputdir = fullfile(p.proj_root, p.dirname.modality, p.dirname.current);
if ~exist(outputdir, 'dir')
    mkdir(outputdir)
end

%% Estimation for each peak window
for peak=1:num_peak
% Set time window
time_win_sec = time_peak_win_sec(peak, :);

% Set index of the time window for stimulus-evoked activity
[~, from_ix] = min(abs(time_sec-time_win_sec(1)));
[~, to_ix]   = min(abs(time_sec-time_win_sec(2)));
twin_meg_ix  = [from_ix to_ix];

%% Start estimation
for me=1:length(METHOD)
    method = METHOD{me};
    
    % Set ID of output files
    fileid = [p.task '_' method '_' num2str(peak)];
    
    % Set output current file
    filename = [fileid '.curr.mat'];
    currfile = fullfile(outputdir, filename);

    % Estimation using selected method
    disp(['Start estimation by ' method ' ...'])
    switch method
        case 'wmn' % Estimation by wMN
            wmn_parm = set_wmn_parm( ...
                'brainfile',brainfile,...
                'megfile',   datafile,...
                'basisfile', basisfile,...
                'areafile',  areafile,...
                'actfile',   actfile,...
                'twin_meg',  twin_meg_ix,...
                'twin_noise',twin_noise_ix,...
                'currfile',  currfile);
            job_wmn(wmn_parm);

        case 'lcmv' % Estimation by scalar beamformer
            lcmv_parm = set_lcmv_parm( ...
                'brainfile',brainfile,...
                'megfile',   datafile,...
                'basisfile', basisfile,...
                'areafile',  areafile,...
                'actfile',   actfile,...
                'twin_meg',  twin_meg_ix,...
                'twin_noise',twin_noise_ix,...
                'currfile',  currfile,...
                'cov_reg',   0.05,...
                'basis_normalization', OFF);
            job_lcmv(lcmv_parm);

        case 'mc' % Estimation by Minimum current (L1-norm)
            mc_parm = set_mc_parm( ...
                'brainfile',brainfile,...
                'megfile',   datafile,...
                'basisfile', basisfile,...
                'areafile',  areafile,...
                'actfile',   actfile,...
                'twin_meg',  twin_meg_ix,...
                'twin_noise',twin_noise_ix,...
                'currfile',  currfile);
            job_mc(mc_parm);

        case 'loreta' % Estimation by LORETA
            laplacianfile = fullfile(outputdir, [fileid '_laplacian.mat']);
            lor_parm = set_loreta_parm( ...
                'brainfile',brainfile,...
                'megfile',   datafile,...
                'basisfile', basisfile,...
                'laplacianfile', laplacianfile,...
                'areafile',  areafile,...
                'actfile',   actfile,...
                'twin_meg',  twin_meg_ix,...
                'twin_noise',twin_noise_ix,...
                'currfile',  currfile);
            job_loreta(lor_parm);

        case 'sloreta' % Estimation by sLORETA
            slor_parm = set_sloreta_parm( ...
                'brainfile',brainfile,...
                'megfile',   datafile,...
                'basisfile', basisfile,...
                'areafile',  areafile,...
                'actfile',   actfile,...
                'twin_meg',  twin_meg_ix,...
                'twin_noise',twin_noise_ix,...
                'currfile',  currfile);
            job_sloreta(slor_parm);

        otherwise
            warning(['Select method from {''wmn'', ''lcmv'', ''mc'', ''loreta'', ''sloreta''}'])

    end % End of method switch

    % Append temporal peak to current file
    save(currfile, 'time_win_sec', '-append')

    % Append input MEG/EEG file to current file
    save(currfile, 'datafile', '-append')

end % End of METHOD loop

end % End of peak loop