function estimate_source_current_VBMEG(p, prefix_in)
% Estimate source currents from MEG/EEG data using VBMEG
%
% Copyright (C) 2011, ATR All Rights Reserved.
% License : New BSD License(see VBMEG_LICENSE.txt)

disp(mfilename);

% No additional prefix

% Set prefix of input file
if isempty(prefix_in)
    prefix_in_ = [];
else
    prefix_in_ = [prefix_in '_'];
end

% Set parameter
time_noise_sec = p.time_noise_sec; % Period for baseline [s]

% Set input file
data_file = fullfile(p.proj_root, p.dirname.modality, p.dirname.trial, [prefix_in_ p.task '.info.mat']);

% Load temporal peaks detected in find_temporal_peak.m
load(data_file, 'time_peak_sec', 'time_peak_win_sec')
if ~exist('time_peak_win_sec','var') || isempty(time_peak_win_sec)
    % If window of temporal peak is not specified, use time of interest
    time_peak_win_sec = p.time_of_interest_sec;
end
num_peak = size(time_peak_win_sec, 1);

% Load time information
[~, ~, time_info] = vb_load_meg_data(data_file);
time_sec = time_info.time;

%% Set params for the estimation of variance
clear bp
bayes_parm = vb_set_bayes_default_parameters;

% Set input files (abs path)
bayes_parm.brainfile   = fullfile(p.proj_root, p.dirname.brain, [p.struct_name, '.brain.mat']);
bayes_parm.areafile    = fullfile(p.proj_root, p.dirname.brain, [p.struct_name, '.area.mat']);
bayes_parm.actfile     = fullfile(p.proj_root, p.dirname.brain, [p.struct_name, '.act.mat']);
bayes_parm.megfile{1}  = data_file;
bayes_parm.basisfile   = fullfile(p.proj_root, p.dirname.modality, p.dirname.leadfield, [p.task '.basis.mat']); % Use modified basis
bayes_parm.megfile_baseline = bayes_parm.megfile;

% Specify current area and fMRI activity
bayes_parm.area_key     = 'Cortex'; % Current area
bayes_parm.act_key      = p.prior;  % Prior for current variance
bayes_parm.prior_weight = p.prior_weight; % Relative influence of prior information (0-1)

% Set time window for baseline
[~, from_ix] = min(abs(time_sec-time_noise_sec(1)));
[~, to_ix]   = min(abs(time_sec-time_noise_sec(2)));
bayes_parm.twin_noise    = [from_ix to_ix];
bayes_parm.twin_baseline = [from_ix to_ix];

% Estimation parameters
bayes_parm.patch_norm    = ON; % Patch area normalization is applied or not
bayes_parm.trial_average = OFF; % Estimation using averaged signal or not
bayes_parm.Ntrain        = 3000;

% Set parameters for noise covariance estimation
bayes_parm = vb_set_noise_estimation_model(bayes_parm);

%% Set params for the estimation of variance
% Set input file (abs path)
current_parm.megfile = bayes_parm.megfile;

% Trial-averaged current will be estimated (if ON).
% current_parm.trial_average = ON;
current_parm.trial_average = OFF;

% Set time window for estimating current
% (including baseline)
current_parm.twin_meg = [1 length(time_sec)];
current_parm.Tperiod = length(time_sec);

%% Estimation for each peak window
for peak=1:num_peak
    % Set ID of output files
    fileid = [p.task '_' p.prior strrep(num2str(p.prior_weight),'.','') '_' num2str(peak)];

    % Set time window
    time_win_sec = time_peak_win_sec(peak, :);

    %% Estimate current variance
    % Set time window for stimulus-evoked activity
    [~, from_ix] = min(abs(time_sec-time_win_sec(1)));
    [~, to_ix]   = min(abs(time_sec-time_win_sec(2)));
    bayes_parm.twin_meg = [from_ix to_ix]; % Time window for analysis
    bayes_parm.Tperiod  = to_ix-from_ix+1; % Time period for current estimation
    bayes_parm.Tnext    = to_ix-from_ix+1; % Time step for next period

    % Set output file
    bayes_parm.bayesfile = fullfile(p.proj_root, p.dirname.modality, p.dirname.current, [fileid '.bayes.mat']);

    % Estimate current variance
    vb_job_vb(bayes_parm)

    %% Estimate source current
    % Set input file (abs path)
    current_parm.bayesfile = bayes_parm.bayesfile;

    % Set output file
    curr_file = fullfile(p.proj_root, p.dirname.modality, p.dirname.current, [fileid '.curr.mat']);
    current_parm.currfile = curr_file;
    current_parm.jactdir  = fileid;

    % Estimate source current
    vb_job_current(current_parm);

    % Append temporal peak to current file
    save(curr_file, 'time_win_sec', '-append')

    %% Append condition information to current file
    % Load condition information
    load(data_file, 'fileinfo')
    trial_ix = find(fileinfo.ActiveTrial == 1);
    cond = fileinfo.cond_id(trial_ix); % Extract cond of active trials

    % Save condition information
    vb_save(curr_file, 'cond', '-append');

end
