function estimate_source_current_dipole(p, prefix_in)
% Estimate source currents from MEG/EEG data by dipole-fitting
%
% Copyright (C) 2011, ATR All Rights Reserved.
% License : New BSD License(see VBMEG_LICENSE.txt)

disp(mfilename);

% No additional prefix

% Set prefix of input file
if isempty(prefix_in)
    prefix_in_ = [];
else
    prefix_in_ = [prefix_in '_'];
end

% Set input file
data_file = fullfile(p.proj_root, p.dirname.modality, p.dirname.trial, [prefix_in_ p.task '.info.mat']);

% Load temporal peaks detected in find_temporal_peak.m
load(data_file, 'time_peak_sec', 'time_peak_win_sec')
if ~exist('time_peak_win_sec','var') || isempty(time_peak_win_sec)
    % If window of temporal peak is not specified, use time of interest
    time_peak_win_sec = p.time_of_interest_sec;
end
num_peak = size(time_peak_win_sec, 1);

% Load bexp and time information
[bexp, ~, time_info] = vb_load_meg_data(data_file);
time_sec = time_info.time;

% Trial-averaging
bexp_ave = mean(bexp, 3);

% Set input files 
brain_file   = fullfile(p.proj_root, p.dirname.brain, [p.struct_name, '.brain.mat']);
basis_file   = fullfile(p.proj_root, p.dirname.modality, p.dirname.leadfield, [p.task '.basis.mat']); % Use modified basis

% Load basis with patch normalization
patch_norm = ON;
G = vb_load_basis(basis_file, patch_norm, brain_file);
basis = G';

% Load brain
B = load(brain_file);
% Get index of cortices
cortexL_ix = B.Vinfo.cortexL;
cortexR_ix = B.Vinfo.cortexR;
cortexLR_ix = [cortexL_ix; cortexR_ix];

% Set index of the time window for baseline
twin_noise_sec = p.time_noise_sec; % Period for baseline [s]
[~, from_ix] = min(abs(time_sec-twin_noise_sec(1)));
[~, to_ix] = min(abs(time_sec-twin_noise_sec(2)));
twin_noise_ix = [from_ix to_ix];

% Compute noise covariance
noise_parm = struct(...
                'megfile_baseline', data_file,...
                'twin_noise',       twin_noise_ix,...
                'noise_reg',        0.01,...
                'noise_model',      3,...
                'temporal_filter',  false,...
                'trial_average',    false);

Cov = vb_observation_noise_specification(noise_parm);

% Set output directory
output_dir = fullfile(p.proj_root, p.dirname.modality, p.dirname.current);
if ~exist(output_dir, 'dir')
    mkdir(output_dir)
end

%% Estimation for each peak window
for peak=1:num_peak
    % Set signal time window
    time_win_sec = time_peak_win_sec(peak, :);
    % sec -> index
    [~, from_signal_ix] = min(abs(time_sec-time_win_sec(1)));
    [~, to_signal_ix]   = min(abs(time_sec-time_win_sec(2)));

    % Extract signal duration
    bexp_signal = bexp_ave(:,from_signal_ix:to_signal_ix);

    % --- Single dipole on the cortex
    Area = [];
    Area{1}.Iextract = cortexLR_ix; % Set whole cotex
    Area{1}.key      = 'Cortex';
    opt.Ntrain = 1;
    disp('Start single dipole fitting ...')
    [D, sig2, Dindex, IF, GoF] = fmri_dipole_estimation(basis, bexp_signal, Area, Cov{1}, opt);
    % Save results
    curr_file = fullfile(output_dir, [p.task '_dipole1_' num2str(peak) '.curr.mat']);
    save(curr_file, 'time_win_sec', 'D', 'Dindex', 'IF', 'sig2', 'Area', 'data_file', 'bexp', 'time_info', 'GoF')

    % --- Double dipole on the cortex (single dipole per hemisphere)
    Area = [];
    Area{1}.Iextract = cortexL_ix; 
    Area{1}.key      = 'CortexL';
    Area{2}.Iextract = cortexR_ix; 
    Area{2}.key      = 'CortexR';
    opt.Ntrain = 5;
    disp('Start double dipole fitting ...')
    [D, sig2, Dindex, IF, GoF] = fmri_dipole_estimation(basis, bexp_signal, Area, Cov{1}, opt);
    % Save results
    curr_file = fullfile(output_dir, [p.task '_dipole2_' num2str(peak) '.curr.mat']);
    save(curr_file, 'time_win_sec', 'D', 'Dindex', 'IF', 'sig2', 'Area', 'data_file', 'bexp', 'time_info', 'GoF')
end
