clear, close all

%% 2.2 Adding toolbox and dataset to the search path
% Adding VBMEG and SPM8 to search path (modify this for your environment)
path_of_VBMEG = 'path/to/vbmeg3';
path_of_SPM8 = 'path/to/spm8';
addpath(path_of_VBMEG);
vbmeg
addpath(path_of_SPM8);

% Adding all the directories in the current directory to search path
addpath(genpath(cd))

% Setting path of the dataset
dir_dataset = 'path/to/OSEdatset';

% Setting a path that results will be saved in
% Figures will be saved in $dir_result/figure
dir_result  = 'path/to/result';
if ~exist(dir_result, 'dir'), mkdir(dir_result); end

%% 2.3 Setting parameters for EEG/SQUID/OPM analysis
% Getting properties of dataset
d = define_dataset;

% Setting the subject, task, number of trials and runs
ss   = 3; % 1, 2, 3, or 4
sub  = d.sub_list{ss};
tt   = 3; % 1: Auditory, 2: Motor, or 3: Somatosensory
task = d.task_list{tt};
num_trial = 150; % Select 150 trials for all tasks and measurements
num_run_eeg   = d.num_run_table_eeg{sub, task};
num_run_squid = d.num_run_table_squid{sub, task};
num_run_opm   = d.num_run_table_opm{sub, task};

% Setting basic parameters for analysis
p_eeg   = set_parameters(dir_dataset, dir_result, 'eeg', sub, task, num_run_eeg);
p_squid = set_parameters(dir_dataset, dir_result, 'squid', sub, task, num_run_squid);
p_opm   = set_parameters(dir_dataset, dir_result, 'opm', sub, task, num_run_opm);

% Adding manually selected bad channels to parameters
p_opm   = set_badch(p_opm);

% Adding manually selected brain/noise ICs to parameters
p_squid = set_ic(p_squid);

%% 3 Brain modeling
% This is a part of Solving forward problem
% Resulted brain model is common across measurements and tasks
% Execute once per subject 

% Converting T1 DICOM files to NIfTI
import_T1(p_opm);
correct_bias_in_T1(p_opm);
segment_T1(p_opm);
% Brain modeling using Freesurfer
% This takes about one day
construct_cortical_surface_model(p_opm);
% Defining the current sources
construct_brain_model(p_opm);
% Checking the sensor position and brain model
check_brain_model(p_opm);

%% 4.1 Importing EEG and SQUID data
% Note: some of them are cross-referred in preprocessing
import_eeg(p_eeg)
import_squid(p_squid)

%% 4.2 Preparation for SQUID data
% Importing EOG data from EEG into SQUID-MEG
import_eog_squid(p_squid, p_squid.dirname.load, p_eeg.dirname.modality)

if strcmp(p_squid.task, 'Motor')
    % Correcting the definition of EMG-ch of Motor task
    correct_emgch_squid(p_squid, p_squid.dirname.load)
    % Detecting onset from EMG
    convert_emg_trigger(p_squid, p_squid.dirname.load)
else
    % Re-defining the trigger ch
    correct_trigch_squid(p_squid, p_squid.dirname.load)
end

% Modifying trigger signal
% In the case of Auditory task, trigger onsets will be shifted by 20 msec
if strcmp(p_squid.task, 'Auditory')
    modify_trigger(p_squid, p_squid.dirname.load, 20);
else
    modify_trigger(p_squid, p_squid.dirname.load);
end

%% 4.3 SQUID preprocessing
% ---- Start signal processing ----
% The order of process is easily changable
% ----------------------------
PREP_squid = {p_squid.dirname.load}; % Preprocessed directories will be stored

% tPCA using reference data
PREP_squid{end+1} = apply_tpca_meg(p_squid, PREP_squid{end}, p_squid.dirname.tpca);

% Prefiltering
PREP_squid{end+1} = apply_filtering(p_squid, PREP_squid{end}, p_squid.dirname.prefilter, p_squid.prefilter);

% ICA
PREP_squid{end+1} = apply_ica(p_squid, PREP_squid{end}, p_squid.dirname.ica);

% Filtering
PREP_squid{end+1} = apply_filtering(p_squid, PREP_squid{end}, p_squid.dirname.filter, p_squid.filter);

% Visualizing preprocessed data
close all
for pp=1:length(PREP_squid)
    show_processed_data(p_squid, PREP_squid{pp}, 'NO_REF');
end
compare_processed_psd(p_squid, PREP_squid, 'NO_REF');

% ---- End of signal processing ----

%% 4.4 SQUID trial-level processing
% Set the exit point of signal processing pipeline
% Note: Trig file will be make in this dirctory
pipe_exit = PREP_squid{end};

% ---- Start trial-level processing ----
% The executed process will be appended to the file prefix
prefix_squid = []; % init prefix
% Trial-segmentation
make_trial(p_squid, pipe_exit); % trigger file will be saved in pipe_exit directory
% Bad ch. and trial rejection
prefix_squid = reject_chtr(p_squid, prefix_squid); % prefix:r
% Baseline correction
prefix_squid = correct_baseline(p_squid, prefix_squid); % prefix:b
% Consolidating runs into single file (info.mat)
prefix_squid = merge_run(p_squid, prefix_squid);  % prefix:m
% Selecting 150 trials randomly
prefix_squid = select_trials(p_squid, prefix_squid, num_trial); % prefix:s
% ---- End of trial-level processing ----

% Finding the temporal peaks
file_peak_squid = find_temporal_peak(p_squid, prefix_squid); % Detected peak of SQUID is used in EEG
% Visualizing trial-averaged sensor signals
show_trial_average(p_squid, prefix_squid); 

%% 5.1 Preparation for EEG data
if (strcmp(p_eeg.task, 'Motor') || strcmp(p_eeg.task, 'Rest')) && strcmp(p_eeg.sub, '093')
    % Importing EOG data from SQUID-MEG into EEG
    import_eog_eeg(p_eeg, p_eeg.dirname.load, p_squid.dirname.modality)
end

if strcmp(p_eeg.task, 'Motor') || (strcmp(p_eeg.task, 'Somatosensory') && strcmp(p_eeg.sub, '093'))
    % Importing trigger from SQUID-MEG into EEG
    import_trigger(p_eeg, p_eeg.dirname.load, p_squid.dirname.modality)
else
    % Re-defining the trigger ch
    correct_trigch_eeg(p_eeg, p_eeg.dirname.load)
end

% Modifying trigger signal
% In the case of Auditory task, trigger onsets will be shifted by 20 msec
if strcmp(p_eeg.task, 'Auditory')
    modify_trigger(p_eeg, p_eeg.dirname.load, 20);
else
    modify_trigger(p_eeg, p_eeg.dirname.load);
end

%% 5.2 EEG preprocessing
% ---- Start signal processing ----
% The order of process is easily changable
% ----------------------------
PREP_eeg = {p_eeg.dirname.load}; % Preprocessed directories will be stored

% Regressing out EOGs
PREP_eeg{end+1} = apply_removing_eog(p_eeg, PREP_eeg{end}, p_eeg.dirname.eog);

% Prefiltering
PREP_eeg{end+1} = apply_filtering(p_eeg, PREP_eeg{end}, p_eeg.dirname.prefilter, p_eeg.prefilter);

% Clean channel
PREP_eeg{end+1} = apply_clean_channels_eeg(p_eeg, PREP_eeg{end}, p_eeg.dirname.clean, false);

% Filtering
PREP_eeg{end+1} = apply_filtering(p_eeg, PREP_eeg{end}, p_eeg.dirname.filter, p_eeg.filter);

% Visualizing preprocessed data
close all
for pp=1:length(PREP_eeg)
    show_processed_data(p_eeg, PREP_eeg{pp}, 'NO_REF');
end
compare_processed_psd(p_eeg, PREP_eeg, 'NO_REF');

% ---- End of signal processing ----
% Set the exit point of signal processing pipeline
% Note: Trig file will be make in this dirctory
pipe_exit = PREP_eeg{end};

%% 5.3 EEG trial-level processing
% ---- Start trial-level processing ----
% The executed process will be appended to the file prefix
prefix_eeg = []; % init prefix
% Trial-segmentation
make_trial(p_eeg, pipe_exit); % trigger file will be saved in pipe_exit directory
% Bad ch. and trial rejection
prefix_eeg = reject_chtr(p_eeg, prefix_eeg); % prefix:r
% Baseline correction
prefix_eeg = correct_baseline(p_eeg, prefix_eeg); % prefix:b
% Taking common average reference
prefix_eeg = common_average_eeg(p_eeg, prefix_eeg); % prefix:c
% Consolidating runs into a single file (info.mat)
prefix_eeg = merge_run(p_eeg, prefix_eeg); % prefix:m 
% Selecting 150 trials randomly
prefix_eeg = select_trials(p_eeg, prefix_eeg, num_trial); % prefix:s
% ---- End of trial-level processing ----

% Setting the same temporal peaks as SQUID
find_temporal_peak(p_eeg, prefix_eeg, file_peak_squid);
% Visualizing trial-averaged sensor signals
show_trial_average(p_eeg, prefix_eeg);

%% 6.1 Preparation for OPM data
% Importing data
import_opm(p_opm)

% Importing EOG data (saved in eeg.mat)
import_eog_opm(p_opm, p_opm.dirname.load)

% Modifying trigger signal
% In the case of Auditory task, trigger onsets will be shifted by 60 msec
if strcmp(p_opm.task, 'Auditory')
    modify_trigger(p_opm, p_opm.dirname.load, 60);
else
    modify_trigger(p_opm, p_opm.dirname.load);
end

% Channel rejection (overwrite input file)
import_bad_ch(p_opm, p_opm.dirname.load);

%% 6.2 OPM preprocessing
% ---- Start signal processing ----
% The order of process is easily changable
% ----------------------------
PREP_opm = {p_opm.dirname.load}; % Preprocessed directories will be stored
PREP_opm_lf = []; % Directories requires updating lead field

% Detrending
PREP_opm{end+1} = apply_detrending(p_opm, PREP_opm{end}, p_opm.dirname.detrend);

% Regressing out EOGs
PREP_opm{end+1} = apply_removing_eog(p_opm, PREP_opm{end}, p_opm.dirname.eog);

% Prefiltering
PREP_opm{end+1} = apply_filtering(p_opm, PREP_opm{end}, p_opm.dirname.prefilter, p_opm.prefilter);

% SSP 
% (SSP requires updating lead field)
[PREP_opm{end+1}, PREP_opm_lf{end+1}] = deal(apply_ssp_meg(p_opm, PREP_opm{end}, p_opm.dirname.ssp, p_opm.ssp));

% Filtering
PREP_opm{end+1} = apply_filtering(p_opm, PREP_opm{end}, p_opm.dirname.filter);

% Visualizing preprocessed data
close all
for pp=1:length(PREP_opm)
    show_processed_data(p_opm, PREP_opm{pp}, 'NO_REF');
    show_processed_data_row(p_opm, PREP_opm{pp}, 'NO_REF');
end
compare_processed_psd(p_opm, PREP_opm, 'NO_REF');

% ---- End of signal processing ----
% Set the exit point of signal processing pipeline
% Note: Trig file will be make in this dirctory
pipe_exit = PREP_opm{end};

%% 6.3 OPM trial-level processing
% ---- Start trial-level processing ----
% The executed process will be appended to the file prefix
prefix_opm = []; % init prefix
% Trial-segmentation
make_trial(p_opm, pipe_exit); % trigger file will be saved in pipe_exit directory
% Bad ch. and trial rejection
prefix_opm = reject_chtr(p_opm, prefix_opm, pipe_exit); % prefix:r
% Baseline correction
prefix_opm = correct_baseline(p_opm, prefix_opm); % prefix:b
% Consolidate runs into a single file (info.mat)
prefix_opm = merge_run(p_opm, prefix_opm); % prefix:m
% Selecting 150 trials randomly
prefix_opm = select_trials(p_opm, prefix_opm, num_trial); % prefix:s
% ---- End of trial-level processing ----

% Find the temporal peaks
find_temporal_peak(p_opm, prefix_opm);
% Visualizing trial-averaged sensor signals
show_trial_average(p_opm, prefix_opm);

%% 7. Solving forward problem
% Computing lead field matrix for each measurement
prepare_leadfield(p_eeg, prefix_eeg);
prepare_leadfield(p_squid, prefix_squid);
prepare_leadfield(p_opm, prefix_opm);

% Checking the sensor position (pick) and leadfield norm
check_pick(p_eeg);   
check_pick(p_squid); 
check_pick(p_opm);   
check_lfnorm(p_eeg, prefix_eeg);
check_lfnorm(p_squid, prefix_squid);
check_lfnorm(p_opm, prefix_opm);

% Updating lead field due to projection
for ll=1:length(PREP_opm_lf)
    update_leadfield(p_opm, prefix_opm, PREP_opm_lf{ll}, 'average');
end

%% 8. Solving inverse problem for the auditory and somatosensory task
if strcmp(p_opm.task, 'Auditory') || strcmp(p_opm.task, 'Somatosensory')
    % ---- Source imaging by dipole-fitting ----
    % EEG
    close all
    estimate_source_current_dipole(p_eeg, prefix_eeg);
    show_source_current_dipole(p_eeg);

    % SQUID
    close all
    estimate_source_current_dipole(p_squid, prefix_squid);
    show_source_current_dipole(p_squid);

    % OPM
    close all
    estimate_source_current_dipole(p_opm, prefix_opm);
    show_source_current_dipole(p_opm);
end

%% 9. Time-frequency analysis for the motor task
if strcmp(p_opm.task, 'Motor')
    % Making time-frequency representation of data
    make_tfr(p_squid, PREP_squid{end}, prefix_squid);
    make_tfr(p_opm, PREP_opm{end}, prefix_opm);
    make_tfr(p_eeg, PREP_eeg{end}, prefix_eeg);

    % Checking time-frequency representation
    % For SQUID, channels are read separately to reduce memory consumption
    show_tfr(p_squid, prefix_squid, [1 50]);
    show_tfr(p_squid, prefix_squid, [51 100]);
    show_tfr(p_squid, prefix_squid, [101 150]);
    show_tfr(p_squid, prefix_squid, [151 200]);
    show_tfr(p_squid, prefix_squid, [201 250]);
    show_tfr(p_squid, prefix_squid, [300 350]);
    show_tfr(p_squid, prefix_squid, [351 400]);
    show_tfr(p_opm, prefix_opm);
    show_tfr(p_eeg, prefix_eeg);

    % Setting time of interests [sec]
    toi_sec = [0.1 0.4];

    % Setting frequency of interests [Hz] for each subject
    switch sub
        case '002'
            foi_hz = [15 18];
        case '005'
            foi_hz = [21 24];
        case '006'
            foi_hz = [11 14];
        case '093'
            foi_hz = [11 14];
    end

    % ---- Source imaging using DICS beamformer ----
    close all
    estimate_source_current_freq(p_squid, prefix_squid, toi_sec, foi_hz);
    estimate_source_current_freq(p_opm, prefix_opm, toi_sec, foi_hz);
    estimate_source_current_freq(p_eeg, prefix_eeg, toi_sec, foi_hz);
    show_source_current_freq(p_squid);
    show_source_current_freq(p_opm);
    show_source_current_freq(p_eeg);
end
