function output_dirname = apply_clean_channels_eeg(p, input_dirname, output_dirname, interp_badch, unify_run, do_plot)
% Apply automatic bad channel detection and cleaning using clean_data plugin of EEGlab
% If interp_badch is true, bad channels will be interpolated by a spherical method.
% Otherwise, they will be rejected.
%
% Copyright (C) 2011, ATR All Rights Reserved.
% License : New BSD License(see VBMEG_LICENSE.txt)

disp(mfilename);

% Set input directory
input_dir = fullfile(p.proj_root, p.dirname.modality, input_dirname, p.task);

% Set output directory
output_dir = fullfile(p.proj_root, p.dirname.modality, output_dirname, p.task);
if ~exist(output_dir, 'dir')
    mkdir(output_dir)
end

% If true, bad channels will be interpolated by a spherical method
% Otherwise, they will be rejected
% Default: true
if ~exist('interp_badch', 'var') || isempty(interp_badch)
    interp_badch = true;
end

% If true, active channels will be consistent across runs
% Default: true
if ~exist('unify_run', 'var') || isempty(unify_run)
    unify_run = true;
end

% If true, visualize the results
% Default: true
if ~exist('do_plot', 'var') || isempty(do_plot)
    do_plot = true;
    set_fig_property(4, 4, 10, 10);
end

% Set modality-specific parameters
switch p.dirname.modality
    case 'eeg'
        chtype = 'EEG';
        extention = '.eeg.mat';
    otherwise
        warning('This function is for EEG')
end

% Set parameters for clean_channels
chancorr_crit = 0.8;
line_crit = 4;
channel_crit_maxbad_time = 0.4;
num_samples = 1000;

id_badch_run = {};
h = [];
%% Bad channel detection
for run = 1:p.num_run
    % Set input files
    file_name = [sprintf('run%02d', run), extention];
    input_file = fullfile(input_dir, file_name);

    % Load data
    loadspec = [];
    loadspec.ChannelType = chtype;
    [bexp, ch_info, time_info] = vb_load_meg_data(input_file, loadspec);
    freq = time_info.sample_frequency;
    [pick, ~] = vb_load_sensor(input_file, chtype);

    % Make data structure for EEGlab function
    EEG = [];
    EEG.data = bexp;
    EEG.srate = freq;
    EEG.nbchan = size(bexp,1);
    EEG.pnts   = size(bexp,2);
    EEG.trials = size(bexp,3);
    EEG.chanlocs = cell2struct(num2cell(pick),{'X','Y','Z'},2);
    EEG.etc = [];

    % Apply clean_channels to find bad channels
    [EEG, ix_badch] = clean_channels(EEG,chancorr_crit,line_crit,[],channel_crit_maxbad_time, num_samples);

    % Get ID of bad channels
    id_badch = ch_info.ID(ix_badch);

    % Store ID of badch of this run
    id_badch_run{run} = id_badch;

    % Plot before cleaning (rejection or interpolation)
    if do_plot
        h(run) = figure;
        tiledlayout(2,4, 'TileSpacing','loose', 'Padding','compact');
        inner_plot(bexp, pick, ch_info.Name, ix_badch, 'Before')
    end
end

%% Make bad channels consistent across runs
if unify_run
    disp('Unify bad channels across runs')
    % Find common badch across runs
    id_badch_common = [];
    for run = 1:p.num_run
        id_badch_common = union(id_badch_common, id_badch_run{run});
    end
    % Overwrite id_badch_run by common one
    id_badch_run = repmat({id_badch_common},1,length(id_badch_run));
end

%% Apply detected bad channels to data
for run = 1:p.num_run
    % Set input and output files
    file_name = [sprintf('run%02d', run), extention];
    input_file = fullfile(input_dir, file_name);
    output_file = fullfile(output_dir, file_name);

    % Prepare output file by copying input
    copyfile(input_file, output_file)

    % Load input data
    loadspec = [];
    loadspec.ChannelType = chtype;
    [bexp, ch_info, time_info] = vb_load_meg_data(input_file, loadspec);
    freq = time_info.sample_frequency;
    [pick, ~] = vb_load_sensor(input_file, chtype);

    % Set bad channels
    id_badch = id_badch_run{run};

    % Convert ID to index
    ix_badch = cellfun(@(x) find(ch_info.ID==x), num2cell(id_badch));

    % Calc min of max of powermap before cleaning for visualziation
    yp = mean(bexp.^2, 2);
    mi = min(yp(:));
    ma = max(yp(:));

    % How to clean detected bad channels?
    % Interpolation or Rejection
    if interp_badch
        % Make data structure for EEGlab function
        EEG = [];
        EEG.data = bexp;
        EEG.srate = freq;
        EEG.nbchan = size(bexp,1);
        EEG.pnts   = size(bexp,2);
        EEG.trials = size(bexp,3);
        EEG.chanlocs = cell2struct(num2cell(pick),{'X','Y','Z'},2);
        EEG.etc = [];
        % Interpolate bad channels
        bexp_interp = interp_eeg_spherical(EEG, find(ix_badch));
        bexp(ix_badch,:) = bexp_interp;

        % Re-referencing by taking common average
        % fparm.common_flg = 1;
        % fparm.freq = freq;
        % for tr = 1:size(bexp, 3)
        %     bexp(:,:,tr) = vb_filter_raw_data(bexp(:,:,tr), fparm);
        % end

        % Overwrite bexp with updated data
        append_ch = [];
        append_ch.data = bexp;
        append_ch.name = ch_info.Name;
        append_ch.type = ch_info.Type;
        append_data_ch(output_file, append_ch); % It makes output_file
        fig_suffix = 'interpolation';
    else
        % Load EEGinfo
        B = load(output_file, 'EEGinfo'); % To be overwritten
        % Reject bad channels
        B.EEGinfo.ActiveChannel(ix_badch) = 0;
        B.EEGinfo.ChannelInfo.Active(ix_badch) = 0;
        % Overwrite output file
        vb_fsave(output_file, '-struct', 'B', '-append');
        disp([output_file ' was overwritten with active channels.'])
        % For plot
        bexp(ix_badch,:) = [];
        pick(ix_badch,:) = [];
        ch_info.Name(ix_badch) = [];
        ix_badch = [];
        fig_suffix = 'rejection';
    end

    % Plot after cleaning (rejection or interpolation)
    if do_plot
        fig_file = fullfile(p.fig_root, mfilename, p.task, [p.sub '_run' num2str(run) '_' fig_suffix]);
        figure(h(run));
        inner_plot(bexp, pick, ch_info.Name, ix_badch, ['After ' fig_suffix], mi, ma)
        vb_savefig_as_shown(h(run), fig_file)
        disp([fig_file ' was saved.'])
        close(h(run))
    end
end

end

function inner_plot(y, pos, chname, ix_badch, plot_title, mi, ma)
% Section of time-series
ix1 = 5000;
ix2 = 10000;

% Flatten epoched data
y = reshape(y, size(y,1),size(y,2)*size(y,3));

% Calculate power
yp = mean(y.^2, 2);

% Calc minimum and maximum values of power
if ~exist('mi','var')||isempty(mi)
    mi = min(yp(:));
end
if ~exist('ma','var')||isempty(ma)
    ma = max(yp(:));
end
% Visualize sensor topo
%subplot(5,2,1)
nexttile([1,1])
vb_plot_sensor_2d(pos, yp, [mi ma], true, ix_badch);
axis square off
text(-1.4, -2, 'black: good', 'FontSize',7)
text(-1.4, -1.7, 'white: bad', 'FontSize',7)
title(plot_title)

% Visualize time-series
%subplot(5,2,[3 5 7 9])
nexttile([1,3])
plot_rows(y(:,ix1:ix2), chname, ix_badch)
end