function output_dirname = apply_filtering(p, input_dirname, output_dirname, parm_filt)
% Apply filtering
%
% Copyright (C) 2011, ATR All Rights Reserved.
% License : New BSD License(see VBMEG_LICENSE.txt)

disp(mfilename);

% Set input directory
input_dir = fullfile(p.proj_root, p.dirname.modality, input_dirname, p.task);

% Set output directory
output_dir = fullfile(p.proj_root, p.dirname.modality, output_dirname, p.task);
if ~exist(output_dir, 'dir')
    mkdir(output_dir)
end

% If parm_filt is not specified in input, p.filter is used
if ~exist('parm_filt','var') || isempty(parm_filt)
    parm_filt = p.filter;
end

% Set modality-specific parameters
switch p.dirname.modality
    case 'opm-meg'
        chtype = 'MEG';
        extention = '.meg.mat';
    case 'squid-meg'
        chtype = 'MEG';
        extention = '.meg.mat';
    case 'eeg'
        chtype = 'EEG';
        extention = '.eeg.mat';
end

% EOG signals will be filtered
chname_extra = {'EOG1', 'EOG2'};

% In the case of the Motor task, EMG signals will be filtered as well
if strcmp(p.task, 'Motor')
    % Set reference channel
    chname_extra{end+1} = 'trigger_emg';
end

for run = 1:p.num_run
    % Set input and output files
    file_name = [sprintf('run%02d', run), extention];
    input_file = fullfile(input_dir, file_name);
    output_file = fullfile(output_dir, file_name);

    % Load data (includes inactive ch because of down-sampling)
    loadspec = [];
    loadspec.ChannelType = chtype;
    loadspec.ActiveChannel = false; % Load all channels including inactive channels
    loadspec.ActiveTrial   = false; % Load all trial
    [bexp, ch_info, time_info] = vb_load_meg_data(input_file, loadspec);
    freq = time_info.sample_frequency;
    
    if strcmp(chtype, 'MEG')
        % Load reference
        loadspec = [];
        loadspec.ActiveChannel = false; % Load all channels including inactive channels
        loadspec.ActiveTrial   = false; % Load all trial
        loadspec.ChannelType = 'REFERENCE';
        [refmg, ~, ~] = vb_load_meg_data(input_file,loadspec);
    else
        % EEG does not have ref
        refmg = [];
    end

    % Load extra (EOG and trigger)
    loadspec = [];
    loadspec.ActiveChannel = false; % Load all channels including inactive channels
    loadspec.ActiveTrial   = false; % Load all trial
    loadspec.ChannelType = 'EXTRA';
    [bexp_ext, ch_info_ext, ~] = vb_load_meg_data(input_file,loadspec);
    
    % Apply Filtering on data (only active channels are filtered)
    parm_filt.chanNum = find(ch_info.Active); % Specify active channels
    bexp = filter2d(bexp, freq, parm_filt, 0);
    
    % Filter specified extra-channel data using same filtering parameters
    parm_filt_ex = parm_filt;
    ix_extra = [];
    if ~isempty(bexp_ext) && ~isempty(chname_extra)
        for ex = 1:length(chname_extra)
            ix_ex = find(strcmp(ch_info_ext.Name, chname_extra{ex}));
            if ~isempty(ix_ex)
                ix_extra = [ix_extra, ix_ex];
            else
                disp(['Extra channel ''' chname_extra{ex} ''' does not exist.'])
            end
        end
        parm_filt_ex.chanNum = ix_extra;
        bexp_ext = filter2d(bexp_ext, freq, parm_filt_ex, 0);
    end

    % Down sampling
    if isfield(parm_filt, 'fsamp') && freq ~= parm_filt.fsamp
        bexp     = vb_convert_freq(bexp, freq, parm_filt.fsamp);
        bexp_ext = vb_convert_freq(bexp_ext, freq, parm_filt.fsamp);
        refmg    = vb_convert_freq(refmg, freq, parm_filt.fsamp);
        new_freq = parm_filt.fsamp;
    else
        new_freq = freq;
    end
    
    % Overwrite B with updated data
    B = load(input_file); % To be overwritten
    switch chtype
        case 'MEG'
            B.bexp = bexp;
            B.bexp_ext  = bexp_ext;
            B.refmg     = refmg;
            B.MEGinfo.SampleFreq = new_freq;
            B.MEGinfo.Nsample = size(bexp, 2);
        case 'EEG'
            new_eeg_data = zeros(size(B.eeg_data,1), size(bexp,2));
            new_eeg_data(ch_info.ID,:) = bexp;
            new_eeg_data(ch_info_ext.ID,:) = bexp_ext;
            B.eeg_data = new_eeg_data;
            B.EEGinfo.SampleFrequency = new_freq;
            B.EEGinfo.Nsample = size(bexp, 2);
    end

    % Save filtered data
    vb_fsave(output_file, '-struct', 'B');
    disp([output_file ' was saved.'])
end



