function output_dirname = apply_ica(p, input_dirname, output_dirname, use_result)
% Apply independent component analysis.
% Inut data must not be trial-segmented.

% Copyright (C) 2011, ATR All Rights Reserved.
% License : New BSD License(see VBMEG_LICENSE.txt)

disp(mfilename);

% Set input directory
input_dir = fullfile(p.proj_root, p.dirname.modality, input_dirname, p.task);

% Set output directory
output_dir = fullfile(p.proj_root, p.dirname.modality, output_dirname, p.task);
if ~exist(output_dir, 'dir')
    mkdir(output_dir)
end

% Set result file
ica_result = fullfile(output_dir, 'results.mat');

if exist(ica_result, 'file')
    exist_result = true;
else
    exist_result = false;
end

% If use_result and exist_result are true, then load pre-computed result
% instead of computing runica.
if ~exist('use_result', 'var') || isempty(use_result)
    use_result = false;
end

% Set modality-specific parameters
switch p.dirname.modality
    case 'opm-meg'
        extention = '.meg.mat';
        % For ICA
        opt_method = 'picard';
        opt_pca = 0; % Automatically determined based on rank
        % For visualization
        ncol = 4;
        nrow = 5;
        set_fig_property(8, 6, 10, 10);
        Nplot = 25; % Plot max Nplot ICs
        plot_chtype = [6, 5]; % y and z
        plot_label = {'z', 'y'}; % label of saved fig
    case 'squid-meg'
        extention = '.meg.mat';
        % For ICA
        opt_method = 'picard';
        opt_pca = 100;
        % For visualization
        ncol = 4;
        nrow = 5;
        set_fig_property(8, 6, 10, 10);
        Nplot = 100; % Plot max Nplot ICs
        plot_chtype = [2, 3]; % axial or planar
        plot_label = {'axial', 'planar'}; % label of saved fig
    case 'eeg'
        extention = '.eeg.mat';
        % For ICA
        opt_method = 'picard';
        opt_pca = 0; % Automatically determined based on rank
        % For visualization
        ncol = 4;
        nrow = 4;
        set_fig_property(6, 4, 10, 10);
        Nplot = 63; % Plot max Nplot ICs
        plot_chtype = 1; % eeg channel
        plot_label = {'eeg'}; % label of saved fig
end

%% Take 'and' operation to ActiveChannel across run before concat run
temp_name = [sprintf('run%02d', 1), extention];
temp_file = fullfile(input_dir, temp_name);
temp_info = vb_load_measurement_info(temp_file, 'MEGINFO');

Nchannel = temp_info.Nchannel;
commonActiveChannel = ones(Nchannel, 1);
for run = 1:p.num_run
    % Load active information of MEG/EEG data
    file_name = [sprintf('run%02d', run), extention];
    input_file = fullfile(input_dir, file_name);
    info = vb_load_measurement_info(input_file, 'MEGINFO');
    % This is 'and' operation for ActiveChannel of all run files
    commonActiveChannel(~info.ActiveChannel) = 0;
end

% Get name of common active ch
commonActiveChannelName = temp_info.ChannelInfo.Name(logical(commonActiveChannel));

% Prepare loadspec to load common active ch
loadspec = [];
loadspec.ChannelName = commonActiveChannelName;
loadspec.ActiveTrial = true; % Load active trial

% Prepare loadspec to load trigger ch
% Trigger will be used for visualization
loadspec_trg = [];
loadspec_trg.ChannelName = {'trigger_modified'};
loadspec_trg.ActiveTrial = true; 
loadspec_trg.ChannelType = 'EXTRA';

% Set loadspec to load EOG ch
% EOGs will be used for visualization
loadspec_eog = [];
loadspec_eog.ChannelName = {'EOG1', 'EOG2'};
loadspec_eog.ActiveTrial = true; 
loadspec_eog.ChannelType = 'EXTRA';

%% Concatinate all run
data_long = [];
trigger_long = [];
eog_long = [];
for run = 1:p.num_run
    % Load MEG/EEG data
    file_name = [sprintf('run%02d', run), extention];
    input_file = fullfile(input_dir, file_name);
    [data,~,time_info] = vb_load_meg_data(input_file, loadspec);
    % Load trigger
    [trigger] = vb_load_meg_data(input_file, loadspec_trg);
    % Load EOGs
    [eog] = vb_load_meg_data(input_file, loadspec_eog);

    % Remove saturated signal in the case of OPM
    if strcmp(p.dirname.modality, 'opm-meg')
        % Set raw data, which is required for checking saturation
        raw_file = fullfile(p.proj_root, p.dirname.modality, p.dirname.load, p.task, file_name);
        [raw_data,~,raw_time_info] = vb_load_meg_data(raw_file, loadspec);
        % Check if even one channel is saturated
        flag_sat = any( or(raw_data>p.dr_ub, raw_data<p.dr_lb) );
        % Down sampling (matching the time index)
        if time_info.sample_frequency~=raw_time_info.sample_frequency
            % index_ds = dsearchn(raw_time_info.time', time_info.time'); % takes minutes...
            % flag_sat = flag_sat(index_ds);
            flag_sat = vb_convert_freq(flag_sat, raw_time_info.sample_frequency, time_info.sample_frequency);
        end

        % Remove saturated time points
        data(:,flag_sat) = []; 
        trigger(:,flag_sat) = [];
        eog(:,flag_sat) = [];
    end

    % Demean
    % data = bsxfun(@minus, data, mean(data,2));

    % Normalize data before concatinating across run
    data = normalize_data(data, 'standardize');
    eog  = normalize_data(eog, 'ch_standardize');

    % Concat data
    data_long = [data_long data];
    trigger_long = [trigger_long trigger];
    eog_long = [eog_long eog];
end

%% Apply ICA using all run
if use_result && exist_result
    disp('Pre-computed ICA result is used.')
    disp('Skip runica()')
    load(ica_result, 'w', 's')
else
    rng(0,'twister') % Fix the random seed

     % Check rank of the input data
     data_rank = getrank(data_long);

     % Configure opt_pca in the case of rank defficient data
     if data_rank < size(data_long,1)
         disp(['Data rank(' num2str(data_rank) ') is lower than #ch(' num2str(size(data_long,1)) ') due to preprocess.'])
         disp('Conduct ICA with dimension reduction by PCA.')
         if ~opt_pca || opt_pca<=0
             opt_pca = data_rank;
         else
             temp_pca = min(opt_pca, data_rank);
             disp(['Specified opt_pca: ' num2str(opt_pca)])
             disp(['Practical opt_pca: ' num2str(temp_pca)])
             opt_pca = temp_pca;
         end
     end

     % Init
     [w, s, mix, unmix] = deal([]);

     switch opt_method
         case 'infomax'
             if ~opt_pca || opt_pca<=0
                 % Without PCA
                 [w, s] = runica(data_long, 'reset_randomseed','off', 'stop',1e-7, 'maxsteps',2000);
             else
                 % With PCA
                 [w, s] = runica(data_long, 'reset_randomseed','off', 'stop',1e-7, 'maxsteps',2000, 'pca',opt_pca);
             end
         case 'sobi'
             % sobi.m does not accept PCA parameter inside the function.
             % If you process rank-deficient data using sobi,
             % consider runpca before conducting sobi
             % and update w after that (see l.555 in pop_runica.m).
             mix = sobi(data_long, opt_pca);
         case 'picard'
             mode_picard = 'ortho'; % same as fastICA (extended is automatically used)
             if ~opt_pca || opt_pca<=0
                 % Without PCA
                 % Here w = unmix (unmix will be set as w)
                 [~, w] = picard(data_long, 'whiten',true, 'mode',mode_picard, 'verbose',true);
             else
                 % With PCA
                 [~, w] = picard(data_long, 'whiten',true, 'pca',opt_pca, 'mode',mode_picard, 'verbose',true);
             end
         otherwise
             error([opt_method ' is not supported.'])
     end
end

% data = mix x IC
% IC = unmix x data

if isempty(w)
    w = pinv(mix);
end
if isempty(s)
    s = eye(size(w,2)); 
end
if isempty(unmix)
    unmix = w*s; 
end
if isempty(mix)
    % mix = inv(unmix);
    mix = pinv(unmix); % unmix is not square due to PCA
end

% Save result
vb_save(ica_result, 'w', 's', 'unmix', 'mix');
disp([ica_result ' was saved.'])

%% Classify ICs into brain activites or noises
% Load positions of active channels
[pos, ~] = vb_load_channel(input_file, false); 
pos = pos(logical(commonActiveChannel),:);

% Calculate ICs using all run
ic_long = unmix * data_long;  % [component x sample]

% Prepare parameters
Nch = size(data_long,1);
Ncomp = size(w,1);

% Classification using selected methods
classify_method = p.ica.classify_method;

switch classify_method
    case 'heuristic'
        feature_list = p.ica.heuristic_feature_list;
        Nfeat = length(feature_list);
        threshold_outlier = p.ica.heuristic_threshold_outlier; % Threshold for normalized features
        noise_ic_each_feature = zeros(Ncomp, Nfeat);
        features = zeros(Ncomp, Nfeat); % Normalized values will be saved in

        % Classify each IC based on the number of outliers
        for ff = 1:Nfeat
            value = zeros(Ncomp,1);
            for comp = 1:Ncomp
                eval(['value(comp)=' feature_list{ff} '(ic_long(comp,:));']);
            end
            mean_val = mean(value(:));% Mean value across ICs
            std_val = std(value(:));% Standard deviation across ICs

            normalized_value = (value-repmat(mean_val, [Ncomp,1]))./std_val;
            noise_ic_each_feature(:,ff) = abs(normalized_value) > threshold_outlier;
            features(:,ff) = normalized_value;
        end

        % Determine brain and noise ICs based on each feature
        noise_ic = find( all(noise_ic_each_feature,2) ); % AND
        % noise_ic = find( any(noise_ic_each_feature,2) ); % OR
        brain_ic = 1:Ncomp;
        brain_ic(noise_ic) = [];

        % Set title of time-series (feature name and value)
        label_class = cell(Ncomp,1);
        for ii=1:Ncomp
            label_class{ii} = strjoin(strcat(feature_list', ':' , num2str(features(ii,:)','%.2f'))', ', ');
        end

        % Append classification results
         vb_save(ica_result, 'brain_ic', 'noise_ic', 'classify_method', 'feature_list', 'features', 'threshold_outlier', '-append');
            disp(['Classification results were appended to ' ica_result])

    case 'ICL'
        % Probability of noise ICs will be masked to eliminate ambiguous classification
        threshold_noise_prob = p.ica.ICL_threshold_noise_prob; % p<threshod_noise will be discarded

        % Extract features for ICL
        feature_list = my_ICL_feature_extractor(ic_long, w, s, pos, time_info.sample_frequency);

        % Run ICL
        version = 'default'; % Use default network
        labels = run_ICL(version, feature_list{:});
        classes = {'Brain', 'Muscle', 'Eye', 'Heart', ...
            'Line Noise', 'Channel Noise', 'Other'};
        ic_label = array2table(labels, 'VariableNames', classes);

        % Reject below classes as noise component
        % classes_noise = {'Muscle', 'Eye', 'Heart', 'Line Noise', 'Channel Noise', 'Other'};
        classes_noise = {'Muscle'};
        ix_noise = cellfun(@(x) find(strcmp(x, classes)), classes_noise, 'UniformOutput', false);
        ix_noise = cell2mat(ix_noise);
        ix_brain = setdiff([1:length(classes)], ix_noise);

        % Masking noise ICs by a threshold
        labels_b = labels(:,ix_brain); % Brain class
        labels_n = labels(:,ix_noise); % Noise classes
        labels_n_masked = labels_n.*(labels_n>threshold_noise_prob);
        labels2 = zeros(size(labels));
        labels2(:, ix_brain) = labels_b;
        labels2(:, ix_noise) = labels_n;
        labels2_masked = zeros(size(labels));
        labels2_masked(:, ix_brain) = labels_b;
        labels2_masked(:, ix_noise) = labels_n_masked;

        % Classfify based on masked probability
        [icl_prob_masked, icl_label_masked] = max(labels2_masked,[],2);
        [icl_prob, icl_label] = max(labels2,[],2); % For plot

        % Determine brain and noise ICs
        brain_ic = find(ismember(icl_label_masked, ix_brain));
        noise_ic = find(ismember(icl_label_masked, ix_noise));

        % Set title of time-series (blank)
        label_class = cell(Ncomp,1);
        for ii=1:Ncomp
            label_class{ii} = [classes{icl_label(ii)} ' : ' num2str(icl_prob(ii,:)','%.2f')];
        end

        % Append classification
        vb_save(ica_result, 'brain_ic', 'noise_ic', 'classify_method', 'ic_label', 'threshold_noise_prob', '-append');
        disp(['Classification results were appended to ' ica_result])

    case ''
        % No automatic selection is conducted
        brain_ic = 1:Ncomp; % All ICs are brain
        noise_ic = [];

        % Set title of time-series (blank)
        label_class = cell(Ncomp,1);
        for ii=1:Ncomp
            label_class{ii} = '';
        end

        % Append classification
        vb_save(ica_result, 'brain_ic', 'noise_ic', 'classify_method', '-append');
        disp(['Classification results were appended to ' ica_result])
end

% Find noise ICs related to EOG channels
if isfield(p.ica, 'eog_threshold') 
    eog_threshold = p.ica.eog_threshold;
    R2 = zeros(Ncomp, 1);
    for comp = 1:Ncomp
        [~, R2(comp)] = inner_calc_r2(ic_long(comp,:), eog_long, true);
    end
    noise_ic_eog = find(R2 > eog_threshold);

    % Overwrite brain and noise ICs with EOG ICs
    noise_ic = union(noise_ic, noise_ic_eog);
    brain_ic = 1:Ncomp;
    brain_ic(noise_ic) = [];

    % Overwrite the title of time-series
    for ii=1:length(noise_ic_eog)
        label_class{noise_ic_eog(ii)} = ['EOG (r2 = ' num2str(R2(noise_ic_eog(ii)), '%.02f') ')'];
    end
end

% Apply manual selection (specified in set_ic.m)
if p.ica.manual_selection
    brain_ic_manual = [];
    noise_ic_manual = [];
    if isfield(p.ica,'brain_ic') && ~isempty(p.ica.brain_ic)
        brain_ic_manual = p.ica.brain_ic; 
    end
    if isfield(p.ica,'noise_ic') && ~isempty(p.ica.noise_ic)
        noise_ic_manual = p.ica.noise_ic; 
    end

    % Append brain and noise ICs
    brain_ic = union(brain_ic, brain_ic_manual);
    noise_ic = setdiff(noise_ic, brain_ic);
    noise_ic = union(noise_ic, noise_ic_manual);
    brain_ic = setdiff(brain_ic, noise_ic);

    % Check contradiction
    all_ic = [brain_ic(:); noise_ic(:)];
    if ~isempty(setdiff(all_ic, 1:Ncomp))
        error(['Inconsistent definition of IC labels were detected.' newline 'Check "set_ic.m"'])
    end

    % Overwrite the title of time-series
    for ii=1:length(brain_ic_manual)
        label_class{brain_ic_manual(ii)} = 'manually selected brain IC';
    end
    for ii=1:length(noise_ic_manual)
        label_class{noise_ic_manual(ii)} = 'manually selected noise IC';
    end
end

% Display
Nn = length(noise_ic);
if Nn>0
    fprintf('Noise ICs: ');
    for ic1 = 1:Nn-1
        fprintf('%s, ', num2str(noise_ic(ic1)));
    end
    fprintf('%s\n', num2str(noise_ic(Nn)));
else
    disp('No noise IC was classified')
end

%% Make denoising filter
g = zeros(Ncomp,Ncomp);
for ii = 1:length(brain_ic)
    g(brain_ic(ii), brain_ic(ii)) = 1;
end
denoise = mix*g*unmix;

% Append denoising filter
vb_save(ica_result, 'denoise', 'g', '-append');
disp(['Denoising filter was appended to ' ica_result])

%% Remove noise ICs from each run 
for run = 1:p.num_run
    % Set files
    file_name = [sprintf('run%02d', run), extention];
    input_file = fullfile(input_dir, file_name);
    output_file = fullfile(output_dir, file_name);
        
    % Load MEG/EEG data (only active ch and trial)
    [bexp, ch_info] = vb_load_meg_data(input_file, loadspec);

    % Demean
    bexp = bsxfun(@minus, bexp, mean(bexp,2));

    % Apply denoising filter
    bexp = denoise*bexp;

    % Make output file and overwrite bexp with updated data
    append_ch = [];
    append_ch.data = bexp;
    append_ch.name = ch_info.Name;
    append_ch.type = ch_info.Type;
    append_data_ch(input_file, append_ch, output_file); % It makes output_file

    % Overwrite ActiveChannel of MEGinfo/EEGinfo by common active channels across runs
    switch p.dirname.modality
        case {'opm-meg', 'squid-meg'}
            temp = load(output_file, 'MEGinfo');
            temp.MEGinfo.ActiveChannel = commonActiveChannel;
            temp.MEGinfo.ChannelInfo.Active = commonActiveChannel;
        case 'eeg'
            temp = load(output_file, 'EEGinfo');
            temp.EEGinfo.ActiveChannel = commonActiveChannel;
            temp.EEGinfo.ChannelInfo.Active = commonActiveChannel;
    end
    save(output_file, '-struct', 'temp', '-append');

end

%% Visualization (segment of data)
% Calculate power spectrum density of ICs
% [pow, f] = vb_fftpower(ic_long, time_info.sample_frequency); % Calculate power spectrum
[freq, psdx] = calc_psd(ic_long, time_info.sample_frequency, @hanning, 4000, 'psds');

% Obtain classification
classification = -ones(1, Ncomp);
classification(brain_ic) = 1;% 1: brain_ic, -1: noise_ic

% Show Nsec of long IC
start_sec = 60;
show_sec = 20;
start_ix = start_sec*time_info.sample_frequency;
end_ix   = start_ix + show_sec*time_info.sample_frequency;
time_ix = start_ix:end_ix;
time_plot = start_sec:1/time_info.sample_frequency:start_sec+show_sec;
ic_plot = ic_long(:,time_ix);
trigger_plot = trigger_long(:,time_ix);
eog_plot = eog_long(:,time_ix);
eog_plot = eog_plot ./ max(abs(eog_plot(:)));

% Show and save ICs
h = figure;
ccount = 1;
while ccount <= min(Ncomp, Nplot) % Plot max Nplot ICs
    clf(h)
    set(0, 'CurrentFigure', h)
    rr = 1; % Row index of subplot
    for c = ccount:min([ccount+ncol-1, Ncomp])
        if classification(c) == 1
            color = [0 0.4470 0.7410];
        else
            color = [0.8500 0.3250 0.0980];
        end
        
        % Plot spatial pattern of the mixing vector
        for ctype=1:length(plot_chtype)
            subplot(ncol, nrow, ctype+nrow*(rr-1))
            % Extract specified chtype
            chix = ch_info.Type==plot_chtype(ctype);
            chlabel = plot_label{ctype};
            mix_plot = mix(chix,:);
            pos_plot = pos(chix,:);
            ma = max(abs(mix(:, c))); % Max of all channels
            vb_plot_sensor_2d(pos_plot, mix_plot(:, c), [-ma ma], true);
            axis square off
            colorbar, colormap(my_parula())
            if ctype==1
                title(['IC ' num2str(c) ' (' chlabel ')'])
            else
                title(['(' chlabel ')'])
            end
        end
        
        % Plot time-series of components
        subplot(ncol, nrow, ctype+1+nrow*(rr-1):ctype+2+nrow*(rr-1))
        hold on
        plot(time_plot, ic_plot(c, :), 'Color',color);
        % plot(time_plot, eog_plot'.*max(ic_plot(c,:))*0.99, 'Color',[0.2 0.2 0.2], 'LineWidth',0.4); % Plot EOGs time-series on ICs
        plot(time_plot, trigger_plot.*max(ic_plot(c,:))*0.5, 'Color',[0.3 0.3 0.3]); % Plot trigger time-series on ICs
        xlim([time_plot(1) time_plot(end)])
        grid on
        title(label_class{c}) % Plot criteria of classification
        if rr == ncol
            xlabel('Time [s]')
        end
        
        % Plot power spectrum
        subplot(ncol, nrow, ctype+3+nrow*(rr-1))
        hold on
        plot(freq, log(psdx(:,c)), 'Color',color);
        xlim([0.1 60])
        ylabel('Log(psd)')
        grid on
        if rr == 1
            title('Power spectrum')
        elseif rr == ncol
            xlabel('Frequency [Hz]')
        end
        
        % Increment count of row (subplot)
        rr = rr+1;
    end
    ax1 = axes('Position', [0 0 1 1], 'Visible', 'off');
    %text1 = {'Blue: Brain'; 'Red: Noise'; 'Green: Trigger'; 'Black: EOGs'};
    text1 = {'Blue: Brain'; 'Red: Noise'; 'Black: Trigger'};
    axes(ax1)
    text(0.91, 0.87, text1, 'fontsize', 10)
    
    % Save figure
    fig_file = fullfile(p.fig_root, mfilename, p.dirname.modality, p.task, p.sub, [num2str(ccount) '-' num2str(ccount+ncol-1)]);
    vb_savefig_as_shown(h, fig_file)
    disp([fig_file '.png was saved.'])

    % Increment count of components
    ccount = ccount+ncol;
end

%% Visualization (whole time-series)
start_ix = 1;
end_ix   = size(ic_long,2);
start_sec = 0;
show_sec = size(ic_long,2)/time_info.sample_frequency;
time_ix = start_ix:end_ix;
time_plot = start_sec:1/time_info.sample_frequency:start_sec+show_sec-1/time_info.sample_frequency;
ic_plot = ic_long(:,time_ix);
eog_plot = eog_long(:,time_ix);
eog_plot = eog_plot ./ max(abs(eog_plot(:)));

% Show and save ICs
h = figure;
ccount = 1;
while ccount <= min(Ncomp, Nplot) % Plot max Nplot ICs
    clf(h)
    set(0, 'CurrentFigure', h)
    rr = 1; % Row index of subplot
    for c = ccount:min([ccount+ncol-1, Ncomp])
        if classification(c) == 1
            color = [0 0.4470 0.7410];
        else
            color = [0.8500 0.3250 0.0980];
        end
        
        % Calc corr with EOGs
        reog1 = corr(ic_plot(c,:)', eog_plot(1,:)');
        reog2 = corr(ic_plot(c,:)', eog_plot(2,:)');

        % Scaling
        eog_plt = eog_plot'.*max(ic_plot(c,:))*0.8;

        % Plot time-series of components
        subplot(ncol, 1, rr)
        hold on
        plot(time_plot, ic_plot(c, :), 'Color',color);
        % Plot EOGs
        if sum(abs(eog_plt(:,1))) > sum(abs(eog_plt(:,2)))
            plot(time_plot, eog_plt(:,1)', 'Color',[0.9290 0.6940 0.1250], 'LineWidth',0.2); % Plot EOGs time-series on ICs
            plot(time_plot, eog_plt(:,2)', 'Color',[0.4660 0.6740 0.1880], 'LineWidth',0.2); % Plot EOGs time-series on ICs
        else
            plot(time_plot, eog_plt(:,2)', 'Color',[0.4660 0.6740 0.1880], 'LineWidth',0.2); % Plot EOGs time-series on ICs
            plot(time_plot, eog_plt(:,1)', 'Color',[0.9290 0.6940 0.1250], 'LineWidth',0.2); % Plot EOGs time-series on ICs
        end
        xlim([time_plot(1) time_plot(end)])
        grid on
        title(['IC ' num2str(c) ' (corr EOG1:' num2str(reog1,'%.02f') ' EOG2:' num2str(reog2,'%.02f') ')'])
        if rr == ncol
            xlabel('Time [s]')
        end
        
        % Increment count of row (subplot)
        rr = rr+1;
    end
    ax1 = axes('Position', [0 0 1 1], 'Visible', 'off');
    text1 = {'Blue: Brain'; 'Red: Noise'; 'Yellow: EOG1'; 'Green: EOG2'};
    axes(ax1)
    text(0.91, 0.87, text1, 'fontsize', 10)
    
    % Save figure
    fig_file = fullfile(p.fig_root, mfilename, p.dirname.modality, p.task, p.sub, ['whole_' num2str(ccount) '-' num2str(ccount+ncol-1)]);
    vb_savefig_as_shown(h, fig_file)
    disp([fig_file '.png was saved.'])

    % Increment count of components
    ccount = ccount+ncol;
end

end % End of function


% Inner function to calculate entropy
function h = entropy(x, n)
%ENTROPY Computes a first-order estimate of the entropy of a matrix.
%   H = ENTROPY(X, N) returns the first-order estimate of matrix X
%   with N symbols (N = 256 if omitted) in bits/symbol. The estimate
%   assumes a statistically independent source characterized by the
%   relative frequency of occurrence of the elements in X.

%   Copyright 2002-2004 R. C. Gonzalez, R. E. Woods, & S. L. Eddins
%   Digital Image Processing Using MATLAB, Prentice-Hall, 2004
%   $Revision: 1.4 $  $Date: 2003/10/26 18:35:35 $

if nargin < 2
    n = 256;                           % Default for n.
end

x = double(x);                        % Make input double
xh = hist(x(:), n);                   % Compute N-bin histogram
xh = xh / sum(xh(:));                 % Compute probabilities

% Make mask to eliminate 0's since log2(0) = -inf.
i = find(xh);

h = -sum(xh(i) .* log2(xh(i)));       % Compute entropy
end

% Inner function to calculate kurtosis
function k = kurtosis(x)
dm = x-mean(x);
n = length(x);
k = n*sum(dm.^4)/sum(dm.^2)^2;
end

% Inner function to calculate R2 and explained variance
function [r2, ev] = inner_calc_r2(data, refdata, add_deriv)

if ~exist('add_deriv','var') || isempty(add_deriv)
    add_deriv = false;
end

Nt = size(data,2);
Y = data';
varexp = refdata;

% Add 1st order temporal derivatives to varexp
if(add_deriv)
    dref = diff(refdata');
    dref = [dref(1,:); dref]';
    varexp = [varexp; dref];
end

% Construct design matrix
X = [ones(Nt,1) varexp']; % Design matrix

% Normalization each column
Xn = bsxfun(@rdivide, X, sqrt(sum(X.^2, 1)));

% Solve least-square
%beta = pinv(X'*X) * X' * Y;
beta = pinv(Xn) * Y;

% Compute residuals
Yhat = Xn * beta;
Yres = Y - Yhat; 

% Compute coefficient of determination
r2 = 1 - sum(Yres.^2)/sum((Y-mean(Y)).^2);

% Compute explained variance
ev = 1 - var(Yres)/var(Y);

end

function inner_calc_autocorr(ic, w, s, srate)
% Prepare ICA results
icawinv    = pinv(w*s);
icaweights = w;
icasphere  = s;

icaact = ic;
[ncomp, pnts, trials] = size(icaact);

icaact = double(icaact); % Cast to double

% check ica is real
assert(isreal(icaact), 'Your ICA decomposition must be real to use ICLabel')

% Make EEG structure to call EEGlab functions
EEG = [];
EEG.icaweights = icaweights;
EEG.icaact = icaact;
EEG.pnts = pnts;
EEG.trials = trials;
EEG.srate = srate;

% Calc autocorrelation using EEGlab function
if trials == 1
    if pnts / srate > 5
        autocorr = eeg_autocorr_welch(EEG);
    else
        autocorr = eeg_autocorr(EEG);
    end
else
    autocorr = eeg_autocorr_fftw(EEG);
end
end

function tmprank2 = getrank(tmpdata, pca_opt)
    
    tmprank = rank(tmpdata);
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %Here: alternate computation of the rank by Sven Hoffman
    %tmprank = rank(tmpdata(:,1:min(3000, size(tmpdata,2)))); old code
    covarianceMatrix = cov(tmpdata', 1);
    [~, D] = eig (covarianceMatrix);
    rankTolerance = 1e-7;
    tmprank2=sum (diag (D) > rankTolerance);
    if tmprank ~= tmprank2
        if nargin >= 2 && pca_opt ~= 0
            fprintf('Warning: fixing rank computation inconsistency (%d vs %d) most likely because running under Linux 64-bit Matlab, taking the minimum\n', tmprank, tmprank2);
        end
        tmprank2 = min(tmprank, tmprank2);
    end
end