function output_dirname = apply_removing_eog(p, input_dirname, output_dirname)
% Regress-out EOGs.
%
% Copyright (C) 2011, ATR All Rights Reserved.
% License : New BSD License(see VBMEG_LICENSE.txt)

disp(mfilename);

% Set input directory
input_dir = fullfile(p.proj_root, p.dirname.modality, input_dirname, p.task);

% Set output directory
output_dir = fullfile(p.proj_root, p.dirname.modality, output_dirname, p.task);
if ~exist(output_dir, 'dir')
    mkdir(output_dir)
end

% Set modality-specific parameters
switch p.dirname.modality
    case 'opm-meg'
        chtype = 'MEG';
        extention = '.meg.mat';
        plot_unit = 'T';
        plot_chtype = [5, 6]; % y and z
        plot_label = {'y', 'z'}; % label of saved fig
    case 'squid-meg'
        chtype = 'MEG';
        extention = '.meg.mat';
        plot_unit = 'T';
        plot_chtype = [2, 3]; % axial or planar
        plot_label = {'axial', 'planar'}; % label of saved fig
    case 'eeg'
        chtype = 'EEG';
        extention = '.eeg.mat';
        plot_unit = 'V';
        plot_chtype = 1; % eeg channel
        plot_label = {'eeg'}; % label of saved fig
end

remove_extra = true;
% Set extra channels will be removed
loadspec_ext = [];
loadspec_ext.ChannelType = 'EXTRA';
loadspec_ext.ChannelName = {'EOG1', 'EOG2'};

for run = 1:p.num_run
    file_name = [sprintf('run%02d', run), extention];
    input_file = fullfile(input_dir, file_name);
    output_file = fullfile(output_dir, file_name);

    % Load MEG data
    loadspec = [];
    loadspec.ActiveChannel = false; % Load all ch
    loadspec.ActiveTrial   = false; % Load all trial
    [bexp, ch_info, time_info] = vb_load_meg_data(input_file, loadspec);
    time = time_info.time;
    freq = time_info.sample_frequency;
    [nch, ntime, ntrial] = size(bexp);
    % If bexp is already epoched, make it flat
    if ntrial~=1
        bexp = reshape(bexp, [nch, ntime*ntrial]);
    end

    % Init
    y = bexp;
    X = [];
    Xlabel = [];

    % Demean
    y = bsxfun(@minus, y, mean(y,2));

    if remove_extra
        [bexp_ext, ~, ~] = vb_load_meg_data(input_file, loadspec_ext);
        % Demean
        bexp_ext = bsxfun(@minus, bexp_ext, mean(bexp_ext,2));
        % Normalize
        bexp_ext = bsxfun(@rdivide, bexp_ext, std(bexp_ext,[],2));
        % Append to the design matrix
        X = [X; bexp_ext];
        Xlabel = [Xlabel loadspec_ext.ChannelName];
    end

    if ~isempty(X)
        % Regress-out specified components
        y_denoised = remove_by_regression(y, X);
        % Replace bexp with denoised data
        bexp = y_denoised;
        doPlot = true;
    else
        doPlot = false;
    end

    if doPlot
        % Load positions of channels
        [pos, ~] = vb_load_channel(input_file, false);
        % Show results of denoising
        set_fig_property(4, 4, 10, 10);

        % Loop for ch type (in the case of OPM, each axis will be shown seperately)
        for cc = 1:length(plot_chtype)
            chtype = plot_chtype(cc);
            chlabel = plot_label{cc};
            h = inner_check_removing_eog(y, y_denoised, X, Xlabel, pos, ch_info, time_info, plot_unit, chtype);
            fig_file = fullfile(p.fig_root, mfilename, p.task, p.dirname.modality, [p.sub '_run' num2str(run) '_' chlabel]);
            vb_savefig_as_shown(h, fig_file)
            close(h)
        end
    end

    %% Save denoised data
    % If original bexp was epoched, re-shape it to original size
    if ntrial~=1
        bexp = reshape(bexp, [nch, ntime, ntrial]);
    end

    % Make output file and overwrite bexp with updated data
    append_ch = [];
    append_ch.data = bexp;
    append_ch.name = ch_info.Name;
    append_ch.type = ch_info.Type;
    append_data_ch(input_file, append_ch, output_file); % It makes output_file
end

end % End of function

function h = inner_check_removing_eog(y1, y2, eog, eog_label, pos, channel_info, time_info, unit_disp, channel_type)
% Check the result of removing EOG components

h = figure; hold on

% Get time in sec
time = time_info.time;

% Extract data with specific ch type
ix_ch = find(channel_info.Type == channel_type);
pos = pos(ix_ch, :);
y1 = y1(ix_ch, :);
y2 = y2(ix_ch, :);

% Difference
dy = y1 - y2;

% Calculate power
p1 = mean(y1.^2, 2);
p2 = mean(y2.^2, 2);
dp = p1-p2;

%% Plot time-series
subplot(4, 4, 1:3), plot(time, eog'), title('Normalized EOGs')
xlim([min(time) max(time)]), ylim([min(eog(:)) max(eog(:))])
xlabel('Time [sec]'), ylabel('A.U.'), legend(eog_label)

subplot(4, 4, 5:7), plot(time, y1'), title('Before removing EOGs')
xlim([min(time) max(time)]), ylim([min(y1(:)) max(y1(:))])
ylabel(unit_disp)

subplot(4, 4, 9:11), plot(time, y2'), title('After removing EOGs')
xlim([min(time) max(time)]), ylim([min(y2(:)) max(y2(:))])
ylabel(unit_disp)

subplot(4, 4, 13:15), plot(time, dy'), title('Differences')
xlim([min(time) max(time)]), ylim([min(dy(:)) max(dy(:))])
ylabel(unit_disp)

%% Plot sensor space
% Get minimum and maximum values of power
ma = max([p1(:); p2(:)]);
mi = min([p1(:); p2(:)]);

subplot(4, 4, 8)
vb_plot_sensor_2d(pos, p1, [mi ma], true);
axis square off
colorbar
title('Power (before)')
colormap(my_parula())

subplot(4, 4, 12)
vb_plot_sensor_2d(pos, p2, [mi ma], true);
axis square off
colorbar
title('Power (after)')
colormap(my_parula())

subplot(4, 4, 16)
vb_plot_sensor_2d(pos, dp, [mi ma], true);
axis square off
colorbar
title('Power (diff)')
colormap(my_parula())

end % End of function
