function output_dirname = apply_ssp_meg(p, input_dirname, output_dirname, parm_ssp, doPlot)
% Remove environmental noise using SSP-based denoising method.
% parm_ssp: Parameters of SSP
%   ssp_basis: Basis of the noise space are selected from below options.
%              Each option requires specific parameters. 
%      'HFC'   : Homogeneous Field Correction.
%                Results are equivalent to VSH with lmax=1.
%                (Parameters) 
%                   No additional parameters are required.
%      'VSH'   : Vector Spherical Harmonics function.
%                (Parameters) 
%                   lmax: lmax (order) of VSH function.
%      'EMPTY' : Noise space is computed using empty room data.
%                (Parameters) 
%                   num_comp: Number of components to be used.
%                   empty_file: MEG file of the empty room recording.
%      'EXTRA' : Use leadfield of the extra-dipoles as noise basis.
%                (Parameters)
%                   extra_basis_file: Leadfield file of extra-dipoles.
%                                     This can be made by make_extra_dipole.m.
%
%   use_vbcssp: If this is true, coefficients of basis and channels are computed using vbCSSP.
%               (Default: false)
%
%   remove_by_regression: If this is true, denoising is performed by regress-out.
%                         (Default: false)
%
% [References]
% Uusitalo and Ilmoniemi, Signal-space projection method for separating MEG 
% or EEG into components. Med. Biol. Eng. Comput. 35 (1997) 135-140.
% Tierney et al. Modelling optically pumped magnetometer inteference in MEG
% as a spatially homogeneous magnetic field. NeuroImage 244 (2021) 118484.
% Tierney et al. Spherical harmonic based noise rejection and neuronal 
% sampling with multi-axis OPMs. NeuroImage 258 (2022) 119338.
% Copyright (C) 2011, ATR All Rights Reserved.
% License : New BSD License(see VBMEG_LICENSE.txt)

disp(mfilename);

% Set input directory
input_dir = fullfile(p.proj_root, p.dirname.modality, input_dirname, p.task);

% Set output directory
output_dir = fullfile(p.proj_root, p.dirname.modality, output_dirname, p.task);
if ~exist(output_dir, 'dir')
    mkdir(output_dir)
end

% If parm_ssp is not specified in input, the default values are used 
if ~exist('parm_ssp','var') || isempty(parm_ssp)
    parm_ssp =[];
    parm_ssp.ssp_basis = 'VSH'; % Vector Spherical Harmonics
    parm_ssp.vsh_lmax = 2; % VSH with lmax=2
    parm_ssp.remove_by_regression = false; % If true, denoising with regress-out
    parm_ssp.use_vbcssp = false; % If true, compute coefficients of basis and channels using vbCSSP
end

% Save figures of results?
if ~exist('doPlot', 'var') || isempty(doPlot)
    doPlot = true;
end

%% Check SSP parameters
% Get basis of the noise space
ssp_basis = upper(parm_ssp.ssp_basis);

% Check input parameters for each ssp_basis
switch ssp_basis
    case 'HFC'
        %--- Homogeneous Field Correction
        % No additional parameters are required

    case 'VSH'
        %--- Noise space composed of Vector Spherical Harmonics function
        % Check lmax (order) of VSH function
        if ~isfield(parm_ssp, 'lmax') || isempty(parm_ssp.lmax)
            lmax = 2;
        else
            lmax = parm_ssp.lmax;
        end

    case 'EMPTY'
        %--- Noise space derived from empty room data
        % Check empty room file
        if ~isfield(parm_ssp, 'empty_file') || isempty(parm_ssp.empty_file)
            % error('''empty_file'' is required for basis composed of empty room recording')
            % In this dataset, empty room data is set in below directory
            empty_file = fullfile(fileparts(p.proj_root), 'EMPTY', p.dirname.modality, input_dirname, 'run01.meg.mat');
        else
            empty_file = parm_ssp.empty_file;
        end
        % Check number of components to be used
        if ~isfield(parm_ssp, 'num_comp') || isempty(parm_ssp.num_comp)
            num_comp = 5;
        else
            num_comp = parm_ssp.num_comp;
        end

    case 'EXTRA'
        %--- Use leadfield of the extra-dipoles as noise basis
        if ~isfield(parm_ssp, 'extra_basis_file') || isempty(parm_ssp.extra_basis_file)
            error('''extra_basis_file'' is required for basis of extra-dipoles')
        else
            extra_basis_file = parm_ssp.extra_basis_file;
        end

    otherwise
        error([ssp_basis 'is not supported'])
end

% If use_vbcssp is true, coefficients of basis and channels are computed using vbCSSP
% (Default: false)
if isfield(parm_ssp, 'use_vbcssp')
    use_vbcssp = parm_ssp.use_vbcssp;
else
    use_vbcssp = false;
end

% If remove_by_regression is true, denoising is conducted by regress-out
% (Default: false)
if isfield(parm_ssp, 'remove_by_regression')
    remove_by_regression = parm_ssp.remove_by_regression;
else
    remove_by_regression = false;
end

%% Start processing
for run = 1:p.num_run
    file_name = sprintf('run%02d', run);
    input_file = fullfile(input_dir, [file_name '.meg.mat']);
    output_file = fullfile(output_dir, [file_name '.meg.mat']);

    % Load MEG data
    % (bad channels are ignored because they may affect estimated homogeneous field)
    loadspec = [];
    loadspec.ChannelType = 'MEG';
    loadspec.ActiveChannel = true; % Load active channels
    loadspec.ActiveTrial   = false; % Load all the trials
    [bexp, ch_info, time_info] = vb_load_meg_data(input_file, loadspec);
    time = time_info.time;
    freq = time_info.sample_frequency;
    [nch, ntime, ntrial] = size(bexp);
    % If bexp is already epoched, make it flat
    if ntrial~=1
        bexp = reshape(bexp, [nch, ntime*ntrial]);
    end

    y = bexp;

    % Select basis of the noise space
    switch ssp_basis
        case 'HFC'
            %--- Homogeneous Field Correction
            % Use Qpick of the active channels
            [~, Qpick] = vb_load_sensor(input_file, 'MEG');
            S = Qpick;
            % Set the name of new ref channels
            CHNAMES = {'HF_X', 'HF_Y', 'HF_Z'};
        case 'VSH'
            %--- Noise space composed of Vector Spherical Harmonics function
            [pick_m, Qpick] = vb_load_sensor(input_file, 'MEG');
            S = make_vsh(pick_m, Qpick, lmax);
            % Set the name of new ref channels
            CHNAMES = cell(1, size(S,2));
            ccount = 1;
            for ll=1:lmax
                mmax = 2*ll+1;
                for mm=1:mmax
                    CHNAMES{ccount} = ['l' num2str(ll) 'm' num2str(mm)];
                    ccount = ccount+1;
                end
            end
        case 'EMPTY'
            %--- Noise space derived from empty room data
            [bexp_empty, ch_info_empty, ~] = vb_load_meg_data(empty_file, loadspec);
            ixact = cellfun(@(x) find(strcmp(ch_info_empty.Name, x)), ch_info.Name);
            % bexp_empty = bexp_empty(ixact, :);
            [U, ~, ~] = svd(bexp_empty, 'econ');
            S = U(:, 1:num_comp);
            S = S(ixact, :); % Extract basis for active channels
            % Set the name of new ref channels
            CHNAMES = cellstr('sv' + string(1:num_comp));

        case 'EXTRA'
            %--- Use leadfield of the extra-dipoles as noise basis
            Gex = load(extra_basis_file, 'basis', 'Pointlist');
            S = Gex.basis';
            % Set the name of new ref channels
            CHNAMES = [];
            for ee=1:length(Gex.Pointlist)
                CHNAMES = [CHNAMES {[Gex.Pointlist{ee}.name '_X'], [Gex.Pointlist{ee}.name '_Y'], [Gex.Pointlist{ee}.name '_Z']}];
            end  
    end

    %% Remove saturated signal
    ynosat = y;

    % Remove saturated signal in the case of OPM
    if strcmp(p.dirname.modality, 'opm-meg')
        % Set raw data, which is required for checking saturation
        raw_file = fullfile(p.proj_root, p.dirname.modality, p.dirname.load, p.task, [file_name '.meg.mat']);
        [raw_data,~,raw_time_info] = vb_load_meg_data(raw_file, loadspec);
        % Check if even one channel is saturated
        flag_sat = any( or(raw_data>p.dr_ub, raw_data<p.dr_lb) );
        % Down sampling (matching the time index)
        if time_info.sample_frequency~=raw_time_info.sample_frequency
            % index_ds = dsearchn(raw_time_info.time', time_info.time'); % takes minutes...
            % flag_sat = flag_sat(index_ds);
            flag_sat = vb_convert_freq(flag_sat, raw_time_info.sample_frequency, time_info.sample_frequency);
        end

        % Remove saturated time points
        ynosat(:,flag_sat) = []; 
    end

    %% Apply denoising using selected basis
    % Objective: Computing coefficients using normalized non-saturated data and applying them to normalized data.
    % 1. Normalizing data with no saturation (ynosat -> ynosat_z).
    [ynosat_z, ynosat_ave, ynosat_std] = normalize_data(ynosat, 'variance');

    % 2. Compute coefficients of basis using ynosat_z.
    if use_vbcssp
        % Compute VB
        [~, M, Hfilt, C, FE] = vbCSSP(ynosat_z, S, 1000);
        S = C * S; % Replace S by gain mudalated one
    else
        % Compute OLS filters
        Hfilt = pinv(S); % (X'X)^(-1)*X'
        M = eye(size(S,1)) - S*Hfilt; % Projection matrix to epsilon (residuals of noise = brain space)
        C = [];
        FE = [];
    end

    % 3. Normalizing y using pre-calculated std
    %    without saturated signals (y -> yz).
    [yz] = normalize_data(y, 'variance', ynosat_ave, ynosat_std);

    % 4. Apply denoising in normalized space to all data.
    Hz = Hfilt * yz; % Compute time-series of the noises
    if remove_by_regression
        % yz_denoised = remove_by_regression(yz, Hz);
        % Mz = eye(size(yz,2)) - Hz' * inv(Hz*Hz') * Hz; % Use TxT memory
        % yz_denoised = yz*Mz; % Multiply from right
        M = eye(size(S,1)) - yz*Hz' * inv(Hz*Hz') * Hfilt;
        yz_denoised = M*yz;
    else
        yz_denoised = M*yz;
    end

    % 5. Re-normalizing
    y_denoised = re_normalize_data(yz_denoised, 'variance', ynosat_ave, ynosat_std);
    H          = re_normalize_data(Hz, 'variance', ynosat_ave, ynosat_std);

    bexp = y_denoised;

    if doPlot
        % Show results
        set_fig_property(12, 2, 10, 10);
        h = figure; hold on
        subplot(2, 1, 1), plot(time, y'), title(['Before ' ssp_basis]), set_axis
        xlim([min(time) max(time)]), ylim([min(y(:)) max(y(:))])
        ylabel('Magnetic field [T]')
        subplot(2, 1, 2), plot(time, y_denoised'), title(['After ' ssp_basis]), set_axis
        xlim([min(time) max(time)]), ylim([min(y_denoised(:)) max(y_denoised(:))])
        ylabel('Magnetic field [T]')
        xlabel('Time [sec]')
        fig_file = fullfile(p.fig_root, mfilename, p.task, p.dirname.modality, output_dirname, [p.sub '_run' num2str(run) '_denoised']);
        vb_savefig_as_shown(h, fig_file)
        close(h)

        % Show weights
        set_fig_property(4, 2, 10, 10);
        h = figure;
        H_plot = mean(abs(H), 2);
        SH_plot = mean(abs(S*H), 2);
        subplot(2,1,1)
        bar(H_plot),
        title('Absoluted mean of estimated weights')
        xlabel(['Basis (' ssp_basis ')'])
        xticks(1:length(CHNAMES))
        xticklabels(CHNAMES)
        xtickangle(50)
        set_axis
        subplot(2,1,2)
        bar(SH_plot),
        title('Absoluted mean of estimated weights for each ch')
        xlabel('Channels')
        set_axis
        fig_file = fullfile(p.fig_root, mfilename, p.task, p.dirname.modality, output_dirname, [p.sub '_run' num2str(run) '_weights']);
        vb_savefig_as_shown(h, fig_file)
        close(h)

        % Show time-series of noises (weights of basis)
        Nref = length(CHNAMES);
        Nrow = min(6, Nref);
        Nfig = ceil(Nref/Nrow);
        set_fig_property(12, Nrow, 10, 10);
        for ff=1:Nfig
            h = figure; hold on
            if Nref-(Nrow*ff) < 0
                rmax = Nrow + Nref-(Nrow*ff);
            else 
                rmax = Nrow;
            end
            for rr=1:rmax
                ix_plot = rr+(ff-1)*Nrow;
                H_plot = H(ix_plot,:);
                CHNAME_plot = CHNAMES(ix_plot);
                subplot(Nrow, 1, rr), plot(time, H_plot), title(CHNAME_plot), set_axis
                xlim([min(time) max(time)])
                % ylim([min(H(:)) max(H(:))]) 
                if rr==rmax
                    xlabel('Time [sec]')
                end
            end
            fig_file = fullfile(p.fig_root, mfilename, p.task, p.dirname.modality, output_dirname, [p.sub '_run' num2str(run) '_noise' num2str(ff)]);
            vb_savefig_as_shown(h, fig_file)
            close(h)
        end

        if ~isempty(C)
            set_fig_property(4, 2, 10, 10);
            % Show estimated channel gains
            h = figure;
            C_plot = diag(C);
            bar(C_plot),set_axis, title('Estimated channel gains')
            grid minor
            xlabel('Channels')
            xlim([0 length(C_plot)+1])
            ylim(minmax_with_margine(C_plot, 5))
            fig_file = fullfile(p.fig_root, mfilename, p.task, p.dirname.modality, output_dirname, [p.sub '_run' num2str(run) '_cgain']);
            vb_savefig_as_shown(h, fig_file)
            close(h)
        end

        if ~isempty(FE)
            set_fig_property(4, 2, 10, 10);
            % Show estimation of the free energy
            h = figure;  plot(FE)
            title('Free energy'), set_axis
            fig_file = fullfile(p.fig_root, mfilename, p.task, p.dirname.modality, output_dirname, [p.sub '_run' num2str(run) '_evidence']);
            vb_savefig_as_shown(h, fig_file)
            close(h)
        end
    end

    %% Save projection matrix M
    file_proj = fullfile(output_dir, ['M_' file_name '.mat']);
    save(file_proj, 'M', 'Hfilt', 'S', 'C', 'FE');

    disp(['The projection matrix M was saved in ' file_proj '.'])

    %% Save denoised data
    % If original bexp was epoched, re-shape it to original size
    if ntrial~=1
        bexp = reshape(bexp, [nch, ntime, ntrial]);
        % Reshape estiamted homogeneous field as well
        H    = reshape(H, [size(H,1), ntime, ntrial]);
    end

    % Make output file and append estimated noises as reference ch
    append_ch = [];
    append_ch.data = H;
    append_ch.name = CHNAMES;
    append_ch.type = repmat(257, 1, size(H,1));
    append_extra_ch(input_file, append_ch, output_file); % It makes output_file

    % Overwrite bexp with updated data
    append_ch = [];
    append_ch.data = bexp;
    append_ch.name = ch_info.Name;
    append_ch.type = ch_info.Type;
    append_data_ch(output_file, append_ch)
    disp([output_file ' was saved.'])
end



