function [data_file] = find_temporal_peak(p, prefix_in, reference_file)
% Find the temporal peaks
% Detected peaks will be save in the data_file
% If reference_file is specified, then time_peak_sec is copied
%
% Copyright (C) 2011, ATR All Rights Reserved.
% License : New BSD License(see VBMEG_LICENSE.txt)

disp(mfilename);

% Set prefix of input file
if isempty(prefix_in)
    prefix_in_ = [];
else
    prefix_in_ = [prefix_in '_'];
end

%% Set parameters
% Temporal peaks are searched in time_of_interest_sec
twin_of_interest_sec = p.time_of_interest_sec;
% Set the maximum number of temporal peaks that will be detected
if ~isfield(p, 'num_temporal_peak') || isempty(p.num_temporal_peak)
    num_peak = 1;
else
    num_peak = p.num_temporal_peak;
end
% Set window around the detected peaks
if ~isfield(p, 'peak_win_sec') || isempty(p.peak_win_sec)
    peak_win_msec = 0; % Only the peak time
else
    peak_win_msec = p.peak_win_sec * 1000; % sec -> msec
end

% Detected peak will be saved in data_file
data_file = fullfile(p.proj_root, p.dirname.modality, p.dirname.trial, [prefix_in_ p.task '.info.mat']);

if exist('reference_file','var') && ~isempty(reference_file)
    % If the reference_file is specified, the time of temporal peaks is copied from it
    disp(['Load temporal peaks from ' reference_file])
    load(reference_file, 'time_peak_sec', 'time_peak_win_sec')
    disp(['Temporal peaks are appended to ' data_file])
    save(data_file, 'time_peak_sec', 'time_peak_win_sec', '-append')
    return
end

%% Load data and time information
[bexp, ~, time_info] = vb_load_meg_data(data_file);
% Get index of the time of interest
[~, from_toi_ix] = min(abs(time_info.time-twin_of_interest_sec(1)));
[~, to_toi_ix] = min(abs(time_info.time-twin_of_interest_sec(2)));

time_sec = time_info.time;
time_toi_sec = time_info.time(from_toi_ix:to_toi_ix);

% Calc the size of the window around the peaks
Tmsec = 1000/time_info.sample_frequency; 
% The window must include peak time itself
margin_msec =  peak_win_msec - Tmsec;
if margin_msec < Tmsec
    % No margin before and after the peak
    win_msec(1) = 0;
    win_msec(2) = 0;
else
    win_msec(1) = -margin_msec/2;
    win_msec(2) = margin_msec/2;
end
% msec -> index
% If margin_msec is divisible, -win_ix(1)=win_ix(2)
win_ix(1) = floor(win_msec(1)/Tmsec);
win_ix(2) = ceil(win_msec(2)/Tmsec);

%% Detect temporal peaks from trial-averaged MEG/EEG data
% Average data across trials
data = mean(bexp, 3);

% Compute power within TOI
py = mean(data(:,from_toi_ix:to_toi_ix).^2, 1);

% Calc diff of power
dpy = diff(py);

% Detect peaks of power time-series
ix = find(dpy(1:end-1) > 0 & dpy(2:end) < 0) + 1;
num_detected_peak = length(ix);
if  num_detected_peak == 1
    time_peak_ix = ix+from_toi_ix-1;
elseif num_detected_peak >= 2
    % num_peak must not be bigger than actually detected number
    num_peak = min(num_peak, num_detected_peak); 
    % If more than 2 peaks are detected, get num_peak in descending order
    ix2 = sortrows([ix(:) py(ix)'], 2);
    time_peak_ix = sort(ix2(end-num_peak+1:end, 1))+from_toi_ix-1;% index of peak time
elseif isempty(ix)
    % When no peak is detected, select a near local peak
    [~,ix_lpeak] = min(abs(dpy));
    time_peak_ix = ix_lpeak+from_toi_ix-1;
end

% Add the margin to make windows around the peaks
time_peak_win_ix = repmat(time_peak_ix, [1,2]) + repmat(win_ix, [length(time_peak_ix),1]);
% Fit the window to inside the time of interest
time_peak_win_ix = [max(time_peak_win_ix(:,1),from_toi_ix), min(time_peak_win_ix(:,2),to_toi_ix)]; % 'from' is lower bound and 'to' is upper bound
% ix -> sec
time_peak_sec = time_sec(time_peak_ix)';
time_peak_win_sec = time_sec(time_peak_win_ix);

% Append the time of temporal peaks to info.mat
disp(['Detected temporal peaks are appended to ' data_file])
save(data_file, 'time_peak_sec', 'time_peak_win_sec', '-append')
