function features = my_ICL_feature_extractor(ic, w, s, pos, srate)
% Extract features for the ICLabel Classifier
% This function is based on ICL_feature_extractor.m

% If autocorr is empty in features, run_ICL returns empty
flag_autocorr = true;

% Prepare ICA results
icawinv    = pinv(w*s);
icaweights = w;
icasphere  = s;

icaact = ic;
[ncomp, pnts, trials] = size(icaact);

icaact = double(icaact); % Cast to double

% check ica is real
assert(isreal(icaact), 'Your ICA decomposition must be real to use ICLabel')

%% Make EEG structure to call EEGlab functions
EEG = [];
EEG.icaweights = icaweights;
EEG.icaact = icaact;
EEG.pnts = pnts;
EEG.trials = trials;
EEG.srate = srate;

%% Calc topo using a part of VBMEG function
topo = zeros(32, 32, 1, ncomp);
for it = 1:ncomp
    temp_topo = inner_calc_topo(pos, icawinv(:,it), 32);
    temp_topo(isnan(temp_topo)) = 0;
    topo(:, :, 1, it) = temp_topo / max(abs(temp_topo(:)));
end

% cast
topo = single(topo);

%% Calc psd using EEGlab function
psd = eeg_rpsd(EEG, 100);

% extrapolate or prune as needed
nfreq = size(psd, 2);
if nfreq < 100
    psd = [psd, repmat(psd(:, end), 1, 100 - nfreq)];
end

notch_freqs = [55 65]; % This depends on the filtering before ICA
notch_width = notch_freqs(2)-notch_freqs(1);

% undo notch filter
for linenoise_ind = [notch_freqs(1):notch_freqs(2)]
    linenoise_around = [linenoise_ind - notch_width, linenoise_ind + notch_width];
    difference = bsxfun(@minus, psd(:, linenoise_around), ...
        psd(:, linenoise_ind));
    notch_ind = all(difference > 5, 2);
    if any(notch_ind)
        psd(notch_ind, linenoise_ind) = mean(psd(notch_ind, linenoise_around), 2);
    end
end

% normalize
psd = bsxfun(@rdivide, psd, max(abs(psd), [], 2));

% reshape and cast
psd = single(permute(psd, [3 2 4 1]));

%% Calc autocorrelation using EEGlab function
if flag_autocorr
    if trials == 1
        if pnts / srate > 5
            autocorr = eeg_autocorr_welch(EEG);
        else
            autocorr = eeg_autocorr(EEG);
        end
    else
        autocorr = eeg_autocorr_fftw(EEG);
    end

    % reshape and cast
    autocorr = single(permute(autocorr, [3 2 4 1]));
end

%% format outputs
if flag_autocorr
    features = {0.99 * topo, 0.99 * psd, 0.99 * autocorr};
else
    features = {0.99 * topo, 0.99 * psd};
end

end % End of the function

function [Z] = inner_calc_topo(x,z, XYres)
% Plot M/EEG spatial pattern on 2 dimentional surface.
% 
% [input]
% x                    : <<matrix>> Sensor position.
% z                    : <<vector>> Signal value.
% clim                 : <optional> <<vector>> Minimum and maximum of contour map.
% face_disp            : <optional> plot nose and ears(true or false) (default : false)
% highlight_channel_ix : <optional> <<vector>> highlight channel index(x's indices)
% XY_scale             : <optional> face radius expand/shrink ratio(default : 1.0)
%
% [note]
% If the coordinate system of the sensor position x is RAS (default of
% VBMEG), the contour plot will be drawn in the neurological convention,
% i.e., the view is above the subject and the right of the plot
% corresponds to the right of the subject. 
%
% [history]
% 2010-10-04 Taku Yoshioka
% 2012-06-20 taku-y
%  [trivial] Comment was modified. 
%
% Copyright (C) 2011, ATR All Rights Reserved.
% License : New BSD License(see VBMEG_LICENSE.txt)

% --- Input arguments
if size(z,2)==1, z = z'; end

XY_scale = 1;

% Resolution of the outout image
if ~exist('XYres','var') || isempty(XYres)
    XYres = 32; % 32x32
end
XYres = XYres-1;

[phi,theta,r] = cart2sph(x(:,1),x(:,2),x(:,3));
[xa,ya,za] = pol2cart(phi,0.5*pi-theta,r);
% xa = mean(xa)-xa;
% ya = mean(ya)-ya;
% ya = ya * -1;
% xa = xa * -1;

dx = (max(xa)-min(xa))/200;
dy = (max(ya)-min(ya))/200;
r_max = max(sqrt(xa.^2+ya.^2));
xaa = xa;
yaa = ya;

xa = [xa; (XY_scale*1.1*r_max*cos(0:0.02*pi:1.99*pi))'];
ya = [ya; (XY_scale*1.1*r_max*sin(0:0.02*pi:1.99*pi))'];
za = [z'; zeros(100,1)];

% Set resolution of output image
dx = (max(xa)-min(xa)) / XYres;
dy = (max(ya)-min(ya)) / XYres;

% Colormap
warning('off', 'MATLAB:griddata:DuplicateDataPoints');
[X,Y,Z] = griddata(xa,ya,za,...
		   (min(xa):dx:max(xa)),...
		   (min(ya):dy:max(ya))','cubic');
end