function update_leadfield(p, prefix, dir_ssp, update_opt)
% Update leadfield 
%
% Copyright (C) 2011, ATR All Rights Reserved.
% License : New BSD License(see VBMEG_LICENSE.txt)

% Show leadfield norm with angles
angle = [-90 0; 90 0; -90 0; 90 0; 0 90];
Nangle = size(angle,1);

% Set modality-specific parameters
switch p.dirname.modality
    case 'opm-meg'
        extention = '.meg.mat';
        ch_type  = [6, 5];
        ch_label = {'Z', 'Y'};
    otherwise
        warning('This function is for OPM-MEG')
        return
end

% Option for preparation of projection matrix M
% Default: use M of run01
if ~exist('update_opt', 'var') || isempty(update_opt)
    update_opt = 'run01';
end

% Set preprocessed MEG file 
ref_meg_file = fullfile(p.proj_root, p.dirname.modality, p.dirname.trial, [prefix '_' p.task '.info.mat']);

% Load channel info of active ch from preprocessed MEG file (corresponds to leadfield)
[~, channel_info] = vb_load_channel(ref_meg_file, true);
Nch = length(channel_info.Name);

% Set MEG and projection files in SSP dir
if strcmp(update_opt(1:3), 'run')
    % Use single run (assume M is consistent across runs)
    method = 'single';
    proj_file{1} = fullfile(p.proj_root, p.dirname.modality, dir_ssp, p.task, ['M_' update_opt '.mat']);
    meg_file{1}  = fullfile(p.proj_root, p.dirname.modality, dir_ssp, p.task, [update_opt '.meg.mat']);
else
    switch update_opt
        case 'average'
            % M will be averaged across runs
            method = 'average';
            for run=1:p.num_run
                proj_file{run} = fullfile(p.proj_root, p.dirname.modality, dir_ssp, p.task, ['M_' sprintf('run%02d', run) '.mat']);
                meg_file{run}  = fullfile(p.proj_root, p.dirname.modality, dir_ssp, p.task, [sprintf('run%02d', run) '.meg.mat']);
            end
        otherwise
            error([update_opt ' is not supported.'])
    end
end

% Prepare projection matrix to brain space
Mall = zeros(Nch, Nch, length(proj_file));
for ff=1:length(proj_file)
    % Load channel info when SSP applied
    [~, channel_info_ssp] = vb_load_channel(meg_file{ff}, true);

    % Load projection matrix
    load(proj_file{ff}, 'M')

    % Check rejected channels after SSP
    chid_rejected = setdiff(channel_info_ssp.ID, channel_info.ID);
    chix_rejected = ismember(channel_info_ssp.ID, chid_rejected);

    % Remove projection matrix of rejected channels
    M = M(~chix_rejected, ~chix_rejected);

    % Store M
    Mall(:,:,ff) = M;
end

% Summarize projection matrices across run
switch method
    case 'single'
        M = Mall(:,:,1);
    case 'average'
        M = mean(Mall, 3);
end

% Load leadfild
% Note: LF is computed only for active ch
basis_file = fullfile(p.proj_root, p.dirname.modality, p.dirname.leadfield, [p.task '.basis.mat']);
basis = vb_load_basis(basis_file);

% Update leadfield
basis_new = basis*M'; % = (M*Leadfield)'

%% Save results
% Copy original basis file
[d,f,e] = fileparts(basis_file);
basis_file_backup = fullfile(d, ['_' f e]);

% If a backup file does not exists, make it
if ~exist(basis_file_backup, 'file')
    % Copy to backup file
    copyfile(basis_file, basis_file_backup)
end

% Save updated leadfield matrix by overwriting
G = load(basis_file);
G.basis = basis_new;
save(basis_file, '-struct', 'G')
disp([basis_file ' was overwitten.'])

%% Prepare visualization
% Load brain model
brain_file = fullfile(p.proj_root, p.dirname.brain, [p.struct_name, '.brain.mat']);
[V, F, xx, inf_C] = vb_load_cortex(brain_file, 'Inflate');
plot_parm = vb_set_plot_parm;

% Find corresponds index of specified ch_type
chix = [];
for cc=1:length(ch_type)
    chix = [chix, channel_info.Type==ch_type(cc)];
end

% In case of OPM/SQUID, add 'All' index
% by checking if chix is a part of all or not
% (In EEG, chix(:,1) covers all index of channel)
if sum(chix(:,1)) ~= Nch
    ch_label{end+1} = 'All';
    chix(:,end+1) = ones(Nch,1);
end

% Set column of subplot
Nax = length(ch_label);

% Set figure properties
set_fig_property(3, Nax, 10, 10);

%% Plot gain of leadfield norm
h = figure;

% Calc lfnorm using all channels for consistent colorbar
lfn0 = sum(basis.^2, 2).^0.5;
lfn1 = sum(basis_new.^2, 2).^0.5;
lfn_gain = lfn1 ./ lfn0;
ma = max(lfn_gain(:));

for ax = 1:Nax
    % Set ch
    chix_plot = find(chix(:,ax));
    nch = length(chix_plot);

    for an = 1:Nangle
        if an<=2
            plot_parm.LRflag = 'L';
        elseif an<=4
            plot_parm.LRflag = 'R';
        else
            plot_parm.LRflag = 'LR';
        end

        subplot(Nax, Nangle, an+Nangle*(ax-1)) 
        
        % Calc lfnorm
        lfn0 = sum(basis(:,chix_plot).^2, 2).^0.5;
        lfn1 = sum(basis_new(:,chix_plot).^2, 2).^0.5;
        lfn_gain = lfn1 ./ lfn0;

        % Plot cortex and lfnorm
        vb_plot_cortex(plot_parm, V, F, inf_C, lfn_gain, ma*0.05, ma);
        axis equal off
        colormap(my_parula)
        % Set view angle
        view(angle(an, :))

        if an == 1
            title(['lfn_{new} / lfn (' ch_label{ax} ': ' num2str(nch) ' channels)'])
        end
        if an==Nangle
            colorbar
        end
    end
end

% Save figure
fig_file = fullfile(p.fig_root, mfilename, p.dirname.modality, ['lfngain_' p.sub '_' p.task '_' dir_ssp]);
vb_savefig_as_shown(h, fig_file)
disp([fig_file '.png was saved.'])

%% Plot diff of leadfield norm
h = figure;

% Calc lfnorm using all channels for consistent colorbar
lfn0 = sum(basis.^2, 2).^0.5;
lfn1 = sum(basis_new.^2, 2).^0.5;
lfn_diff = lfn0 - lfn1;
ma = max(lfn_diff(:));

for ax = 1:Nax
    % Set ch
    chix_plot = find(chix(:,ax));
    nch = length(chix_plot);

    for an = 1:Nangle
        if an<=2
            plot_parm.LRflag = 'L';
        elseif an<=4
            plot_parm.LRflag = 'R';
        else
            plot_parm.LRflag = 'LR';
        end

        subplot(Nax, Nangle, an+Nangle*(ax-1))   

        % Calc lfnorm
        lfn0 = sum(basis(:,chix_plot).^2, 2).^0.5;
        lfn1 = sum(basis_new(:,chix_plot).^2, 2).^0.5;
        lfn_diff = lfn0 - lfn1;

        % Plot cortex and lfnorm
        vb_plot_cortex(plot_parm, V, F, inf_C, lfn_diff, ma*0.05, ma);
        axis equal off
        colormap(my_parula)
        % Set view angle
        view(angle(an, :))

        if an == 1
            title(['lfn - lfn_{new} (' ch_label{ax} ': ' num2str(nch) ' channels)'])
        end
        if an==Nangle
            colorbar
        end
    end
end

% Save figure
fig_file = fullfile(p.fig_root, mfilename, p.dirname.modality, ['lfndiff_' p.sub '_' p.task '_' dir_ssp]);
vb_savefig_as_shown(h, fig_file)
disp([fig_file '.png was saved.'])

%% Plot diff of leadfield
h = figure;

basis_diff = basis - basis_new;
% Calc lfnorm using all channels for consistent colorbar
lfn_diff = sum(basis_diff.^2, 2).^0.5;
ma = max(lfn_diff(:));

for ax = 1:Nax
    % Set ch
    chix_plot = find(chix(:,ax));
    nch = length(chix_plot);

    for an = 1:Nangle
        if an<=2
            plot_parm.LRflag = 'L';
        elseif an<=4
            plot_parm.LRflag = 'R';
        else
            plot_parm.LRflag = 'LR';
        end

        subplot(Nax, Nangle, an+Nangle*(ax-1))
        
        % Calc lfnorm
        lfn_diff = sum(basis_diff(:,chix_plot).^2, 2).^0.5;

        % Plot cortex and lfnorm
        vb_plot_cortex(plot_parm, V, F, inf_C, lfn_diff, ma*0.05, ma);
        axis equal off
        colormap(my_parula)
        % Set view angle
        view(angle(an, :))

        if an == 1
            title(['Norm of lf - lf_{new} (' ch_label{ax} ': ' num2str(nch) ' channels)'])
        end
        if an==Nangle
            colorbar
        end
    end
end

% Save figure
fig_file = fullfile(p.fig_root, mfilename, p.dirname.modality, ['diff_' p.sub '_' p.task '_' dir_ssp]);
vb_savefig_as_shown(h, fig_file)
disp([fig_file '.png was saved.'])