function p = set_parameters(root_raw, root_analysis, modality, sub, task, num_run)
% Set parameters and directory names for analyzing OPM-MEG data
%
% Copyright (C) 2011, ATR All Rights Reserved.
% License : New BSD License(see VBMEG_LICENSE.txt)

%% Set base directories and files
p = struct;
p.sub = sub;
p.task = task;
p.raw_data_root = fullfile(root_raw, p.sub);% Raw data directory of a subject
p.proj_root = fullfile(root_analysis, p.sub);% Directory to save analyzed data
p.fig_root = fullfile(root_analysis, 'figure');% Directory to save figures
if ~exist(p.fig_root, 'dir'), mkdir(p.fig_root); end

if isempty(num_run)
    p.num_run = 1;
else
    p.num_run = num_run;
end

%% Set parameters for each task
switch task
    case 'Somatosensory'
        pretrigger_ms  = 300;
        posttrigger_ms = 500;
        time_base_ms = [-pretrigger_ms -200];
        toi = [0.01 0.05];
        twin_sec = 0.002;
        num_max_peak = 3;
    case 'Auditory'
        pretrigger_ms  = 300;
        posttrigger_ms = 500;
        time_base_ms = [-pretrigger_ms -200];
        toi = [0.02 0.15];
        twin_sec = 0.04;
        num_max_peak = 3;
    case 'Motor'
        pretrigger_ms  = 700;
        posttrigger_ms = 500;
        time_base_ms = [-pretrigger_ms -500];
        toi = [0.05 0.15];
        twin_sec = 0.02;
        num_max_peak = 3;
    otherwise
        pretrigger_ms  = [];
        posttrigger_ms = [];
        time_base_ms = [];
        toi = [];
        twin_sec = [];
        num_max_peak = [];
end

% For making the trial data
p.Pretrigger_ms  = pretrigger_ms; % Before trigger [msec]
p.Posttrigger_ms = posttrigger_ms; % After trigger [msec]
% Set the baseline period
p.time_base_sec = time_base_ms ./ 1000 ; % time window of baseline [s]
% Temporal peaks are searched in time_of_interest_sec by find_temporal_peak
p.time_of_interest_sec = toi;
% Set the maximum number of temporal peaks that will be detected
p.num_temporal_peak = num_max_peak;
% Set the size of a window around the detected peaks
p.peak_win_sec = twin_sec;

%% For visualizing the trial-averaged MEG/EEG signals and estimated source currents
p.time_show_sec = [-p.Pretrigger_ms p.Posttrigger_ms] ./ 1000;

%% For estimating source current using VBMEG
p.time_noise_sec  = p.time_base_sec; % Period for baseline [s]

% Set Uniform prior
p.prior = 'Uniform';
p.prior_weight = 10^-4; % Relative influence of prior information (0-1)

%% Threshold for trial and channel rejection used in reject_chtr.m
% Lower/Upper, and standard deviation bounds
switch modality
    case 'squid'
        p.threshold_rejection = [0.1, 15, 10];    
    case 'eeg'
        p.threshold_rejection = [0.1, 9, 10];
    case 'opm'
        p.threshold_rejection = [1, 6, 10];
end

%% Parameters of tPCA for denoising SQUID data
[parm_tpca] = struct(...
    'Twin_sec', [0.05 0.05],...
    'smin',     1e-6);
p.tpca = parm_tpca;

%% Pre-filtering parameters used in apply_filtering.m
[parm_filter] = struct(...
    'bandpass_freq', [1 100],...
    'bandpass_filtertype' , 'butter',...
    'bandpass_filterorder', 4,...
    'bandstop_freq', [57 63; 117 123],...
    'bandstop_filtertype' , 'butter',...
    'bandstop_filterorder', 4,...
    'fsamp', 500);
p.prefilter = parm_filter;

%% Filtering parameters used in apply_filtering.m
[parm_filter] = struct(...
    'bandpass_freq', [6 50],...
    'bandpass_filtertype' , 'butter',...
    'bandpass_filterorder', 4);
p.filter = parm_filter;

%% Dynamic-range of OPM for detecting saturation used in vb_job_trial_onset_opm.m
p.dr_ub = 1.5*10^-9;  % Upper bound
p.dr_lb = -1*p.dr_ub; % Lower bound

%% Parameters of SSP
if strcmp(modality, 'opm')
    parm_ssp = [];
    parm_ssp.ssp_basis = 'VSH'; % Vector Spherical Harmonics
    parm_ssp.lmax = 2; % Order of VSH
    parm_ssp.remove_by_regression = false; % If true, denoising with regress-out
    parm_ssp.use_vbcssp = false; % If true, compute coefficients of basis and channels using vbCSSP
    
    p.ssp = parm_ssp;
end

%% Parameters of ICA for classifying ICs
parm_ica = [];
switch modality
    case 'squid'
        parm_ica.classify_method = '';
        parm_ica.manual_selection = true; % Apply manually selected ICs in set_ic.m
        parm_ica.eog_threshold = 0.2; % IC is noise if explained variance of EOGs > threshold
    case 'eeg'
        parm_ica.classify_method = 'ICL'; % IClabeling implemented in the EEGlab plugin
        parm_ica.ICL_threshold_noise_prob = 0.9; % Noise ICs with low prob will be discarded
        parm_ica.manual_selection = false; % Apply manually selected ICs in set_ic.m
        parm_ica.eog_threshold = 0.2; % IC is noise if explained variance of EOGs > threshold
    case 'opm'
        parm_ica.classify_method = '';
        parm_ica.manual_selection = false; % Apply manually selected ICs in set_ic.m
end

p.ica = parm_ica;

%% Set the parameters of EMG trigger detection (refered in convert_emg_trigger.m)
% These were fine-tuned by hands.
if strcmp(modality, 'squid')
    switch sub
        case '002'
            th_scale  = [1.5, 1, 1, 1];
            len_close = [1000, 1000, 1000, 1000];
            len_open  = [10, 10, 10, 10];
        case '005'
            th_scale  = [2.7, 1.7, 1.5, 2.5];
            len_close = [1000, 1000, 1000, 1000];
            len_open  = [10, 10, 10, 10];
        case '006'
            th_scale  = [1.2, 1.2, 0.9, 1.3];
            len_close = [1600, 500, 500, 800];
            len_open  = [10, 10, 10, 10];
        case '093'
            th_scale  = [0.9, 0.9, 0.9];
            len_close = [550, 500, 500];
            len_open  = [10, 10, 10];
        case '095'
            th_scale = [0.9, 0.9, 0.9, 0.9, 0.9];
            len_close = [200, 200, 200, 200, 200];
            len_open = [700, 400, 400, 400, 400];
    end
    p.emg_detect.th_scale = th_scale;
    p.emg_detect.len_close = len_close;
    p.emg_detect.len_open = len_open;
end

%% File and directory names
%--- Common fields across all modality
p.struct_name = 'Subject';

[parm_dirname] = struct(...
    ... % For brain modeling
    't1',       'struct',...
    'fs',       'freesurfer',...
    'brain',    'brain', ...
    ... % For source current estimation
    'leadfield','leadfield',...
    'current',  'current' ...
    );

% Add data and preprocess dir for each modality
switch modality
    case 'eeg'
        parm_dirname.modality = 'eeg';
        parm_dirname.load = 'load';
        parm_dirname.prefilter = 'prefilter';
        parm_dirname.filter = 'filter';
        parm_dirname.trial = 'trial';
        parm_dirname.eog = 'eog';
        parm_dirname.clean = 'clean';        
    case 'squid'
        parm_dirname.modality = 'squid-meg';
        parm_dirname.load = 'load';
        parm_dirname.ica = 'ica';
        parm_dirname.tpca = 'tpca';
        parm_dirname.prefilter = 'prefilter';
        parm_dirname.filter = 'filter';
        parm_dirname.trial = 'trial';
    case 'opm'
        parm_dirname.modality = 'opm-meg';
        parm_dirname.load = 'load';
        parm_dirname.ssp = 'ssp';
        parm_dirname.detrend = 'detrend';
        parm_dirname.prefilter = 'prefilter';
        parm_dirname.filter = 'filter';
        parm_dirname.trial = 'trial';
        parm_dirname.eog = 'eog';
    case 'meeg'
        parm_dirname.modality = 'meeg';
end

p.dirname = parm_dirname;
