function estimate_source_current_freq(p, prefix_in, toi_sec, foi_hz)
% Estimate source currents from MEG/EEG data in the frequency space
%
% Copyright (C) 2011, ATR All Rights Reserved.
% License : New BSD License(see VBMEG_LICENSE.txt)

disp(mfilename);

% Set prefix of input file
if isempty(prefix_in)
    prefix_in_ = [];
else
    prefix_in_ = [prefix_in '_'];
end

if size(toi_sec,2)~=2 || size(foi_hz,2)~=2
    error('Wrong shape of the window.')
end

if size(toi_sec,1) ~= size(foi_hz,1)
    error('Time and freq of Interest must have consistent num of windows.')
end

num_win = size(toi_sec, 1);

% Max size of the channels that can be loaded at the same time for saving RAM
% If limit_ch_size < num_ch, do block computation in compute_csd (slow but saving RAM)
% If it is too large, an out of memory error may occur when loading TFR data
limit_ch_size = 80;

%% Set common parameters
time_noise_sec = p.time_noise_sec; % Period for baseline [s]
%time_noise_sec = [-0.3 -0.15]; % Period for baseline [s]

% Set input files (abs path)
brainfile = fullfile(p.proj_root, p.dirname.brain, [p.struct_name, '.brain.mat']);
areafile  = fullfile(p.proj_root, p.dirname.brain, [p.struct_name, '.area.mat']);
actfile   = fullfile(p.proj_root, p.dirname.brain, [p.struct_name, '.act.mat']);
tfrfile  = fullfile(p.proj_root, p.dirname.modality, p.dirname.trial, [prefix_in_ p.task '_tfr.info.mat']);
basisfile = fullfile(p.proj_root, p.dirname.modality, p.dirname.leadfield, [p.task '.basis.mat']); 

% In case of SQUID-MEG, use (active) AXIAL channels only
% (because of too many channels)
if strcmp(p.dirname.modality, 'squid-meg')
    load_spec = [];
    load_spec.ChannelType = 'AXIAL';
    load_spec.ActiveChannel = true;
    channel_info = get_channel_info(tfrfile, load_spec);
    ch_list = channel_info.Name;
else
    ch_list = {};
end

% Load time-freq information]
tfr_info = get_tfr_info(tfrfile);
time_sec = tfr_info.time;
freq_hz  = tfr_info.frequency;

% Set index of the time window for baseline
[~, from_ix] = min(abs(time_sec-time_noise_sec(1)));
[~, to_ix] = min(abs(time_sec-time_noise_sec(2)));
twin_noise_ix = repmat([from_ix to_ix], [num_win,1]);

% Set output directory
outputdir = fullfile(p.proj_root, p.dirname.modality, p.dirname.current);
if ~exist(outputdir, 'dir')
    mkdir(outputdir)
end

%% Prepare windows and current files
twin_meg_ix = zeros(num_win ,2);
fwin_ix     = zeros(num_win ,2);
currfile    = cell(num_win, 1);
for win=1:num_win
    % Set time and freq window
    twin_toi_sec = toi_sec(win, :);
    fwin_foi_hz  = foi_hz(win, :);

    % Get index of the time window for stimulus-evoked activity
    [~, from_ix] = min(abs(time_sec-twin_toi_sec(1)));
    [~, to_ix]   = min(abs(time_sec-twin_toi_sec(2)));
    twin_meg_ix(win,:)  = [from_ix to_ix];

    % Get index of the freq window for analysis
    [~, from_ix] = min(abs(freq_hz-fwin_foi_hz(1)));
    [~, to_ix]   = min(abs(freq_hz-fwin_foi_hz(2)));
    fwin_ix(win,:)  = [from_ix to_ix];

    % Set ID of output files
    fileid = [p.task '_dics_' num2str(win)];

    % Set output current file
    filename = [fileid '.curr.mat'];
    currfile{win} = fullfile(outputdir, filename);
end

%% Start estimation
% Estimation using selected method
disp(['Start estimation by DICS beamformer ...'])

dics_parm = set_dics_parm( ...
    'brainfile', brainfile,...
    'tfrfile',   tfrfile,...
    'basisfile', basisfile,...
    'areafile',  areafile,...
    'actfile',   actfile,...
    'twin_meg',  twin_meg_ix,...
    'twin_noise',twin_noise_ix,...
    'fwin',      fwin_ix,...
    'currfile',  currfile,...
    'ch_list',   ch_list,...
    'limit_ch_size', limit_ch_size,...
    'cov_reg',   0.05,...
    'basis_normalization', OFF);

job_dics(dics_parm);

% Append input MEG/EEG file to current file
for win=1:num_win
    save(currfile{win}, 'tfrfile', '-append')
end

end % End of peak loop