function show_source_current_freq(p, norm_mode, polarity)
% Show source current estimated from MEG data in the frequency space
% norm_mode : Normalization mode (See load_current_dics). 
% polarity : Porality of the visualized results ('both', 'pos', 'neg').
%
% Copyright (C) 2011, ATR All Rights Reserved.
% License : New BSD License(see VBMEG_LICENSE.txt)

disp(mfilename);
close all

if ~exist('norm_mode', 'var') || isempty(norm_mode)
    norm_mode = 'F'; % Pseudo F-ratio
end

if ~exist('polarity', 'var') || isempty(polarity)
    polarity = 'both'; % Retain both pos and neg
end

% Angles to see brain
angle = [-90 0; 90 0; 0 90];

% Set brain file
brain_file = fullfile(p.proj_root, p.dirname.brain, [p.struct_name, '.brain.mat']);

% Set dir of currents
current_dir = fullfile(p.proj_root, p.dirname.modality, p.dirname.current);

method = 'dics';

% Set dir and prefix of current file
prefix_fileid = [p.task '_' method '_'];

% Find current files having specified fileid in current_dir
% current_files is struct
current_files = dir(fullfile(current_dir, [prefix_fileid '*' '.curr.mat']));

% Sort the files
fid = zeros(length(current_files), 1);
for ff=1:length(current_files)
    % Extract the number included in the name
    temp = extract(current_files(ff).name(end-10:end-9), digitsPattern);
    fid(ff) = str2double(temp{1});
end
[~, newix] = sort(fid);
current_files = current_files(newix);

% Loop for files
for ff=1:length(current_files)
filename = current_files(ff).name;
current_file = fullfile(current_dir, filename);

% Load results of DICS and index of currents
load(current_file, 'Model', 'dics_parm')
tfrfile = dics_parm.tfrfile;

% Load ch and time information
ch_info = get_channel_info(tfrfile);
chnames = ch_info.Name;

tfr_info = get_tfr_info(tfrfile);
time_sec = tfr_info.time;
freq_hz  = tfr_info.frequency;

% Prepare CSD for visualization
Cm = abs(Model.CSDm);
Cn = abs(Model.CSDn);

% Load normalized currents
[Jmap, ~, Cr] = load_current_dics(current_file, 2, norm_mode);

% Set label of colorbar
switch norm_mode
    case 'F'
        label_cb = 'Pseudo F-ratio';
    case 'P'
        label_cb = 'Power';
    case 'R'
        label_cb = 'Power of data / power of noise';
end

% For visualizing pos or neg values
switch polarity
    case 'both'
        % Retain both pos and neg
        % (Do nothing)
        % cmap = my_parula();
    case 'pos'
        % Retain positive values to extract ERS
        Jmap(Jmap<0) = 0; % (Set neg to 0)
        % cmap = autumn();
    case 'neg'
        % Retain negative values to extract ERD
        Jmap(Jmap>0) = 0; % (Set pos to 0)
        % cmap = my_sky();
end

%% Plot
set_fig_property(4, 1, 10, 10);

fig_prefix = [method '_'];
fig_suffix = ['_' norm_mode num2str(ff)];
toi = time_sec(Model.twin_meg);
foi = round( freq_hz(Model.fwin), 2);
disp_thresh = max(abs(Jmap))*0.3;
label_map = [method ' ' norm_mode 'map (>' num2str(disp_thresh,'%.2e') ')'];

% Plot for temporal-averaged data (within ToI)
fig_file = fullfile(p.fig_root, mfilename, p.dirname.modality, p.task, [fig_prefix p.sub fig_suffix]);
inner_plot_and_save_figure(brain_file, angle, Jmap, disp_thresh, toi, foi, label_map, label_cb, fig_file)

%% Plot cross-spectrum density matrix
set_fig_property(9, 3, 5, 5);

fig_file = fullfile(p.fig_root, mfilename, p.dirname.modality, p.task, ['CSD_' p.sub fig_suffix]);

h = figure;
subplot(1,3,1)
imagesc(Cm)
axis square
set_axis
xticks([1:length(chnames)])
xticklabels(chnames)
xtickangle(90)
yticks([1:length(chnames)])
yticklabels(chnames)
colorbar
title('abs(CSD data)')

subplot(1,3,2)
imagesc(Cn)
axis square
set_axis
xticks([1:length(chnames)])
xticklabels(chnames)
xtickangle(90)
yticks([1:length(chnames)])
yticklabels(chnames)
colorbar
title('abs(CSD noise)')

subplot(1,3,3)
imagesc(Cr)
axis square
set_axis
xticks([1:length(chnames)])
xticklabels(chnames)
xtickangle(90)
yticks([1:length(chnames)])
yticklabels(chnames)
colorbar
title('abs(CSD ratio)')

% Save figure
vb_savefig_as_shown(h, fig_file)
disp([fig_file '.png was saved.'])

end % end of file loop

end % end of function

function inner_plot_and_save_figure(brain_file, angle, ymap, disp_thresh, time_of_interest, freq_of_interest, label_map, label_cb, fig_file)
h = figure;
% Load brain model on which current is shown
[V, F, xx, inf_C] = vb_load_cortex(brain_file, 'Inflate');
max_xyz = max(abs(V), [], 1);
plot_parm = vb_set_plot_parm;
plot_parm.wmode = false; % Plot raw value

% Plot spatial pattern of absolute value of current
for an = 1:size(angle, 1)
    subplot(1, size(angle, 1), an)
    vb_plot_cortex(plot_parm, V, F, inf_C, ymap, disp_thresh, max(abs(ymap)));
    axis([-max_xyz(1) max_xyz(1) -max_xyz(2) max_xyz(2) -max_xyz(3) max_xyz(3)])
    axis equal off
    view(angle(an, :))
    %zoom(1.5)
    if an==2
        if length(time_of_interest)==2
            toi_plot = ([num2str(time_of_interest(1)) '-' num2str(time_of_interest(2)) 'sec']);
        else
            toi_plot = ([num2str(time_of_interest(1)) 'sec']);
        end
        if length(freq_of_interest)==2
            foi_plot = ([num2str(freq_of_interest(1)) '-' num2str(freq_of_interest(2)) 'Hz']);
        else
            foi_plot = ([num2str(freq_of_interest(1)) 'Hz']);
        end
        title([label_map ' (' toi_plot ', ' foi_plot ')'])
    end
    if an==3
        % Add colorbar
        cb = colorbar;
        cb.Label.String = label_cb;
    end
end
colormap(my_parula())

% Save figure
vb_savefig_as_shown(h, fig_file)
disp([fig_file '.png was saved.'])

end % end of function