function show_tfr(p, prefix_in, show_ch)
% Show time-frequency representation.
% If too many channels are showed at the same time,
% specifiy show_ch to avoid an out of memory error.
% Note: Inactive channels are not shown even if they are listed in show_ch. 
% e.g. show_ch=[1 20] : show 1 to 20 active channels
%
% Copyright (C) 2011, ATR All Rights Reserved.
% License : New BSD License(see VBMEG_LICENSE.txt)

disp(mfilename);
close all

% Set prefix of input file
if isempty(prefix_in)
    prefix_in_ = [];
else
    prefix_in_ = [prefix_in '_'];
end

% If show_ch is input, show specified ch
if ~exist('show_ch','var')
    show_ch = [];
end

% Set figure properties
set_fig_property(3.5, 2.5, 10, 10);

% Set time window parameters
twin_to_show_sec     = p.time_show_sec; % sec
twin_of_interest_sec = p.time_of_interest_sec; % sec
twin_base_sec        = p.time_base_sec; % sec

%% Load tfr data and info
% Set input file
tfr_file = fullfile(p.proj_root, p.dirname.modality, p.dirname.trial, [prefix_in_ p.task '_tfr.info.mat']);

% Load channel info
channel_info = get_channel_info(tfr_file);

% Make ch filter
if isempty(show_ch)
    % If show_ch is empty,
    % show all active channels.
    show_ch_ix = logical(channel_info.Active);
else
    show_ch_ix = false(size(channel_info.Active));
    show_ch_ix(show_ch(1):show_ch(2)) = true;
    % Show only active channels
    show_ch_ix = and(show_ch_ix, channel_info.Active);
end

% Index -> Name
show_ch_name = channel_info.Name(show_ch_ix);
num_ch = length(show_ch_name);
% Logical index -> sequential index
show_ch_list = find(show_ch_ix);

% Load TFR data and info of channel and tfr
load_spec = [];
load_spec.ChannelName = show_ch_name;
[tfr, ~, tfr_info] = vb_load_tfr_data(tfr_file, load_spec);
sfreq = tfr_info.sample_frequency;
frequency = tfr_info.frequency;
num_freq = length(frequency);
time_sec = tfr_info.time;

%% Convert twin from sec to ix
% Time window for showing
[~, from_show_ix] = min(abs(time_sec-twin_to_show_sec(1)));
[~, to_show_ix]   = min(abs(time_sec-twin_to_show_sec(2)));
time_show_sec = tfr_info.time(from_show_ix:to_show_ix);
num_time = length(time_show_sec);

% For calc power of topomap
[~, from_toi_ix] = min(abs(time_sec - twin_of_interest_sec(1)));
[~, to_toi_ix]   = min(abs(time_sec - twin_of_interest_sec(2)));

% For normalization
[~, from_base_ix] = min(abs(time_sec - twin_base_sec(1)));
[~, to_base_ix]   = min(abs(time_sec - twin_base_sec(2)));

% Load positions of channels
[pos, ~] = vb_load_channel(tfr_file);

% % Load MEG/EEG data (used for topo plot)
% meg_file = fullfile(p.proj_root, p.dirname.modality, p.dirname.trial, [prefix_in_ p.task '.info.mat']);
% bexp = vb_load_meg_data(meg_file);
% 
% % Calc mean power within time of interests
% y = mean(bexp(:, from_toi_ix:to_toi_ix, :), 3); % Trial-average
% py = mean(y.^2, 2);

py = zeros(size(pos,1), 1);

%% Prepare for visualizing TFR
% Convert to PSD and trial-averaging
tfr_plot = nanmean( abs(tfr).^2 , 4 );

% Normalization with unit [dB]
tfr_plot = normalize_tfr(tfr_plot, [from_base_ix,to_base_ix], 'db');
 
% Extract time window for showing
tfr_plot = tfr_plot(:,:,from_show_ix:to_show_ix);

%% Compute ITPC
itpc = calc_itpc(tfr);

% Extract time window for showing
itpc = itpc(:,:,from_show_ix:to_show_ix);

%% Compute freq-wise reproducibility
% Decimate data across freq-dim due to computation time
d_step = 3; % Must be integer

% Decimation
frequency_rep = frequency(1:d_step:end);
tfr_rep = tfr(:,1:d_step:end,:,:);

% Convert to PSD 
tfr_rep = abs(tfr_rep).^2;

% Normalization with unit [dB]
tfr_rep = normalize_tfr(tfr_rep, [from_base_ix,to_base_ix], 'db');
 
% Extract time window for showing
tfr_rep = tfr_rep(:,:,from_show_ix:to_show_ix,:);

%% Set the limits of colorbar for each plot
min_tfr = -max(abs(tfr_plot(:)));
max_tfr = max(abs(tfr_plot(:)));
min_itpc = 0;
max_itpc = 1;
min_rep = 0;
max_rep = 1;

%% Show TFR, ITPC, and freq-wise reproducibility
for cc=1:num_ch
    h = figure;
    chname = show_ch_name{cc};

    % Plot TFR
    subplot(2,2,1)
    cb_label = 'Normalized power [dB]';
    inner_plot_tfr(time_show_sec, frequency, tfr_plot(cc,:,:), cb_label, min_tfr, max_tfr, tfr_info.method)

    % Plot ITPC
    subplot(2,2,2)
    cb_label = 'coherence';
    inner_plot_tfr(time_show_sec, frequency, itpc(cc,:,:), cb_label, min_itpc, max_itpc, tfr_info.method)
    title(['ITPC'])

    % Plot repoducibility
    subplot(2,2,3)
    inner_plot_reproducibility(time_show_sec, frequency_rep, tfr_rep(cc,:,:,:), twin_of_interest_sec, min_rep, max_rep, tfr_info.method)

    % Index in show_ch -> all ch
    ch = show_ch_list(cc);

    % Add sensor position
    subplot(2,2,4)
    vb_plot_sensor_2d(pos(:,:), py, [], true, ch);
    axis square off
    % colorbar
    title(['Position of ' chname])

    % Save figure
    fig_file = fullfile(p.fig_root, mfilename, p.dirname.modality, p.task, p.sub, ['ch' num2str(ch)]);
    vb_savefig_as_shown(h, fig_file)
    disp([fig_file '.png was saved.'])
    close(h)
end

end % End of the function

function inner_plot_tfr(time, freq, tfr, cb_label, cb_min, cb_max, method, freq_band_table)
% Input tf must be single ch
% Shape is [freq * time] or [1 * freq * time]
% cb_unit : Unit of colorbar
% cb_min  : Min of colorbar
% cb_max  : Max of colorbar

if ndims(tfr)==3 && size(tfr,1)==1
    % Remove ch dim
    tfr = shiftdim(tfr,1); % It delete the first dim when first dim=1
else
    error('Check dim of tfr');
end

if ~exist('cb_label','var')
    cb_label = '';
end

if ~exist('cb_min','var') || isempty(cb_min)
    cb_min = min(tfr(:));
end

if ~exist('cb_max','var') || isempty(cb_max)
    cb_max = max(tfr(:));
end

if ~exist('method','var') || isempty(method)
    method = 'wavelet';
end

if ~exist('freq_band_table','var') || isempty(freq_band_table)
    % Set the default frequency bands for hilbert
    freq_band_table = get_freq_bands();
end

switch method
    case 'wavelet'
        contourf(time, freq, tfr, 40, 'linecolor', 'none')
        title(['Wavelet decomposition'])
        ylabel('Frequency (Hz)')
    case 'hilbert'
        imagesc(time, freq, tfr)
        title(['Hilbert transform'])
        axis xy
        ylabel('Frequency')
        yticks(freq)
        yticklabels(freq_band_table.Row)
end
xlabel('Time (sec)')
% Add onset line
xline(0, ':', 'LineWidth',2, 'Color',[0.9 0.9 0.9])

% Edit colorbar
caxis([cb_min cb_max])
cb = colorbar;
cb.Label.String = cb_label;
cb.Label.Rotation = 270;
cb.Label.VerticalAlignment = 'bottom';
set_axis

end % End of the function


function inner_plot_reproducibility(time, freq, tfr, time_of_interest, cb_min, cb_max, method, freq_band_table)
%% Input check
if ~exist('cb_min','var') || isempty(cb_min)
    cb_min = 0;
end

if ~exist('cb_max','var') || isempty(cb_max)
    cb_max = 1;
end

if ~exist('method','var') || isempty(method)
    method = 'wavelet';
end

if ~exist('freq_band_table','var') || isempty(freq_band_table)
    % Set the default frequency bands for hilbert
    freq_band_table = get_freq_bands();
end

%% Plot
switch method
    case 'wavelet'
        plot_tfr_reproducibility(tfr, freq, time, time_of_interest, 5)
        ylabel('Frequency (Hz)')
    case 'hilbert'
        plot_tfr_reproducibility(tfr, freq, time, time_of_interest, 5, true)
        ylabel('Frequency')
        yticks(freq)
        yticklabels(freq_band_table.Row)
end
caxis([cb_min cb_max])


end % End of the function