function [D, sig2, Dindex, IF, GoF] = fmri_dipole_estimation(basis, B, area, Cov, option)
% Dipole method with fMRI constraints (by the variable "area") 
%
% --- input
% basis : lead field matrix (Nch*Nj) (Nj=Ndipole*Ndirection)
%         different axes of the same dipole must be continuously aligned
% B     : MEG time series data (Nch*Nt)
% area  : area information where the dipoles are set. (Narea*1 cell)
% Cov   : observation noise covariance (Nch*Nch) [eye(Nch)]
% option.Ntrain      : number of training [5]
% option.compute_GoF : if it's true, compute GoF (default: true)
% option.Ndirection  : number of the current direction
%                      Ndirection=1 (scalar, default)
%                      Ndirection=3 (vector)
%
% --- output
% D      : dipole time series (Narea * Nt)
% sig2   : variance
% Dindex : index of dipoles
% IF     : Inverse Filter
% GoF    : Goodness of Fit for all vertices (Nj*1)
%
% 2005/05/20 Okito Yamashita
% 2023/09/12 Keita Suzuki
%            Inverse filter F was renamed as IF and appended to outputs
% 2024/04/05 Keita Suzuki
%            GoF was computed and added to output
% 2024/06/12 Keita Suzuki
%            option.compute_GoF was added since it is slightly heavy for long time-series data
%            option.direction was added for computing a vector dipole
%

[Nch, Nt] = size(B);
Narea = length(area);

%%% defult parameters 
Ntrain = 5;
compute_GoF = true;
Ndirection = 1;
S = eye(Nch);  % inverse of Cov 

% replace defaults
if exist('option','var') && isfield(option, 'Ntrain')
    Ntrain = option.Ntrain;
end
if exist('option','var') && isfield(option, 'compute_GoF')
    compute_GoF = option.compute_GoF;
end
if exist('option','var') && isfield(option, 'Ndirection')
    Ndirection = option.Ndirection;
end
if exist('Cov','var') == 1 && isempty(Cov) ~=1 
    S = inv(Cov);
end

% define the index function with specified num of direction
ixd = @(ix) (1:Ndirection) + (ix-1)*Ndirection; % index of dipole

% initial explanation matrix 
X0 = [];
for na = 1 : Narea
    ixarea = area{na}.Iextract; % all dipole index in an area
    % x0 = basis(:,ixarea(ixd(1))); % use the first dipole
    x0 = basis(:,ixd(ixarea(1))); % use the first dipole
    X0 = [X0, x0];  
end 
    
BB = B*B'; % tr(BB')

% iterative optimization of dipole position
ixopt = zeros(1,Narea);
for nlearn = 1 : Ntrain
    % searching between regions
    for na = 1 : Narea 
        ixarea = area{na}.Iextract; % all dipole index in an area
        Nix = length(ixarea); % num of dipoles in an area
        ixx = ixd(na); % index in X
   
        X = X0;
        sig2 = NaN * ones(Nix,1);
        % searching inside a region
        for j = 1 : Nix 
            % X(:,na) = basis(:,ix(j)); % in case of single direction
            X(:,ixx) = basis(:,ixd(ixarea(j))); % replace previous basis by new one
   
            IF =  inv(X'*S*X)*X'* S; % inverse filter
            PSP = S-S*X*IF; % projection matrix                                   
            sig2(j) = sum(sum(PSP.*BB))/Nt/Nch; % evaluate new dipole
        end
        [sig2min, jopt] = min(sig2); % find the optimal dipole
        X0(:,ixx) = basis(:, ixd(ixarea(jopt))); % replace previous basis by the optimal one
        temp = ixd(ixarea(jopt));
        ixopt(na) = temp(end) / Ndirection; % index of the optimal dipole (not basis)
    end
    fprintf('iter=%d, error=%e, index:', nlearn, sig2min);
    fprintf(' %.0f ', ixopt);
    fprintf('\n');
end
 
% compute reconstructed time-series of the optimal dipols
X = X0; 
IF =  inv(X'*S*X)*X'* S; % inverse filter
PSP = S-S*X*IF; % projection matrix           
sig2 = sum(sum(PSP.*BB))/Nt/Nch;
D = IF * B;  % Dipole estimate
Dindex = uint32(ixopt);

if compute_GoF
    fprintf('Computing GoF ...')
    % compute GoF for all dipole candidates sololy (assuming single dipole)
    Nix = size(basis,2) / Ndirection; % num of dipoles in all area
    GoF = zeros(Nix,1);
    deno = sum(B.^2);
    % for showing progress
    prog_per = 10;
    prog_next = floor(Nix/prog_per);
    prog_count = 1;
    for j=1:Nix
        if j==prog_next*prog_count
            fprintf('%d%%...', prog_per*prog_count);
            prog_count = prog_count + 1;
        end
        x = basis(:,ixd(j));
        w =  inv(x'*S*x)*x'* S; % inverse filter for a dipole
        d = w * B;  % Dipole estimate

        Bhat = x*d; % Bhat

        nume = sum((B-Bhat).^2);
        GoF(j) = mean(1 - nume./deno); % GoF at ix (temporal averaged)
    end
    fprintf('\n');
else
    disp('Skip computing GoF')
    GoF = [];
end



