function job_lcmv(lcmv_parm)
% Run LCMV beamformer
%
% 2024/03/14 K.Suzuki
% * Simple computation of noise covariance
% * basis_normalization option is added
% * cov_reg option is added (lam in LCMV.m)
% 2015/08/25 O.Yamashita

fprintf('--------------------------------------\n');
fprintf('      LCMV beamformer \n');
fprintf('--------------------------------------\n');

%---------------------------------
% Compatibility with bayes_parm 
% Don't modify 
%---------------------------------

lcmv_parm.megfile_baseline  = lcmv_parm.megfile;
lcmv_parm.reduce = 1;
lcmv_parm.Rfilt  = 0; 
lcmv_parm.Tperiod   = lcmv_parm.twin_meg(2) - lcmv_parm.twin_meg(1)+1;
lcmv_parm.Tnext     = lcmv_parm.Tperiod; % Time step for next period

%---------------------
%  Run job
%---------------------

% Prepare noise covariance
% [Cov, sx0] = vb_observation_noise_specification(lcmv_parm);
% Cn = sx0*Cov{1};
Cn = inner_noise_covariance_preparation(lcmv_parm);

% Prepare MEG data in signal interval
[B, Ntrials, Nsensors, Tsample, Twindow] = vb_megdata_preparation(lcmv_parm);

% Prepare leadfield
[G, ix, ix_ex, W, Norient] = vb_focal_leadfield_preparation(lcmv_parm,1);

if lcmv_parm.basis_normalization
    % Prepare leadfield with normalization (vertex-wise)
    % This is Array-Gain constraint
    Gnorm = vecnorm(G{1}); % after r2017b (vecnorm)
    G{1} = G{1} ./ repmat( Gnorm, [Nsensors,1] ); 
end

[J, Jn, W, Wn, S, Sn] = LCMV(B{1}, G{1}, Cn, lcmv_parm.cov_reg);


Jest = J;


%------------------------------------------
% Save Curr file 
% Compatibility with .curr.mat of VBMEG
%------------------------------------------
Tstart = Twindow(1);
Tend   = Twindow(end);
Jact   = Jest;
ix_act = ix;
Lact   = Norient;
Varact = Sn(:);

Jbck   = [];
ix_bck = [];
Lbck   = [];

Model.nCov = Cn;
if lcmv_parm.saveifilt
Model.IF   = W;
Model.nIF  = Wn;
end


MEGinfo = vb_load_meg_info(lcmv_parm.megfile);

fprintf('Save current reconstruction result to %s ... \n', lcmv_parm.currfile);
save(lcmv_parm.currfile, 'Jact', 'Jbck', 'ix_act', 'ix_bck', 'Varact',...
    'Tstart', 'Tend', 'MEGinfo', 'lcmv_parm', 'Model');
fprintf('Done ... \n');

end

function Cn = inner_noise_covariance_preparation(lcmv_parm)
% Prepare noise parm for loading MEG data in noise interval
noise_parm = lcmv_parm;
noise_parm.megfile  = lcmv_parm.megfile_baseline;
noise_parm.twin_meg = lcmv_parm.twin_noise;

noise_parm.Tperiod   = noise_parm.twin_meg(2) - noise_parm.twin_meg(1)+1;
noise_parm.Tnext     = noise_parm.Tperiod; % Time step for next period

% Prepare MEG data in noise interval
[B, Ntrials, Nsensors, Tsample, Twindow] = vb_megdata_preparation(noise_parm);

B = B{1};
[Nch, Nt, Ntrial] = size(B);

%
B2d = reshape(B,[Nch,Nt*Ntrial]);

Cn = B2d*B2d'/(Nt*Ntrial);
end

