function [Pm, Pn, F, Wm, Wn] = DICS(Cm, G, Cn, lam)
%  DICS BEAMFORMER (SCALAR)
%
% [Input]
%   Cm : data CSD matirx
%   G  : leadfield matrix
%   Cn : noise CSD matrix (optional)
%        Pn, Wn, and F will be empty if Cn is not input.
%  lam : (optional)
%        Regularization parameter for computing the inverse of covariance
%        matrix. This value controls a tradeoff between the amount of detail 
%        in the power maps and their sensitivity to noise. 
%        The default value is 5% (0.05)
%
% [Output]
%   Pm :  power of data currents [Nv]
%   Pn :  power of noise currents [Nv]
%   F  :  Pseudo F-ratio (noise-normalized power) [Nv]
%   Wm :  filter of data currents [Nv*Nch]
%   Wn :  filter of noise currents [Nv*Nch]
%
% 2024/03/14 K.Suzuki
% 2023/10/19 K.Suzuki
% * first version is made based on LCMV.m
% 2017/09/08 O.Yamashita
% * compatible with trial_average = off
% 2015/08/21 O.Yamashita

if ~exist('Cn','var') || isempty(Cn)
    Cn = 0;
end

if ~exist('lam','var') || isempty(lam)
    lam = 0.05;
end

[Nch, Nv] = size(G);
lam = lam/Nch; % For computing averaged singular value (trace)

% Sum of data CSD and noise CSD for F-ratio
Cs = Cm + Cn;

% Calc pinv with diagonal loading
iCm = pinv(Cm+ lam*trace(Cm)*eye(Nch));
iCn = pinv(Cn+ lam*trace(Cn)*eye(Nch));
iCs  = pinv(Cs+ lam*trace(Cs)*eye(Nch));

Pm = zeros(Nv, 1);
Pn = zeros(Nv, 1);
F = zeros(Nv,1);
Wm = zeros(Nv, Nch);
Wn = zeros(Nv, Nch);
Ws = zeros(Nv, Nch);

fprintf('vertex\n');
for vv = 1 : Nv
    if mod(vv, 1000) == 0,
        fprintf('%d ', vv);
    end
   
    g = G(:,vv);
    
    % Calc denominator
    varm = inv(g'*iCm*g);
    varn = inv(g'*iCn*g);
    vars = inv(g'*iCn*g);
    
    % Calc filter
    wm = varm*g'*iCm;
    wn = varn*g'*iCn;
    ws = vars*g'*iCs;
    
    % Calc power
    Pm(vv) = real(wm*Cm*wm');
    Pn(vv) = real(wn*Cn*wn');

    % Calc pseudo F-ratio 
    F(vv) = real( (ws*Cm*ws' - ws*Cn*ws') / (ws*Cn*ws') );

    Wm(vv,:) = wm;
    Wn(vv,:) = wn;
    Ws(vv,:) = ws;
end

fprintf('\n');

% If Cn is not specified in input, related variables become empty
if Cn==0
    Wn = [];
    Pn = [];
    F  = [];
end