function [TF, index_freq] = compute_bandpass(data, sfreq, OPTS)
% Compute bandpass filtering and Hilbert transformation (if required).
%
% data : single ch [#time x #trial]
% sfreq : sampling frequency
% OPTS : 
%    apply_hilbert  : If true, apply Hilbert transformation after
%                     bandppass filtering (default: false)
%                     If you require only bandpass filtered data, disable it
%    ix_baseline    : Index of baseline [ix_start, ix_end]
%    average_trial  : If true, do trial-averaging after wavelet (default: true)
%    calc_power     : If true, do taking absolute-value after wavelet (default: true)
%    normalize_type : If ix_baseline exists, normalize_type are selected from {'db', 'divide', 'z'}
%    freq_band_table     : Table of frequency bands (see get_freq_bands.m)
%    filter_type    : Type of the filter (default: 'butter')
%                     {'butter', 'eegfilt', 'eegfilt_takeda'} 
%    filter_order   : Order of a butterworth filter (default: 4)
%                     It will not be reffered in case of 'eegfilt' or 'eegfilt_takeda' 
%

% Set default options if they are not specified
OPTS = check_options(OPTS);

freq_band_table = OPTS.freq_band_table;
apply_hilbert = OPTS.apply_hilbert;

[num_time, num_trial] = size(data);
num_freq = size(freq_band_table,1);

% Set index of frequencies
index_freq = 1:num_freq;

% Filtering params
filter_type = OPTS.filter_type;
filter_order = OPTS.filter_order; % For 'butter' or 'onlinebutter'
doPlot = false;

% Init
TF = zeros(num_time, num_trial, num_freq);

% Loop on each frequency band
for ff = 1:num_freq
    lb = freq_band_table{ff,'lower_bound'};
    ub = freq_band_table{ff,'upper_bound'};
    % Bandpass filtering
    [filtered,Bb,Ab] = inner_bandpass(data', sfreq, lb, ub, filter_type, filter_order);
    if doPlot, inner_plot_filter_result(data',filtered,Bb,Ab,'band',[lb ub],sfreq,1); end

    if apply_hilbert
        % Apply Hilbert transform
        TF(:,:,ff) = hilbert(filtered');
    else
        % Band-pass filtered data
        TF(:,:,ff) = filtered';
    end
end

% [#time x #trial x #frequency] -> [#frequency x #time x #trial]
TF = permute(TF, [3,1,2]);

if OPTS.calc_power
    % Comute power
    TF = abs(TF).^2;
end

if OPTS.average_trial
    % Trial-averaging
    TF = squeeze( mean(TF,3) );
end

% Baseline normalization
% If average_trial=off, do trial-wise correction
if OPTS.baseline_normalization
    ix_baseline = OPTS.ix_baseline;
    % baseline averaging and calc std for each trial (not trial averaging)
    base_mean = mean( TF(:, ix_baseline(1):ix_baseline(2), :), 2);
    base_std  = std( TF(:, ix_baseline(1):ix_baseline(2), :), [], 2);
    base_mean_rep = repmat(base_mean, [1,num_time,1]);
    base_std_rep  = repmat(base_std, [1,num_time,1]);
    switch OPTS.normalize_type
        case 'db'
            TF = 10*log10(TF ./ base_mean_rep);
        case 'divide'
            TF = TF ./ base_mean_rep;
        case 'z'
            TF = (TF-base_mean_rep) ./ base_std_rep;
    end
end

end % End of the function


function [data, B, A] = inner_bandpass(data, samplefreq, highpassfreq, lowpassfreq, filtertype, order)
% bandpass filtering
% 
% -- Output
% data : bandpassed filtered data
%  B   : filter weights (numerator parts)
%  A   : filter weights (denominator parts). If IIR filter is used, A is empty.
%
%
% 2009/11/20 OY
% * nothing done when filtertype = [];
% 2009/09/02 OY

[Nch,Nt] = size(data);

if isempty(filtertype),
    data = data;
    B = [];
    A = [];
    return
end

switch filtertype
    case 'eegfilt'
        [data,B] = eegfilt(data, samplefreq,  highpassfreq, lowpassfreq);
        A = [];
    case 'eegfilt_takeda'
        data = eegfilt(data, samplefreq,  0, lowpassfreq);
        data = eegfilt(data, samplefreq,  highpassfreq, 0);
        A = [];
        B = [];
    case 'butter'
        [B,A] = butter(order, [highpassfreq/(samplefreq/2), lowpassfreq/(samplefreq/2)],'bandpass');
        for nch = 1 : Nch
            data(nch,:) = filtfilt(B,A,data(nch,:));
        end
        
    otherwise
        error('No filtertype. [''eegfilt'', ''eegfilt_takeda'', ''butter'']');
end
end % End of the inner function

function inner_plot_filter_result(databfr,dataaft,B,A,type,fcut,fs,isplot)
% fs : sampling frequency
% fcut : fcut(1) : low or high, [fcut(1) fcut(2)] : band

if isplot & ~isempty(B)
    switch	type
        case	'low'
            Fmax = fcut(1)*3;
            Tmax = 10/fcut(1);
        case	'high'
            Fmax = fcut(1)*5;
            Tmax = 1/fcut(1);
        case	'band'
            Fmax = fcut(2)*3;
            Tmax = 5/fcut(1);
    end
    
    t  = 0:1/fs:Tmax;
    f  = [0:0.001:1]*Fmax;
    X  = zeros(length(t),1);
    X(2) = 1;
    
    H = freqz(B,A,f,fs);
    X = filter(B,A, X );

    figure,
    subplot(3,2,[1 2])
    plot(databfr(1,1:min(1000,size(databfr,2))),'linewidth',2); hold on; plot(dataaft(1,1:min(1000,size(dataaft,2))),'r-');
    title('timeseries example (ch=1)');
    subplot 323
    plot(f,abs(H))
    xlabel('Freq[Hz]')
    title('Gain of frequency response')
    ylim([ 0  1.1])
    subplot 324
    plot(f,angle(H))
    xlabel('Freq[Hz]')
    title('Angle of frequency response')
    subplot 325
    plot(t,X)
    xlabel('Time[sec]')
    title('Impulse response')
    subplot 326
    plot(B)
    hold on
    plot(A,'--')
    xlabel('Tap number')
    title('Filter coefficient')
end
end % End of the inner function


function OPTS = check_options(OPTS)
if ~isfield(OPTS, 'average_trial') || isempty(OPTS.average_trial)
    OPTS.average_trial = true;
end

if ~isfield(OPTS, 'calc_power') || isempty(OPTS.calc_power)
    OPTS.calc_power = true;
end

% baseline correction if ix_baseline is specified
if isfield(OPTS, 'ix_baseline') && ~isempty(OPTS.ix_baseline)
    OPTS.baseline_normalization = true;
    if ~isfield(OPTS, 'normalize_type') || isempty(OPTS.normalize_type)
        OPTS.normalize_type = 'db'; % Default normalization
    end
else
    OPTS.baseline_normalization = false;
end

if ~isfield(OPTS, 'freq_band_table') || isempty(OPTS.freq_band_table)
    % Define the default frequency bands
    OPTS.freq_band_table = get_freq_bands();
end

if ~isfield(OPTS, 'filter_type') || isempty(OPTS.filter_type)
    OPTS.filter_type = 'butter';
end

if ~isfield(OPTS, 'filter_order') || isempty(OPTS.filter_order)
    OPTS.filter_order = 4;
end

if ~isfield(OPTS, 'apply_hilbert') || isempty(OPTS.apply_hilbert)
    OPTS.apply_hilbert = false;
end

end % End of the function
