function [FOI, CSD_MEG, CSD_NOISE] = compute_csd(tfrfile, csd_parm)
% Compute Cross-Spectral Density from tfrfile.
%
% csd_parm :
%   fwin       : index of freq of interest [from to] (default: all freqs)
%   twin_meg   : time index of MEG/EEG [from to] (default: time after onset)
%   twin_noise : time index of noise [from to] (default: empty)
%   ch_list    : list of channel names for CSD computing (default: all channels)
%
%   limit_ch_size : Max size of the channels that can be loaded at the same time for saving RAM (default: 30)
%                   If limit_ch_size < num_ch, do block computation.
%                   If it is too large, an out of memory error may occur when loading TFR data.
%
%   Note: Multiple windows could be set as [from1 to1; from2 to2; from3 to3]
%
%   TIPS: To minimize the COI, vb_make_tfr_data() should be conducted before trial-segmentation.
%
% [history]
% 2023-12-11 k_suzuki 
%            parameter 'ch_list' is added.
%            multiple windows are supported.
%            parameter 'limit_ch_size' and block computation are supported.
% 2023-11-11 k_suzuki
% Copyright (C) 2011, ATR All Rights Reserved.
% License : New BSD License(see VBMEG_LICENSE.txt)
%
%

%% Check params
if ~isfield(csd_parm, 'fwin')
    csd_parm.fwin = [];
end

if ~isfield(csd_parm, 'twin_meg')
    csd_parm.twin_meg = [];
end

if ~isfield(csd_parm, 'twin_noise')
    csd_parm.twin_noise = [];
end

if ~isfield(csd_parm, 'ch_list')
    csd_parm.ch_list = [];
end

if ~isfield(csd_parm, 'limit_ch_size')
    csd_parm.limit_ch_size = [];
end

%% Unpack params
fwin        = csd_parm.fwin; % index of freq of interest [from to]
twin_meg    = csd_parm.twin_meg; % time index of MEG/EEG [from to]
twin_noise  = csd_parm.twin_noise; % time index of noise [from to]
ch_list     = csd_parm.ch_list; % cell array of channel names
limit_ch_size = csd_parm.limit_ch_size;

%% Set default value for empty param
% Time-freq param
tfr_info = get_tfr_info(tfrfile);

if isempty(fwin)
    % Set all index of tfr_info.frequency
    fwin = [1 num_freq];
end

if isempty(twin_meg)
    % Set time after onst
    t_onset = 0; % Assume onset is t=0
    [~, from_ix] = min(abs(tfr_info.time-t_onset));
    to_ix = length(tfr_info.time);
    twin_meg = [from_ix to_ix];
end

if isempty(twin_noise)
    % Default is empty
    % Then csd_noise becomes empty
end

% Channel param
channel_info = get_channel_info(tfrfile);

if isempty(ch_list)
    % Set all active channels
    ch_list = channel_info.Name;
end
num_ch = length(ch_list);

load_spec = [];
load_spec.ChannelName = ch_list;

% Computation parameter
if isempty(limit_ch_size)
    limit_ch_size = 30;
end

%% Compare num of ch and limit_ch_size
if limit_ch_size >= num_ch
    num_block = 1;
else
    lcs = limit_ch_size / 2; % Because we use 2 blocks
    num_block = ceil(num_ch / lcs);
    % When the block computing, block_size*2 is #ch that will be loaded to RAM at the same time
    block_size = ceil(num_ch / num_block);
end

%% Check size of the windows
num_twin_meg = size(twin_meg, 1);
num_twin_noise = size(twin_noise, 1);
num_fwin = size(fwin, 1);
err = false;

if ~isempty(twin_noise) && num_twin_noise~=num_twin_meg
    err = true;
end

if num_twin_meg~=num_fwin
    err = true;
end

if err, error('Check the size of windows'); end

%% Preparation
FOI = cell(num_fwin,1);
CSD_MEG = cell(num_fwin,1);
if ~isempty(twin_noise)
    CSD_NOISE = cell(num_fwin,1);
else
    CSD_NOISE = [];
end

for win=1:num_fwin
    % Set freq of the interest
    foi = fwin(win,1):fwin(win,2);
    FOI{win} = foi;
    num_foi = length(foi);
    % Init CSD
    CSD_MEG{win} = zeros(num_ch, num_ch, num_foi);
    if ~isempty(twin_noise)
        CSD_NOISE{win} = zeros(num_ch, num_ch, num_foi);
    end
end

%% Launch computation
if num_block==1
% --- Start computing CSD ---
% Loading tfr data
% It may take tens of seconds or minutes
[tfr, ~, ~] = vb_load_tfr_data(tfrfile, load_spec);
% [num_ch, num_freq, num_time, num_trial] = size(tfr);

% Compute CSD for each window
for win=1:num_fwin
    % Set freq of the interest
    foi = FOI{win};
    % Extract tfr in foi
    tfr_win = tfr(:,foi,:,:);
    % Extract ToI (meg)
    tfr_meg = tfr_win(:,:,twin_meg(win,1):twin_meg(win,2),:);

    % Calc CSD of meg interval
    csd_meg = inner_calc_csd(tfr_meg);
    CSD_MEG{win} = csd_meg;

    if ~isempty(twin_noise)
        % Extract noise tfr
        tfr_noise = tfr_win(:,:,twin_noise(win,1):twin_noise(win,2),:);
        % Calc CSD of noise interval
        csd_noise = inner_calc_csd(tfr_noise);
        CSD_NOISE{win} = csd_noise;
    end
end
% ------------------------------------------
else
% --- Start block-wise computation ---
% This is slower but saving RAM
for ii = 1:ceil(num_ch / block_size)
    ix_start1 = (ii - 1) * block_size + 1;
    ix_end1 = min(ii * block_size, num_ch);
    % Load blocked data1
    load_spec = [];
    load_spec.ChannelName = ch_list(ix_start1:ix_end1);
    [tfr1, ~, ~] = vb_load_tfr_data(tfrfile, load_spec);

    for jj = ii:ceil(num_ch / block_size)
        ix_start2 = (jj - 1) * block_size + 1;
        ix_end2 = min(jj * block_size, num_ch);
        % Load blocked data2
        load_spec = [];
        load_spec.ChannelName = ch_list(ix_start2:ix_end2);
        [tfr2, ~, ~] = vb_load_tfr_data(tfrfile, load_spec);

        % Concat data1 and data2
        tfr = cat(1, tfr1, tfr2);

        % Compute blocked CSD for each window
        for win=1:num_fwin
            % Set freq of the interest
            foi = FOI{win};
            % Extract tfr in foi
            tfr_win = tfr(:,foi,:,:);

            % Extract ToI (meg)
            tfr_meg = tfr_win(:,:,twin_meg(win,1):twin_meg(win,2),:);
            % Calc CSD of meg interval
            csd_meg = inner_calc_csd(tfr_meg);
            % Store CSD
            CSD_MEG{win}(ix_start1:ix_end1, ix_start2:ix_end2, :) = csd_meg(1:size(tfr1,1), size(tfr1,1)+1:end, :);

            if ~isempty(twin_noise)
                % Extract noise tfr
                tfr_noise = tfr_win(:,:,twin_noise(win,1):twin_noise(win,2),:);
                % Calc CSD of noise interval
                csd_noise = inner_calc_csd(tfr_noise);
                % Store CSD
                CSD_NOISE{win}(ix_start1:ix_end1, ix_start2:ix_end2, :) = csd_noise(1:size(tfr1,1), size(tfr1,1)+1:end, :);
            end
        end
    end
end

% Compensate lower-triangle by complex transpose
for win=1:num_fwin
    for ff=1:size(CSD_MEG{win},3)
        CSD_MEG{win}(:,:,ff) = triu(CSD_MEG{win}(:,:,ff),1) + triu(CSD_MEG{win}(:,:,ff),1)' + diag(diag(CSD_MEG{win}(:,:,ff)));
        CSD_NOISE{win}(:,:,ff) = triu(CSD_NOISE{win}(:,:,ff),1) + triu(CSD_NOISE{win}(:,:,ff),1)' + diag(diag(CSD_NOISE{win}(:,:,ff)));
    end
end
% ------------------------------------------

end


end % End of the function

function csd = inner_calc_csd(tfr)
% Compute CSD of each frequency
% tfr : [num_ch, num_freq, num_time, num_trial]
% csd : [num_ch, num_ch, num_freq]

% Get size of tfr
[num_ch, num_freq, num_time, num_trial] = size(tfr);

% Shift dim of tfr for easy computation
% [ch,freq,time,trial] -> [time,trial,ch,freq]
tfr = shiftdim(tfr, 2);

% Start computation
csd = zeros(num_ch, num_ch, num_freq);
for ff=1:num_freq
    % Squeeze freq dim -> [time,trial,ch]
    tfr_tmp = tfr(:,:,:,ff);
    % Remove time-index including NaN 
    ix_nan = any(isnan(reshape(tfr_tmp, [num_time, num_trial*num_ch])),2);
    tfr_tmp = tfr_tmp(~ix_nan,:,:);
    % Shift dim -> [ch,time,trial]
    tfr_tmp = shiftdim(tfr_tmp,2);
    % Reshape -> [ch, time*trial]
    tfr_tmp = reshape(tfr_tmp, [num_ch, size(tfr_tmp,2)*num_trial]);
    % Calc CSD
    ee = tfr_tmp*tfr_tmp';
    csd1 = ee ./ size(tfr_tmp,2); % Average across time and trial
    csd(:,:,ff) = csd1;
end
end
