function [TF, frequencies] = compute_wavelet(data, sfreq, OPTS)
% This function requirs Brainstorm3.
%
% data : single ch [#time x #trial]
% sfreq : sampling frequency
% OPTS : 
%    ix_baseline : Index of baseline [ix_start, ix_end]
%    average_trial  : If true, do trial-averaging after wavelet (default true)
%    calc_magnitude : If true, do taking absolute-value after wavelet (default false)
%    calc_power     : If true, do taking power of absolute-value after wavelet (default true)
%    normalize_type : If ix_baseline exists, normalize_type are selected from {'db', 'divide', 'z'}
%    min_freq : (default 1)
%    max_freq : (default 100)
%    num_freq : (default 100)
%    central_freq : (default 1)
%    fwhm     : (default 3)
%    mask_coi       : If true, mask the cone of influence frequencies (default true)
%    baseline_long  : If true, calculate mean and std of baseline using TFR of concatted baseline data (default false)
%                     This option is useful when baseline period is shorter than COI.
%


% Set default options if they are not specified
OPTS = check_options(OPTS);

[num_time, num_trial] = size(data);
num_freq = OPTS.num_freq;

% Mean=0
data = bsxfun(@minus, data, mean(data,1));

% For baseline_fourier
opt_fourier = 'sd';

% Set frequencies
switch OPTS.freq_scale
    case 'log'
        frequencies = logspace(log10(OPTS.min_freq), log10(OPTS.max_freq), OPTS.num_freq);
    case 'linear'
        frequencies = linspace(OPTS.min_freq, OPTS.max_freq, OPTS.num_freq);
end

OPTS.is_mirror = 0;
if OPTS.is_mirror
    % Mirroring to compensate edges
    data = [flip(data,1); data; flip(data,1)];
end

% Compute wavelet decompositions using Brainstorm function
TF = morlet_transform(data', 0:1/sfreq:1, frequencies, OPTS.central_freq, OPTS.fwhm, 'n');

if OPTS.is_mirror
    % Re-mirroring
    TF = TF(:,num_time+1:num_time*2,:);
end

% [#trial x #time x #frequency] -> [#frequency x #time x #trial]
TF = permute(TF, [3,2,1]);

% Calc cone of influence due to the edge effect
coi_sec = OPTS.fwhm * OPTS.central_freq ./ frequencies ./ 2;
% Convert sec to index
time = 0:1/sfreq:(num_time-2)/sfreq;
coi_ix = dsearchn(time', coi_sec');

if OPTS.mask_coi
    % Mask the cone of influence frequencies as NaN
    for ff=1:num_freq
        TF(ff,1:coi_ix(ff),:) = nan;
        TF(ff,end-coi_ix(ff):end,:) = nan;
    end
end

if OPTS.calc_magnitude
    % Comute abs
    TF = abs(TF);
    opt_fourier = 'ad'; 
end

if OPTS.calc_power
    % Comute power
    TF = abs(TF).^2;
    opt_fourier = 'psd';
end

if OPTS.average_trial
    % Trial-averaging
    TF = squeeze( nanmean(TF,3) );
end

% Baseline normalization
% If average_trial=off, do trial-wise correction
if OPTS.baseline_normalization
    ix_baseline = OPTS.ix_baseline;

    % calc mean and std of baseline across time for each trial (not trial averaging)
    base_mean = nanmean( TF(:, ix_baseline(1):ix_baseline(2), :), 2);
    base_std  = nanstd( TF(:, ix_baseline(1):ix_baseline(2), :), [], 2);

%     if OPTS.baseline_fourier
%         % If baseline_fourier is true, base_mean is computed by Fourier.
%         data_baseline = data(ix_baseline(1):ix_baseline(2),:);
%         % data_baseline = reshape(data_baseline, [1,size(data_baseline)]); % Append redundant ch dim
%         data_baseline = reshape(data_baseline, [1,size(data_baseline,1)*size(data_baseline,2)]); % Append redundant ch dim
%         [freqs, spect_density] = calc_psd(data_baseline, sfreq, @hanning, size(data_baseline,2)/sfreq*1000, opt_fourier);
% 
%         % Init mean
%         base_mean2 = zeros(size(base_mean));
% 
%         % Down-sampling in freq domain
%         for ff=1:length(frequencies)
%             fcenter = frequencies(ff);
%             if ff==1
%                 fpre = fcenter;
%             else
%                 fpre = frequencies(ff-1);
%             end
%             if ff==length(frequencies)
%                 fpos = fcenter;
%             else
%                 fpos = frequencies(ff+1);
%             end
%             % Calc half size of a bin
%             fpre_bin = (fcenter-fpre)/2;
%             fpos_bin = (fpos-fcenter)/2;
%             % Define range of freq bin
%             fbin = [fcenter-fpre_bin fcenter+fpos_bin];
%             % Find corresponding index of spect_density
%             fbin_ix = dsearchn(freqs', fbin');
%             % Summarize within a bin
%             base_mean2(ff,:,:) = max(spect_density(fbin_ix(1):fbin_ix(2)));
%         end
% 
%         if strcmp(OPTS.normalize_type, 'z')
%             % In case of base_fourier, base_std is not defined.
%             error('z normalization is not available with baseline_fourier')
%         end
%     end

    if OPTS.baseline_long
        % Extract baseline
        data_baseline = data(ix_baseline(1):ix_baseline(2), :);
        % Concat trials
        data_baseline_long = data_baseline(:);
        % Set options for baseline wavelet
        OPTS_base = OPTS;
        OPTS_base.ix_baseline = [];
        OPTS_base.mask_coi = true;
        OPTS_base.is_mirror = false;
        OPTS_base.baseline_long = false;
        TF_base = compute_wavelet(data_baseline_long, sfreq, OPTS_base);
        base_mean_long = nanmean( TF_base, 2);
        base_std_long  = nanstd( TF_base, [], 2);
        base_mean = repmat(base_mean_long, [1,1,size(base_mean,3)]);
        base_std  = repmat(base_std_long, [1,1,size(base_std,3)]);
    end

    base_mean_rep = repmat(base_mean, [1,num_time,1]);
    base_std_rep  = repmat(base_std, [1,num_time,1]);
    switch OPTS.normalize_type
        case 'db'
            TF = 10*log10(TF ./ base_mean_rep);
        case 'divide'
            TF = TF ./ base_mean_rep;
            %TF = TF ./ base_std_rep;
        case 'z'
            TF = (TF-base_mean_rep) ./ base_std_rep;
    end
end

end % End of the function


function OPTS = check_options(OPTS)
if ~isfield(OPTS, 'average_trial') || isempty(OPTS.average_trial)
    OPTS.average_trial = true;
end

if ~isfield(OPTS, 'calc_magnitude') || isempty(OPTS.calc_magnitude)
    OPTS.calc_magnitude = false;
end

if ~isfield(OPTS, 'calc_power') || isempty(OPTS.calc_power)
    OPTS.calc_power = true;
end

if ~isfield(OPTS, 'mask_coi') || isempty(OPTS.mask_coi)
    OPTS.mask_coi = true;
end

% baseline correction if ix_baseline is specified
% and OPTS.baseline_normalization is not false
if isfield(OPTS, 'baseline_normalization') && ~isempty(OPTS.baseline_normalization)
    check_ix_baseline = OPTS.baseline_normalization;
else 
    check_ix_baseline = true;
end

if check_ix_baseline && isfield(OPTS, 'ix_baseline') && ~isempty(OPTS.ix_baseline)
    OPTS.baseline_normalization = true;
    if ~isfield(OPTS, 'normalize_type') || isempty(OPTS.normalize_type)
        OPTS.normalize_type = 'db'; % Default normalization
    end
else
    OPTS.baseline_normalization = false;
end

if ~isfield(OPTS, 'baseline_long') || isempty(OPTS.baseline_long)
    OPTS.baseline_long = false;
end

if isfield(OPTS, 'is_log_scale') && ~isempty(OPTS.is_log_scale)
    if OPTS.is_log_scale
        OPTS.freq_scale = 'log';
    else
        OPTS.freq_scale = 'linear';
    end
end

if ~isfield(OPTS, 'freq_scale') || isempty(OPTS.freq_scale)
    OPTS.freq_scale = 'linear';
end

% Frequency parameters
if ~isfield(OPTS, 'min_freq') || isempty(OPTS.min_freq)
    OPTS.min_freq = 1;
end
if ~isfield(OPTS, 'max_freq') || isempty(OPTS.max_freq)
    OPTS.max_freq = 100;
end
if ~isfield(OPTS, 'num_freq') || isempty(OPTS.num_freq)
    OPTS.num_freq = 100;
end

% Wavelet parameters
if ~isfield(OPTS, 'central_freq') || isempty(OPTS.central_freq)
    OPTS.central_freq = 1;
end
if ~isfield(OPTS, 'fwhm') || isempty(OPTS.fwhm)
    OPTS.fwhm = 3;
end

end
