function channel_info = get_channel_info(megfile, load_spec)
% Return channel information (available for a tfr.mat file)
%
% [history]
%   2024-05-16 (k_suzuki) corrected a bug when measurement of datafile is 'INFO'
%
% Copyright (C) 2011, ATR All Rights Reserved.
% License : New BSD License(see VBMEG_LICENSE.txt)

% Check load_spec
if ~exist('load_spec','var')
    load_spec = [];
end

% Get measurement
[measurement, device] = vb_load_device(megfile);
Measurement = upper(measurement);

switch Measurement
    case 'INFO'
        load(megfile, 'fileinfo');
        filename  = fileinfo.filename;

        if isfield(fileinfo, 'channel_id')
            channel_id = fileinfo.channel_id;
        elseif isfield(fileinfo, 'ActiveChannel')
            channel_info = vb_load_channel_info(filename{1});
            channel_id = channel_info.ID(fileinfo.ActiveChannel ~= 0);
        end

        if isempty(channel_id)
            warning('(%s) channel_id is empty\n', mfilename);
            channel_info = [];
            return;
        end

        channel_info = vb_load_channel_info(filename{1}, 'USER', channel_id);
        return;
        
    case {'MEG', 'EEG'}
        filename = megfile;
end

load_spec = vb_loadspec_check(load_spec, filename);

% Load MEGinfo
meginfo = vb_tfrfile_load_meginfo(filename);

% ----- narrow channel by user-specified list and active filter
[read_ch_list] = vb_meginfo_get_read_channels(meginfo, ...
  load_spec.ChannelName, load_spec.ChannelSwitch, load_spec.ActiveChannel);

if isempty(read_ch_list)
  fprintf('Read channels are empty\n');
  load_spec = [];
  return;
end
% ------- reset channel spec in loadspec
load_spec.ChannelName = read_ch_list;
load_spec.ChannelSwitch = true;

% Separate ch list into meg, ext, and ref
[read_meg_ch, read_ext_ch, read_ref_ch] = inner_separate_read_ch_list(read_ch_list, meginfo);

% channel_info = vb_megfile_make_channel_info(tfr_file, read_meg_ch, read_ext_ch, read_ref_ch);
% channel_info = vb_eeginfo_make_channel_info(EEGinfo, read_ch_list, 0);
channel_info = inner_make_channel_info(meginfo, read_meg_ch, read_ext_ch, read_ref_ch);

end % --- end of function


function [read_meg_ch, read_ext_ch, read_ref_ch] = inner_separate_read_ch_list(read_ch_list, meginfo)
% Separate read_ch_list into meg, ext, and ref

% Get channel labels
meg_labels = vb_meginfo_get_channel_label_meg(meginfo);
ext_labels = vb_meginfo_get_channel_label_extra(meginfo);
ref_labels = vb_meginfo_get_channel_label_refmg(meginfo);

% Check MEG/EEG channels
if ~isempty(meg_labels)
  [meg_ch_idx] = vb_util_get_index(meg_labels, read_ch_list);
  if ~isempty(meg_ch_idx)
    meg_ch_idx = vb_util_arrange_list(meg_ch_idx, 0);
  end
else
  meg_ch_idx = [];
end

% Check EXTRA channels
if ~isempty(ext_labels)
  [ext_ch_idx] = vb_util_get_index(ext_labels, read_ch_list);
  if ~isempty(ext_ch_idx)
    ext_ch_idx = vb_util_arrange_list(ext_ch_idx, 0);
  end
else
  ext_ch_idx = [];
end

% Check REFERENCE channels
if ~isempty(ref_labels)
  [ref_ch_idx] = vb_util_get_index(ref_labels, read_ch_list);
  if ~isempty(ref_ch_idx)
    ref_ch_idx = vb_util_arrange_list(ref_ch_idx, 0);
  end
else
  ref_ch_idx = [];
end

read_meg_ch = meg_labels(meg_ch_idx);
read_ext_ch = ext_labels(ext_ch_idx);
read_ref_ch = ref_labels(ref_ch_idx);
end % --- end of inner_separate_read_ch_list()


function channel_info = inner_make_channel_info(meginfo, meg_labels, ext_labels, ref_labels)
channel_info.Active = [];
channel_info.Name = [];
channel_info.Type = [];
channel_info.ID = [];

org_meg_ch_info = vb_info_get_channel_info(meginfo);
if isempty(org_meg_ch_info)
  warning('(%s) cannot get channel information\n', mfilename);
  return;
end

org_ext_label = vb_meginfo_get_channel_label_extra(meginfo, false, true);

if ~isempty(org_ext_label) && ~isempty(ext_labels)
  [tmp_ext_idx] = vb_util_get_index(org_ext_label, ext_labels);
else
  tmp_ext_idx = [];
end

if ~isempty(org_ext_label) && ~isempty(ref_labels)
  [tmp_ref_idx] = vb_util_get_index(org_ext_label, ref_labels);
else
  tmp_ref_idx = [];
end


if ~isempty(tmp_ext_idx) || ~isempty(tmp_ref_idx)
  if ~isfield(meginfo, 'ExtraChannelInfo') && isempty(meginfo.ExtraChannelInfo)
    warning('(%s)cannot get extra channel information\n', mfilename);
    return;
  else
    % --- correct extra channel information
    extra_info = meginfo.ExtraChannelInfo;

    % --- active of extra channel 
    ext_active = extra_info.Channel_active(tmp_ext_idx);
    ref_active = extra_info.Channel_active(tmp_ref_idx);

    % --- type of extra channel
    ext_type = extra_info.Channel_type(tmp_ext_idx);
    ref_type = extra_info.Channel_type(tmp_ref_idx);

    % --- index of extra channel
    ext_idx = extra_info.Channel_id(tmp_ext_idx);
    ref_idx = extra_info.Channel_id(tmp_ref_idx);
  end
else
  ext_active = [];
  ref_active = [];
  ext_type = [];
  ref_type = [];
  ext_idx = [];
  ref_idx = [];
end

% --- valid MEG channel index
org_meg_label = vb_meginfo_get_channel_label_meg(meginfo, false);
if ~isempty(org_meg_label) && ~isempty(meg_labels)
  [tmp_meg_idx] = vb_util_get_index(org_meg_label, meg_labels);
else
  tmp_meg_idx = [];
end

% ----- Active
org_meg_active = org_meg_ch_info.Active;
meg_active = org_meg_active(tmp_meg_idx);
channel_info.Active = [meg_active;ext_active;ref_active];

% ----- Name
org_meg_labels = org_meg_ch_info.Name;
meg_labels = org_meg_labels(tmp_meg_idx);
channel_info.Name = [meg_labels;ext_labels;ref_labels];

% ----- Type
org_meg_type = org_meg_ch_info.Type;
meg_type = org_meg_type(tmp_meg_idx);
channel_info.Type = [meg_type;ext_type;ref_type];

% ----- ID
org_meg_idx = org_meg_ch_info.ID;
meg_idx = org_meg_idx(tmp_meg_idx);
channel_info.ID = [meg_idx;ext_idx;ref_idx];
end %--- end of inner_make_channel_info()