function job_dics(dics_parm)
% Run DICS beamformer
%
% 2024/09/03 K.Suzuki
%            Nvertex is saved in currfile
% 2023/12/11 K.Suzuki
%            ch_list is added to csd_parm
%            Multiple windows are supported
% 2023/10/18 K.Suzuki 
%            First ver is made based on job_lcmv.m
% 2015/08/25 O.Yamashita

fprintf('--------------------------------------\n');
fprintf('      DICS beamformer \n');
fprintf('--------------------------------------\n');

%---------------------------------
% Compatibility with bayes_parm 
% Don't modify 
%---------------------------------

% dics_parm.megfile_baseline  = dics_parm.megfile;
dics_parm.reduce = 1;
dics_parm.Rfilt  = 0; 
dics_parm.Tperiod   = dics_parm.twin_meg(2) - dics_parm.twin_meg(1)+1;
dics_parm.Tnext     = dics_parm.Tperiod; % Time step for next period

% Check ch_list
if isempty(dics_parm.ch_list)
    channel_info = get_channel_info(dics_parm.tfrfile);
    % Set all active channels
    dics_parm.ch_list = channel_info.Name;
end

%---------------------
%  Run job
%---------------------

% Get num of windows to for estimation
num_win = length(dics_parm.currfile);

%% Compute CSD matrix using tfr file
csd_parm.fwin        = dics_parm.fwin; % freq index
csd_parm.twin_meg    = dics_parm.twin_meg; % time index of MEG/EEG
csd_parm.twin_noise  = dics_parm.twin_noise; % time index of noise
csd_parm.ch_list     = dics_parm.ch_list; % cell array of channel names
csd_parm.limit_ch_size = dics_parm.limit_ch_size; % max size of the channels that can be loaded at the same time

tfrfile = dics_parm.tfrfile;

[FREQS, CSD_meg, CSD_noise] = compute_csd(tfrfile, csd_parm);

%% Prepare leadfield matrix
[G, ix, ix_ex, W, Norient] = vb_focal_leadfield_preparation(dics_parm,1);

% Filtering leadfield with specified channels
channel_info = get_channel_info(tfrfile);
ix_ch = ismember(channel_info.Name, csd_parm.ch_list);
G{1} = G{1}(ix_ch,:);

if dics_parm.basis_normalization
    % Prepare leadfield with normalization (vertex-wise)
    % This is Array-Gain constraint
    Gnorm = vecnorm(G{1}); % after r2017b (vecnorm)
    G{1} = G{1} ./ repmat( Gnorm, [size(G{1},1),1] ); 
end

% Load brain vertex number
brain_file = fullfile(dics_parm.brainfile);
Nvertex = vb_load_cortex_info(brain_file);

%% Compute DICS and save currfile for each window
for win=1:num_win
csdm = CSD_meg{win};
csdn = CSD_noise{win};
frequencies = FREQS{win};
fwin = dics_parm.fwin(win,:);
twin_meg = dics_parm.twin_meg(win,:);
twin_noise = dics_parm.twin_noise(win,:);
currfile = dics_parm.currfile{win};

% Average CSD across freqs
CSDm = mean(csdm,3);
CSDn = mean(csdn,3);

% DICS beamformer
[Pm, Pn, F, Wm, Wn] = DICS(CSDm, G{1}, CSDn, dics_parm.cov_reg);

%------------------------------------------
% Save Curr file 
% Maybe no compatibility with .curr.mat of VBMEG
%------------------------------------------
Tstart = 1;
Tend   = 1;
Jact   = F;
ix_act = ix;
Lact   = Norient;
Varact = [];

Jbck   = [];
ix_bck = [];
Lbck   = [];

Model.nCov = [];
if dics_parm.saveifilt
Model.IF   = Wm;
Model.nIF  = [];
end

Model.Pm = Pm;
Model.Pn = Pn;
Model.F = F;
Model.Wm = Wm;
Model.Wn = Wn;
Model.CSDm = CSDm;
Model.CSDn = CSDn;
Model.G = G{1};
Model.fwin = fwin;
Model.twin_meg = twin_meg;
Model.twin_noise = twin_noise;
Model.ch_list = dics_parm.ch_list;
Model.frequency = frequencies;

MEGinfo = vb_load_meg_info(tfrfile);

fprintf('Save current reconstruction result to %s ... \n', currfile);
save(currfile, 'Jact', 'Jbck', 'ix_act', 'ix_bck', 'Varact',...
    'Tstart', 'Tend', 'MEGinfo', 'dics_parm', 'Model', 'Model', 'Nvertex');
fprintf('Done ... \n');

end

end

