function save_tfr_data(tfr, channel_info, meginfo, tfr_file_ref, tfr_file_new)
% Save input tfr data and meginfo into tfr_file_new by refferng to template tfr_file_ref
% tfr: TFR data to be saved
%      It can contain MEG/EEG, extra, and reference
% New file is save with internal setting.
%
% 2023.11.10 k_suzuki
%            Separated from vb_load_tfr_data.
%

% Classify names of channels into MEG/EEG, extra, or reference
[ch_name_meg, ch_name_ext, ch_name_ref] = inner_separate_read_ch_list(channel_info.Name, meginfo);

% Get index in tfr
[ch_idx_meg] = vb_util_get_index(channel_info.Name, ch_name_meg);
[ch_idx_ext] = vb_util_get_index(channel_info.Name, ch_name_ext);
[ch_idx_ref] = vb_util_get_index(channel_info.Name, ch_name_ref);

% Separate tfr by channel type
tfr_meg = tfr(ch_idx_meg,:,:,:);
tfr_ext = tfr(ch_idx_ext,:,:,:);
tfr_ref = tfr(ch_idx_ref,:,:,:);

% Get measurement
[measurement, device] = vb_load_device(tfr_file_ref);
Measurement = upper(measurement);

% Load template data except tfr, tfr_ext, and tfr_ref
Btmp = load(tfr_file_ref, '-regexp', '^(?!tfr$|tfr_ext$|tfr_ref$).');

% In case of EEG, convert to EEG format
parm_tmp = Btmp.tfr_parm;
freq_tmp = Btmp.frequency;
if strcmp(Measurement, 'EEG')
    Btmp = convert_eeg2meg(Btmp);
end
% meginfo = vb_load_measurement_info(tfr_file); % equivalent but reconvert is not implemented in it

% data_dir option is disabled (will be saved in a single file)
meginfo.saveman.data_dir = [];

% Get ch list of Btmp
switch Btmp.MEGinfo.Measurement
    case 'MEG'
        ch_list_meg = vb_meginfo_get_channel_label_meg(Btmp.MEGinfo);
        ch_list_ext = vb_meginfo_get_channel_label_extra(Btmp.MEGinfo);
        ch_list_ref = vb_meginfo_get_channel_label_refmg(Btmp.MEGinfo);
    case 'EEG'
        ch_list_meg = vb_meginfo_get_channel_label_meg(Btmp.MEGinfo);
        ch_list_ext = vb_eeginfo_get_channel_label_extra(Btmp.MEGinfo);
        ch_list_ref = []; % There is no ref in EEGinfo
end

% There is no ref in EEGinfo
if strcmp(Btmp.MEGinfo.Measurement,'EEG')
    ch_list_ext = [ch_list_ext; ch_list_ref];
    ch_list_ref = [];
end

% Find coressponding index in Btmp for pick and Qpick
[ch_idx_meg_Btmp] = vb_util_get_index(ch_list_meg, ch_name_meg);
[ch_idx_ref_Btmp] = vb_util_get_index(ch_list_ref, ch_name_ref); 

% Update pick and Qpick
Btmp.pick  = Btmp.pick(ch_idx_meg_Btmp,:);
Btmp.Qpick = Btmp.Qpick(ch_idx_meg_Btmp,:);
Btmp.ref_pick  = Btmp.ref_pick(ch_idx_ref_Btmp,:);
Btmp.ref_Qpick = Btmp.ref_Qpick(ch_idx_ref_Btmp,:);

% Update MEGinfo
Btmp.MEGinfo = meginfo;

% In case of EEG, save Btmp as EEG format
if strcmp(Measurement, 'EEG')
    % Note: reconvert_meg2eeg removes additional tfr fields such as tfr, frequency, and tfr_parm
    Btmp = reconvert_meg2eeg(Btmp);
    % Recover removed fields
    Btmp.tfr_parm  = parm_tmp;
    Btmp.frequency = freq_tmp;
end

% Set new tfr data
Btmp.tfr     = tfr_meg;
Btmp.tfr_ext = tfr_ext;
Btmp.tfr_ref = tfr_ref;

% Save with the same parameters
save(tfr_file_new, '-struct', 'Btmp', Btmp.tfr_parm.saveman.version, Btmp.tfr_parm.saveman.compression)

disp(['New TFR file is saved in: ' tfr_file_new])

end % --- end of the main function

function [read_meg_name, read_ext_name, read_ref_name, ...
          read_meg_idx,  read_ext_idx,  read_ref_idx] = inner_separate_read_ch_list(read_ch_list, meginfo)
% Separate read_ch_list into meg, ext, and ref
% Return name and index of read channels

% Get channel labels
switch meginfo.Measurement
    case 'MEG'
        meg_labels = vb_meginfo_get_channel_label_meg(meginfo);
        ext_labels = vb_meginfo_get_channel_label_extra(meginfo);
        ref_labels = vb_meginfo_get_channel_label_refmg(meginfo);
    case 'EEG'
        meg_labels = vb_meginfo_get_channel_label_meg(meginfo); % Use meginfo func
        ext_labels = vb_eeginfo_get_channel_label_extra(meginfo);
        ref_labels = []; % There is no ref in EEGinfo
end

% Check MEG/EEG channels
if ~isempty(meg_labels)
  [read_meg_idx] = vb_util_get_index(meg_labels, read_ch_list);
  if ~isempty(read_meg_idx)
    read_meg_idx = vb_util_arrange_list(read_meg_idx, 0);
  end
else
  read_meg_idx = [];
end

% Check EXTRA channels
if ~isempty(ext_labels)
  [read_ext_idx] = vb_util_get_index(ext_labels, read_ch_list);
  if ~isempty(read_ext_idx)
    read_ext_idx = vb_util_arrange_list(read_ext_idx, 0);
  end
else
  read_ext_idx = [];
end

% Check REFERENCE channels
if ~isempty(ref_labels)
  [read_ref_idx] = vb_util_get_index(ref_labels, read_ch_list);
  if ~isempty(read_ref_idx)
    read_ref_idx = vb_util_arrange_list(read_ref_idx, 0);
  end
else
  read_ref_idx = [];
end

read_meg_name = meg_labels(read_meg_idx);
read_ext_name = ext_labels(read_ext_idx);
read_ref_name = ref_labels(read_ref_idx);
end % --- end of inner_separate_read_ch_list()