%  Set parameters for DICS beamformer inverse solution.
%  
% - required field 
% megfile, basisfile, brainfile, currfile, twin_meg, twin_noise
%
% ***
% *** Basic Parameters
% ***
% ******************************
% - .tfrfile (.tfr.mat)
%     TFR data file to be analyzed. This is given by cell array of strings.
%    e.g. tfrfile = {'SESSION1.tfr.mat','SESSION2.tfr.mat'}
%
% - .basisfile (.basis.mat)
%     Lead field file for the high resolution model.
%    e.g. basisfile = {'SESSION1.basis.mat','SESSION2.basis.mat'}
%
% - .brainfile (.brain.mat)
%     Brain file. 
%
% - .currfile (.curr.mat)
%     Result file. Estimated currents will be saved into this file. 
%     Multiple files could be specified with multiple twin and fwin parameters.
%
% - .twin_meg
%    Time window of meg data in tfrfile used for analysis. This window 
%   is covered by several small time windows specfied by .Tperiod and
%   .Tnext. It is specified by the start and end of the sampling frame, 
%   not by the actual measurement time. 
%   Muliple windows could be specified.
%   e.g. twin_meg = [1 600]  
%   e.g. In case of multiple specification:
%        twin_meg = [1 600; 600 800; 800 100]
%
% - .twin_noise
%   Time window (start and end) used for estimation 
%   of the covariance of the observation noise
%
% - .fwin
%    Frequency window of meg data in tfrfile used for analysis. It is
%    represented as index. It is common to both MEG and noise data.
%
% - .ch_list
%    Cell array of channel names for CSD computing
%
% - .limit_ch_size 
%    Max size of the channels that can be loaded at the same time for saving RAM (default: 30ch)
%    If limit_ch_size < num_ch, do block computation in compute_csd (slow but saving RAM).
%    If it is too large, an out of memory error may occur when loading TFR data.
%
%%
% ***
% *** Advanced Parameters (not supported yet)
% ***
% ******************************
%
% - .actfile (.act.mat)
%    Activity map file. 
%
% - .areafile (.area.mat)
%    Area file. 
%
% - .act_key
%    ID of an activity map in .actfile. The specified activity map is 
%   used as the prior information to VB variance estimation.
%
% - .area_key
%    ID of an area in .areafile. Possible (focal) dipoles location are 
%   restricted to the specified area. 
%
% - .cov_reg
%   Regularization parameter for computing the inverse of covariance
%   matrix (reffered as 'lam' in LCMV.m). This value controls a tradeoff 
%   between the amount of detail in the power maps and their sensitivity to noise. 
%
% - .basis_normalization
%   If true, basis is normalized (Array-Gain constraint). 
%
%
% 2024/03/14 K.Suzuki
% * disable parameteres related to noise covariance specification (not removed)
% * add cov_reg and basis_normalization
% 2023/12/11 k_suzuki
% * ch_list was added
%   currfile was changed to cell from string
%   limit_ch_size was added
% 2023/11/11 k_suzuki
% * made based on set_lcmv_parm()
%   change : megfile -> tfrfile
%   added  : fwin_meg

function lcmv_parm = set_dics_parm(varargin)

% currfile is either 'string' or 'cell'
% but not 'string array'
ixv = find(strcmp(varargin, 'currfile')) + 1;
currfile = varargin{ixv};
if isstr(currfile)
    varargin{ixv} = {currfile};
end

parm = finputcheck(varargin, ...
              { 'brainfile'     'string'   []       []; ...
                'tfrfile'       'string'   []       []; ...
                'basisfile'     'string'   []       [];...
                'currfile'      'cell'     []       {};...
                'twin_meg'      'integer'  []       [];...
                'twin_noise'    'integer'  []       [];...
                'areafile'      'string'   []       '';...
                'actfile'       'string'   []       '';...
                'area_key'      'string'   []      'Cortex';...
                'act_key'       'string'   []      'Uniform';...
                'patch_norm'    'boolean'  []      ON;...
                'trial_average' 'boolean'  []      OFF;...
                'saveifilt'     'boolean'  []      ON;...
                'fwin'          'integer'  []      [];...
                'ch_list'       'cell'     []      {};...
                'limit_ch_size' 'integer'  []      30;...
                'cov_reg'      'real'     []       0.05;...
                'basis_normalization' 'boolean' [] OFF;...
                });

if isstr(parm), error(parm);end

% Check the consistency of num of currfile and windows
num_curr = length(parm.currfile);
num_twin_meg = size(parm.twin_meg, 1);
num_twin_noise = size(parm.twin_noise, 1);
num_fwin = size(parm.fwin, 1);
err = false;

if ~isempty(num_twin_noise) && num_twin_noise~=num_twin_meg
    err = true;
end

if num_curr~=num_twin_meg || num_curr~=num_fwin
    err = true;
end

if err, error('Check size of the parameters'); end

lcmv_parm = parm;

