function [data, channel_info, cond_id] = vb_load_active_trial_tfr_data(fileinfo)
% load tfr data from multiple files and concatenate them
% [Usage]
%   [data, channel_info, cond_id] = vb_load_active_trial_data(fileinfo)
%
% [Input]
% fileinfo : structure with file information for multiple files
% fileinfo.filename{n}  : n-th session data file names [n=1:Nsession]
% fileinfo.Nchannel     : # of total channels
% fileinfo.Nsample      : # of samples in one trial
% fileinfo.Ntotal       : # of all trials
% fileinfo.Ntrial       : # of trials for each session [1 x Nsession]
% fileinfo.session_id   : session index for each trial [1 x Ntotal]
% fileinfo.cond_id      : condition number for each trials [1 x Ntotal]
% fileinfo.ActiveChannel: Active channel list (boolean array) [Nchannel x 1]
% fileinfo.ActiveTrial  : Active trial list   (boolean array) [Ntotal x 1]
%
% [Output]
%         data : concatenated data data [Nch x Nfrequency x Nsample x Nactive]
% channel_info : <<struct>> channel information of loaded data
%              :  .Active [Nchannel x 1]
%              :  .Name   [Nchannel x 1]
%              :  .Type   [Nchannel x 1]
%              :  .ID     [Nchannel x 1]
%      cond_id : condition number for each trials [1 x Nactive]
%
%   2023-11-10 (k_suzuki) The first version is made based on vb_load_active_trial_data()
%
% Copyright (C) 2011, ATR All Rights Reserved.
% License : New BSD License(see VBMEG_LICENSE.txt)

filename  = fileinfo.filename  ;

Nsample  = fileinfo.Nsample;
Nchannel = fileinfo.Nchannel;
Ntotal   = fileinfo.Ntotal;

if ~isfield(fileinfo, 'Nfrequency')
    warning('Make info.mat using vb_make_tfr_fileinfo()')
    load(filename{1}, 'frequency');
    Nfrequency = length(frequency);
else
    Nfrequency = fileinfo.Nfrequency;
end

% Number of files
Nfile = length(filename);

% active channel & trials
if isfield(fileinfo, 'act_trial')
    act_trial = fileinfo.act_trial;
elseif isfield(fileinfo, 'ActiveTrial');
    act_trial = find(fileinfo.ActiveTrial ~= 0);
end

if isfield(fileinfo, 'channel_id');
    channel_id = fileinfo.channel_id;
elseif isfield(fileinfo, 'ActiveChannel')
    ch_info = vb_load_channel_info(filename{1});
    channel_id = ch_info.ID(fileinfo.ActiveChannel ~= 0);
end

if isempty(channel_id)
    warning('(%s) channel_id is empty\n', mfilename);
    data = [];
    channel_info = [];
    cond_id = [];
    return;
end

% --- get the name list of target channel
ch_info = vb_load_channel_info(filename{1}, 'USER', channel_id);
ch_names = ch_info.Name;

% Number of active channel & trials
Nch  = length(channel_id);
Nall = length(act_trial);

fprintf('Number of files to read: %d\n',Nfile)
fprintf('Number of all trials: %d\n',Ntotal)
fprintf('Number of all channels: %d\n',Nchannel)
fprintf('Number of trials to read: %d\n',Nall)
fprintf('Number of channels to read: %d\n',Nch)

data = zeros(Nch,Nfrequency,Nsample,Ntotal);
Nst = 0;

channel_info = [];

% Merge multiple data files
for n=1:Nfile
    loadspec.ChannelName = ch_names;
    loadspec.ActiveChannel = false;
    loadspec.ActiveTrial   = false;

    if n == 1
        % one channel_info is enough
        [tmp_data, channel_info] = vb_load_tfr_data(filename{n}, loadspec);
    else
        [tmp_data] = vb_load_tfr_data(filename{n}, loadspec);
    end

    Nt = size(tmp_data,4);
    ix = (1:Nt) + Nst; % consecutive index for concatenated data

    data(:,:,:,ix)  = tmp_data(:,:,:,:);

    Nst = Nt + Nst;
end

% Trial-filtering 
data = data(:,:,:,act_trial);
return

