function [tfr_new, channel_info, tfr_info] = vb_load_tfr_data(tfr_file, load_spec, tfr_file_new)
% Convert MEG/EEG data to time-freq representation
%
% tfr_file  : TFR file.
% load_spec : Options.
%
%
% 2023.11.03 k_suzuki
%            Beta version.
% 2023.11.10 k_suzuki
%            segmentation and saving are implemented as different functions
% 2023.12.11 k_suzuki
%            progress message was made when loading externally saved channels
% 2023.12.22 k_suzuki
%            In case of {'hilbert', 'bandpass'}, freq_band_table will be
%            loaded in tfr_info
%

% Check load_spec
if ~exist('load_spec','var')
    load_spec = [];
end

% Get measurement
[measurement, device] = vb_load_device(tfr_file);
Measurement = upper(measurement);

switch Measurement

case 'INFO'
% In case of info.mat file, tfr_file_new is ignored
load(tfr_file, 'fileinfo');

% Priority: load_spec > fileinfo

% --- support loadspec --- %
if ~isempty(load_spec)
    tmp_file = fileinfo.filename{1};

    % ----- check loadspec
    load_spec = vb_loadspec_check(load_spec, tmp_file);

    % ----- narrow channel by user-specified list and active filter
    meginfo = vb_load_meg_info(tmp_file, 4);
    [read_ch, idx] = vb_meginfo_get_read_channels(meginfo, ...
        load_spec.ChannelName, load_spec.ChannelSwitch, ...
        load_spec.ActiveChannel);

    % --- convert loadspec setting to fileinfo setting
    ch_info = vb_info_get_channel_info(meginfo, 2);
    fileinfo.channel_id = ch_info.ID(idx);
end
% --- support loadspec --- %
[tfr_new, channel_info] = vb_load_active_trial_tfr_data(fileinfo);

% Get tfr info
tfr_info = inner_tfr_info(fileinfo.filename{1});

% --- End of case 'INFO' ---

case {'MEG', 'EEG'}
load_spec = vb_loadspec_check(load_spec, tfr_file); % If load_spec is empty, init it

% Check new file
if ~exist('tfr_file_new','var')
    tfr_file_new = [];
end

% Get tfr and channel info
tfr_info = inner_tfr_info(tfr_file);
channel_info = inner_channel_info(tfr_file, load_spec); % Return actie ch

%% Load meginfo and set options
meginfo = vb_tfrfile_load_meginfo(tfr_file);

% If saveman.data_dir is not specified, save_internal became true in vb_make_tfr_data
% Then, TFR of all ch were saved in a single file
% Otherwise, TFR of channels were separately saved in different files
if ~isfield(meginfo.saveman,'data_dir') || isempty(meginfo.saveman.data_dir)
    save_internal = true;
else
    save_internal = false;
    % TFR of each channel was separately saved in real_dir
    [base_dir] = fileparts(tfr_file);
    real_dir = fullfile(base_dir, meginfo.saveman.data_dir);
end

% Separate ch list into meg, ext, and ref
read_ch_list = channel_info.Name;
[read_meg_ch, read_ext_ch, read_ref_ch] = inner_separate_read_ch_list(read_ch_list, meginfo);

num_time  = vb_info_get_sample_number(meginfo);
num_freq  = length(tfr_info.frequency);
num_trial = vb_info_get_Nrepeat(meginfo); 

%% Set loadspec for 1ch files
if ~save_internal
    ls_1ch = [];
    ls_1ch.ChannelType = 'ALL';
    ls_1ch.ActiveChannel = false; % Disable ActiveChannel flag
    ls_1ch.ActiveTrial   = false; % Disable ActiveTrial flag
end

%% Start loading tfr data
% Get MEG/EEG channels
if ~isempty(read_meg_ch)
    num_ch = length(read_meg_ch);
    % Get index to be loaded
    ch_list = meginfo.ChannelInfo.Name;
    read_ch_ix = vb_util_get_index(ch_list, read_meg_ch);

    if save_internal
        load(tfr_file, 'tfr');
        tfr_meg = tfr(read_ch_ix, :, :, :);
    else
        tfr_meg = zeros(num_ch, num_freq, num_time, num_trial);
        % Load channel files
        fprintf('Loading MEG/EEG channels ...')
        for cc=1:num_ch
            if rem(cc,10)==0 && cc~=num_ch
                fprintf([num2str(cc) '...'])
            end
            chix = read_ch_ix(cc);
            chname = read_meg_ch{cc};
            tfr_file_ch = fullfile(real_dir, [chname '.tfr.mat']);
            tfr_ch = vb_load_tfr_data(tfr_file_ch, ls_1ch);
            tfr_meg(cc,:,:,:) = tfr_ch;
            if rem(cc,100)==0 && cc~=num_ch
                fprintf('\n')
            end
        end
        fprintf([num2str(cc) '\n'])
    end
else
    tfr_meg = [];
end

% Get extra channels
if ~isempty(read_ext_ch)
    num_ch = length(read_ext_ch);
    % Get index to be loaded
    ch_list = meginfo.ExtraChannelInfo.Channel_name;
    read_ch_ix = vb_util_get_index(ch_list, read_ext_ch);

    if save_internal
        load(tfr_file, 'tfr_ext');
        tfr_ext = tfr_ext(read_ch_ix, :, :, :);
    else
        tfr_ext = zeros(num_ch, num_freq, num_time, num_trial);
        % Load channel files
        fprintf('Loading EXTRA channels ...')
        for cc=1:num_ch
            if rem(cc,10)==0 && cc~=num_ch
                fprintf([num2str(cc) '...'])
            end
            chix = read_ch_ix(cc);
            chname = read_ext_ch{cc};
            tfr_file_ch = fullfile(real_dir, [chname '.tfr.mat']);
            tfr_ch = vb_load_tfr_data(tfr_file_ch, ls_1ch);
            tfr_ext(cc,:,:,:) = tfr_ch;
            if rem(cc,100)==0 && cc~=num_ch
                fprintf('\n')
            end
        end
        fprintf([num2str(cc) '\n'])
    end
else
    tfr_ext = [];
end

% Get reference channels
if ~isempty(read_ref_ch)
    num_ch = length(read_ref_ch);
    % Get index to be loaded
    ch_list = meginfo.ExtraChannelInfo.Channel_name;
    read_ch_ix = vb_util_get_index(ch_list, read_ref_ch);

    if save_internal
        load(tfr_file, 'tfr_ref');
        tfr_ref = tfr_ref(read_ch_ix, :, :, :);
    else
        tfr_ref = zeros(num_ch, num_freq, num_time, num_trial);
        % Load channel files
        fprintf('Loading REFERENCE channels ...')
        for cc=1:num_ch
            if rem(cc,10)==0 && cc~=num_ch
                fprintf([num2str(cc) '...'])
            end
            chix = read_ch_ix(cc);
            chname = read_ref_ch{cc};
            tfr_file_ch = fullfile(real_dir, [chname '.tfr.mat']);
            tfr_ch = vb_load_tfr_data(tfr_file_ch, ls_1ch);
            tfr_ref(cc,:,:,:) = tfr_ch;
            if rem(cc,100)==0 && cc~=num_ch
                fprintf('\n')
            end
        end
        fprintf([num2str(cc) '\n'])
    end
else
    tfr_ref = [];
end

%% Update fields of meginfo related to channels
meginfo = inner_update_meginfo_ch(meginfo, channel_info);

%% Trial-segmentation and update meginfo
tfr_meg = segment_tfr_data(tfr_meg, meginfo, load_spec);
tfr_ext = segment_tfr_data(tfr_ext, meginfo, load_spec);
[tfr_ref, meginfo] = segment_tfr_data(tfr_ref, meginfo, load_spec); % Update meginfo as well

% Update time and trial
num_time  = vb_info_get_sample_number(meginfo);
num_trial = vb_info_get_Nrepeat(meginfo); 
time = vb_index_to_time(1:num_time, meginfo) / 1000;
tfr_info.time = time;

%% Trial-filtering and update meginfo
% Note: If trial-segmentation is performed, meginfo.ActiveTrial is initialized

% Get active trial index 
read_trial_ix = get_active_trial_index(load_spec, meginfo);

if ~isempty(tfr_meg)
    tfr_meg = tfr_meg(:,:,:,read_trial_ix);
end
if ~isempty(tfr_ext)
    tfr_ext = tfr_ext(:,:,:,read_trial_ix);
end
if ~isempty(tfr_ref)
    tfr_ref = tfr_ref(:,:,:,read_trial_ix);
end

% Update meginfo
meginfo.ActiveTrial = meginfo.ActiveTrial(read_trial_ix);
meginfo.Trial = meginfo.Trial(read_trial_ix);

%% Concatinate MEG/EEG, Extra, and Reference channels
% Concat along ch dim
tfr_new = cat(1, tfr_meg, tfr_ext, tfr_ref);

% channel_info.Name is already arranged by [read_meg_ch; read_ext_ch; read_ref_ch]
% in inner_channel_info()

%% Save in new file
if ~isempty(tfr_file_new)
    save_tfr_data(tfr_new, channel_info, meginfo, tfr_file, tfr_file_new);
end
% --- End of case 'MEG' or 'EEG' ---

end % --- End of Measurement switch ---

end % --- End of the main function ---

function tfr_info = inner_tfr_info(tfr_file)
time_info = struct;

% Calculate time
meginfo = vb_load_meg_info(tfr_file);
Nsample = vb_meginfo_get_sample_number(meginfo);
time = vb_index_to_time(1:Nsample, meginfo) / 1000;

% Create time_info
time_info.time = time;
time_info.sample_frequency = vb_meginfo_get_sampling_frequency(meginfo);

% Append frequency and method
load(tfr_file, 'frequency', 'tfr_parm');
tfr_info = time_info;
tfr_info.frequency = frequency;
tfr_info.method = tfr_parm.method;
if strcmp(tfr_info.method,'hilbert') || strcmp(tfr_info.method,'bandpass')
    tfr_info.freq_band_table = tfr_parm.freq_band_table;
end
end % --- end of inner_tfr_info()

function channel_info = inner_channel_info(tfr_file, load_spec)
% Load MEGinfo
meginfo = vb_tfrfile_load_meginfo(tfr_file);

% ----- narrow channel by user-specified list and active filter
[read_ch_list] = vb_meginfo_get_read_channels(meginfo, ...
  load_spec.ChannelName, load_spec.ChannelSwitch, load_spec.ActiveChannel);

if isempty(read_ch_list)
  fprintf('Read channels are empty\n');
  load_spec = [];
  return;
end
% ------- reset channel spec in loadspec
load_spec.ChannelName = read_ch_list;
load_spec.ChannelSwitch = true;

% Separate ch list into meg, ext, and ref
[read_meg_ch, read_ext_ch, read_ref_ch] = inner_separate_read_ch_list(read_ch_list, meginfo);

% channel_info = vb_megfile_make_channel_info(tfr_file, read_meg_ch, read_ext_ch, read_ref_ch);
% channel_info = vb_eeginfo_make_channel_info(EEGinfo, read_ch_list, 0);
channel_info = inner_make_channel_info(meginfo, read_meg_ch, read_ext_ch, read_ref_ch);

end % --- end of inner_channel_info()

function channel_info = inner_make_channel_info(meginfo, meg_labels, ext_labels, ref_labels)
channel_info.Active = [];
channel_info.Name = [];
channel_info.Type = [];
channel_info.ID = [];

org_meg_ch_info = vb_info_get_channel_info(meginfo);
if isempty(org_meg_ch_info)
  warning('(%s) cannot get channel information\n', mfilename);
  return;
end

org_ext_label = vb_meginfo_get_channel_label_extra(meginfo, false, true);

if ~isempty(org_ext_label) && ~isempty(ext_labels)
  [tmp_ext_idx] = vb_util_get_index(org_ext_label, ext_labels);
else
  tmp_ext_idx = [];
end

if ~isempty(org_ext_label) && ~isempty(ref_labels)
  [tmp_ref_idx] = vb_util_get_index(org_ext_label, ref_labels);
else
  tmp_ref_idx = [];
end


if ~isempty(tmp_ext_idx) || ~isempty(tmp_ref_idx)
  if ~isfield(meginfo, 'ExtraChannelInfo') && isempty(meginfo.ExtraChannelInfo)
    warning('(%s)cannot get extra channel information\n', mfilename);
    return;
  else
    % --- correct extra channel information
    extra_info = meginfo.ExtraChannelInfo;

    % --- active of extra channel 
    ext_active = extra_info.Channel_active(tmp_ext_idx);
    ref_active = extra_info.Channel_active(tmp_ref_idx);

    % --- type of extra channel
    ext_type = extra_info.Channel_type(tmp_ext_idx);
    ref_type = extra_info.Channel_type(tmp_ref_idx);

    % --- index of extra channel
    ext_idx = extra_info.Channel_id(tmp_ext_idx);
    ref_idx = extra_info.Channel_id(tmp_ref_idx);
  end
else
  ext_active = [];
  ref_active = [];
  ext_type = [];
  ref_type = [];
  ext_idx = [];
  ref_idx = [];
end

% --- valid MEG channel index
org_meg_label = vb_meginfo_get_channel_label_meg(meginfo, false);
if ~isempty(org_meg_label) && ~isempty(meg_labels)
  [tmp_meg_idx] = vb_util_get_index(org_meg_label, meg_labels);
else
  tmp_meg_idx = [];
end

% ----- Active
org_meg_active = org_meg_ch_info.Active;
meg_active = org_meg_active(tmp_meg_idx);
channel_info.Active = [meg_active;ext_active;ref_active];

% ----- Name
org_meg_labels = org_meg_ch_info.Name;
meg_labels = org_meg_labels(tmp_meg_idx);
channel_info.Name = [meg_labels;ext_labels;ref_labels];

% ----- Type
org_meg_type = org_meg_ch_info.Type;
meg_type = org_meg_type(tmp_meg_idx);
channel_info.Type = [meg_type;ext_type;ref_type];

% ----- ID
org_meg_idx = org_meg_ch_info.ID;
meg_idx = org_meg_idx(tmp_meg_idx);
channel_info.ID = [meg_idx;ext_idx;ref_idx];
end %--- end of inner_make_channel_info()

function [read_meg_name, read_ext_name, read_ref_name, ...
          read_meg_idx,  read_ext_idx,  read_ref_idx] = inner_separate_read_ch_list(read_ch_list, meginfo)
% Separate read_ch_list into meg, ext, and ref
% Return name and index of read channels

% Get channel labels
switch meginfo.Measurement
    case 'MEG'
        meg_labels = vb_meginfo_get_channel_label_meg(meginfo);
        ext_labels = vb_meginfo_get_channel_label_extra(meginfo);
        ref_labels = vb_meginfo_get_channel_label_refmg(meginfo);
    case 'EEG'
        meg_labels = vb_meginfo_get_channel_label_meg(meginfo); % Use meginfo func
        ext_labels = vb_eeginfo_get_channel_label_extra(meginfo);
        ref_labels = []; % There is no ref in EEGinfo
end

% Check MEG/EEG channels
if ~isempty(meg_labels)
  [read_meg_idx] = vb_util_get_index(meg_labels, read_ch_list);
  if ~isempty(read_meg_idx)
    read_meg_idx = vb_util_arrange_list(read_meg_idx, 0);
  end
else
  read_meg_idx = [];
end

% Check EXTRA channels
if ~isempty(ext_labels)
  [read_ext_idx] = vb_util_get_index(ext_labels, read_ch_list);
  if ~isempty(read_ext_idx)
    read_ext_idx = vb_util_arrange_list(read_ext_idx, 0);
  end
else
  read_ext_idx = [];
end

% Check REFERENCE channels
if ~isempty(ref_labels)
  [read_ref_idx] = vb_util_get_index(ref_labels, read_ch_list);
  if ~isempty(read_ref_idx)
    read_ref_idx = vb_util_arrange_list(read_ref_idx, 0);
  end
else
  read_ref_idx = [];
end

read_meg_name = meg_labels(read_meg_idx);
read_ext_name = ext_labels(read_ext_idx);
read_ref_name = ref_labels(read_ref_idx);
end % --- end of inner_separate_read_ch_list()

function meginfo = inner_update_meginfo_ch(meginfo, channel_info)
% Update meginfo by selected channels
% channel_info.Name: Name of selected channels containing all types of channels
%                    (MEG/EEG, reference, extra)
% Note: Index in fields of meginfo is sorted by the order of channel_info.Name

chnames = channel_info.Name;

% Find coressponding index in Btmp
ch_list_meg = meginfo.ChannelInfo.Name; 
ch_list_ext = meginfo.ExtraChannelInfo.Channel_name; % Contains both ext and ref
ch_ix_meg = []; % ix in all data ch (ch_list_meg)
ch_ix_ext = []; % ix in all extra ch (ch_list_ext)
for cc=1:length(chnames)
    ch_name = chnames(cc);
    ch_ix = vb_util_get_index(ch_list_meg, ch_name);
    if ~isempty(ch_ix)
        ch_ix_meg = [ch_ix_meg; ch_ix];
    end
    ch_ix = vb_util_get_index(ch_list_ext, ch_name);
    if ~isempty(ch_ix)
        ch_ix_ext = [ch_ix_ext; ch_ix];
    end
end

% Update fields of MEG/EEG ch
if ~isempty(ch_ix_meg)
    meginfo.ActiveChannel = meginfo.ActiveChannel(ch_ix_meg);
    meginfo.MEGch_id      = meginfo.MEGch_id(ch_ix_meg);
    meginfo.MEGch_name    = meginfo.MEGch_name(ch_ix_meg);
    meginfo.ChannelInfo.Active = meginfo.ChannelInfo.Active(ch_ix_meg);
    meginfo.ChannelInfo.ID     = meginfo.ChannelInfo.ID(ch_ix_meg);
    meginfo.ChannelInfo.Name   = meginfo.ChannelInfo.Name(ch_ix_meg);
    meginfo.ChannelInfo.Type   = meginfo.ChannelInfo.Type(ch_ix_meg);
    meginfo.Nchannel = length(ch_ix_meg); % It does not include refmg and bexp_ext
else
    meginfo.ActiveChannel = [];
    meginfo.MEGch_name    = [];
    meginfo.MEGch_id      = [];
    meginfo.ChannelInfo.Active = [];
    meginfo.ChannelInfo.ID     = [];
    meginfo.ChannelInfo.Name   = [];
    meginfo.ChannelInfo.Type   = [];
    meginfo.Nchannel = 0; % It does not include refmg and bexp_ext
end
% Update fields of extra ch
if ~isempty(ch_ix_ext)
    meginfo.ExtraChannelInfo.Channel_active = meginfo.ExtraChannelInfo.Channel_active(ch_ix_ext);
    meginfo.ExtraChannelInfo.Channel_id     = meginfo.ExtraChannelInfo.Channel_id(ch_ix_ext);
    meginfo.ExtraChannelInfo.Channel_name   = meginfo.ExtraChannelInfo.Channel_name(ch_ix_ext);
    meginfo.ExtraChannelInfo.Channel_type   = meginfo.ExtraChannelInfo.Channel_type(ch_ix_ext);
else
    meginfo.ExtraChannelInfo.Channel_active = [];
    meginfo.ExtraChannelInfo.Channel_id     = [];
    meginfo.ExtraChannelInfo.Channel_name   = [];
    meginfo.ExtraChannelInfo.Channel_type   = [];
end
end % --- end of inner_update_meginfo_ch()

