function vb_make_tfr_data(data_file, tfr_file, tfr_parm)
% Convert MEG/EEG data to time-freq representation
%
% data_file : Input EEG/MEG file.
% tfr_file  : Output TFR file.
% tfr_parm  : Options.
%
% Note: For method={'hilbert', 'bandpass'}, mask_coi option is not supported yet.
%       coi = cone of influence
%
% 2023.11.03 k_suzuki
%            Beta version.
% 2023.11.09 k_suzuki
%            active ch flag was set to 1 for single ch file.

disp(mfilename)

tfr_parm = inner_check_parm(tfr_parm);

[base_dir] = fileparts(tfr_file);
% New data will be saved in base_dir
if ~exist(base_dir, 'dir')
    mkdir(base_dir);
end

% If saveman.data_dir is not specified, save_internal becomes true
% Then, TFR of all ch will be saved in a single file (file size becomes huge)
% Otherwise, TFR of channels will be separately saved in different files
if isempty(tfr_parm.saveman.data_dir)
    save_internal = true;
else
    save_internal = false;
    % TFR of each channel will be separately saved in real_dir
    real_dir = fullfile(base_dir, tfr_parm.saveman.data_dir);
    if ~exist(real_dir, 'dir')
        mkdir(real_dir);
    end
    % tfr_parm for 1ch files
    tfr_parm_1ch = tfr_parm;
    tfr_parm_1ch.saveman.data_dir = []; % 1ch files contain tfr data
end

% Options for save function
save_ver  = tfr_parm.saveman.version;
save_comp = tfr_parm.saveman.compression;

%% Set time-frequency computation method
switch tfr_parm.method
    case 'wavelet'
        compute_tf = @compute_wavelet;
    case 'hilbert'
        compute_tf = @compute_bandpass;
        % Apply Hilbert transformation after bandpass filtering
        tfr_parm.apply_hilbert = true; 
        % If freq_band_table is empty, the default frequency bands have been set in inner_check_parm
        tfr_parm.num_freq = size(tfr_parm.freq_band_table,1);
    case 'bandpass'
        compute_tf = @compute_bandpass;
        % If freq_band_table is empty, the default frequency bands have been set in inner_check_parm
        tfr_parm.num_freq = size(tfr_parm.freq_band_table,1);
end

disp(['Selected method: ' tfr_parm.method])
    
%% Load data
info = vb_load_measurement_info(data_file);
Measurement = vb_info_get_measurement(info);
switch Measurement
    case 'MEG'
        B = load(data_file);
    case 'EEG'
        E = load(data_file);
        B = convert_eeg2meg(E);
end

% Check saveman field
if isfield(B.MEGinfo, 'saveman') && isfield(B.MEGinfo.saveman, 'data_dir') && ~isempty(B.MEGinfo.saveman.data_dir)
    % Data is saved in external files
    loadspec = [];
    loadspec.ChannelType = Measurement;
    loadspec.ActiveChannel = false; % Load all channels including inactive channels
    loadspec.ActiveTrial   = false; % Load all trial
    B.bexp = vb_load_meg_data(data_file, loadspec);

    if strcmp(Measurement, 'MEG')
        % Load reference ch
        loadspec = [];
        loadspec.ActiveChannel = false; % Load all channels including inactive channels
        loadspec.ActiveTrial   = false; % Load all trial
        loadspec.ChannelType = 'REFERENCE';
        B.refmg = vb_load_meg_data(data_file, loadspec);
    else
        % EEG does not have ref
        B.refmg = [];
    end

    % Load extra ch
    loadspec = [];
    loadspec.ActiveChannel = false; % Load all channels including inactive channels
    loadspec.ActiveTrial   = false; % Load all trial
    loadspec.ChannelType = 'EXTRA';
    B.bexp_ext = vb_load_meg_data(data_file,loadspec);
end

sfreq = B.MEGinfo.SampleFreq;
[num_ch, num_time, num_trial] = size(B.bexp);
[num_ch_ref, ~, ~] = size(B.refmg);
[num_ch_ext, ~, ~] = size(B.bexp_ext);

%% Check channel names to be removed
% Classify ExtraChannelInfo into 'ref' or 'ext'
[~,name_ext,name_ref] = inner_separate_read_ch_list(B.MEGinfo.ExtraChannelInfo.Channel_name, B.MEGinfo);
ix_ext = vb_util_get_index(B.MEGinfo.ExtraChannelInfo.Channel_name, name_ext);
ix_ref = vb_util_get_index(B.MEGinfo.ExtraChannelInfo.Channel_name, name_ref);

% MEG/EEG
chix_active = true(num_ch, 1);
ch_names = B.MEGinfo.ChannelInfo.Name;
if ~isempty(tfr_parm.remove_ch)
    for rch=1:length(tfr_parm.remove_ch)
        rch_name = tfr_parm.remove_ch{rch};
        rchix = strcmp(ch_names, rch_name);
        chix_active(rchix) = false;
    end
end

% Reference
chix_ref_active = true(num_ch_ref, 1);
ch_names = B.MEGinfo.ExtraChannelInfo.Channel_name(ix_ref);
if ~isempty(tfr_parm.remove_ch_ref) && ~isempty(B.refmg)
    for rch=1:length(tfr_parm.remove_ch_ref)
        rch_name = tfr_parm.remove_ch_ref{rch};
        rchix = strcmp(ch_names, rch_name);
        chix_ref_active(rchix) = false;
    end
end

% Extra
chix_ext_active = true(num_ch_ext, 1);
ch_names = B.MEGinfo.ExtraChannelInfo.Channel_name(ix_ext);
if ~isempty(tfr_parm.remove_ch_ext) && ~isempty(B.bexp_ext)
    for rch=1:length(tfr_parm.remove_ch_ext)
        rch_name = tfr_parm.remove_ch_ext{rch};
        rchix = strcmp(ch_names, rch_name);
        chix_ext_active(rchix) = false;
    end
end

%% Remove specified channels
Bnew = B;
% MEG/EEG data
Bnew.bexp  = B.bexp(chix_active,:,:);
Bnew.pick  = B.pick(chix_active,:);
Bnew.Qpick = B.Qpick(chix_active,:);
Bnew.MEGinfo.ActiveChannel = B.MEGinfo.ActiveChannel(chix_active);
Bnew.MEGinfo.MEGch_name    = B.MEGinfo.MEGch_name(chix_active);
Bnew.MEGinfo.MEGch_id      = B.MEGinfo.MEGch_id(chix_active);
Bnew.MEGinfo.sensor_weight = [];
Bnew.MEGinfo.Nchannel      = sum(chix_active);
Bnew.MEGinfo.ChannelInfo.Active = B.MEGinfo.ChannelInfo.Active(chix_active);
Bnew.MEGinfo.ChannelInfo.ID     = B.MEGinfo.ChannelInfo.ID(chix_active);
Bnew.MEGinfo.ChannelInfo.Name   = B.MEGinfo.ChannelInfo.Name(chix_active);
Bnew.MEGinfo.ChannelInfo.Type   = B.MEGinfo.ChannelInfo.Type(chix_active);
[num_ch, num_time, num_trial] = size(Bnew.bexp);

% Reference and Extra
ix_remove = [ix_ref(~chix_ref_active); ix_ext(~chix_ext_active)]; % Concat removing indices
Bnew.refmg     = B.refmg(chix_ref_active,:,:);
Bnew.ref_pick  = B.ref_pick(chix_ref_active,:);
Bnew.ref_Qpick = B.ref_Qpick(chix_ref_active,:);
Bnew.bexp_ext  = B.bexp_ext(chix_ext_active,:,:);
Bnew.MEGinfo.ExtraChannelInfo.Channel_active(ix_remove) = [];
Bnew.MEGinfo.ExtraChannelInfo.Channel_id(ix_remove)     = [];
Bnew.MEGinfo.ExtraChannelInfo.Channel_name(ix_remove)   = [];
Bnew.MEGinfo.ExtraChannelInfo.Channel_type(ix_remove)   = [];
[num_ch_ref, ~, ~] = size(Bnew.refmg);
[num_ch_ext, ~, ~] = size(Bnew.bexp_ext);
Bnew.MEGinfo.saveman = []; % Init saveman of original MEG/EEG data

B = Bnew;

%% Save in a single file or separated files?
if save_internal
    tfr     = zeros(num_ch, tfr_parm.num_freq, num_time, num_trial);
    tfr_ref = zeros(num_ch_ref, tfr_parm.num_freq, num_time, num_trial);
    tfr_ext = zeros(num_ch_ext, tfr_parm.num_freq, num_time, num_trial);
    B.MEGinfo.saveman.data_dir = [];
else
    tfr = [];
    tfr_ref = [];
    tfr_ext = [];
    % Update saveman
    B.MEGinfo.saveman.data_dir = tfr_parm.saveman.data_dir;
end

%% Compute TFR
% Prepare template by removing unnecessary fields
Btmp = inner_rmfield(B, {'bexp','refmg','bexp_ext'});
Btmp.ref_pick  = [];
Btmp.ref_Qpick = [];
Btmp.MEGinfo.sensor_weight = [];
Btmp.MEGinfo.ExtraChannelInfo.Channel_active = [];
Btmp.MEGinfo.ExtraChannelInfo.Channel_id     = [];
Btmp.MEGinfo.ExtraChannelInfo.Channel_name   = [];
Btmp.MEGinfo.ExtraChannelInfo.Channel_type   = [];

disp('Start making TFR file...')
for ch=1:num_ch
    %data_1ch = squeeze(B.bexp(ch,:,:)); % It fails when #trial=1
    data_1ch = zeros(num_time, num_trial);
    data_1ch(:,:) = B.bexp(ch,:,:); % It retains the shape (#time*#trial)
    % Convert data into TFR
    [tfr_1ch, frequency] = compute_tf(data_1ch, sfreq, tfr_parm);
    % Explicitly state the first dim is channel
    tfr_1ch = reshape(tfr_1ch, [1, size(tfr_1ch)]);
    % Save converted TFR
    if save_internal
        tfr(ch,:,:,:) = tfr_1ch;
    else
        % 1ch TFR is saved in a file separately
        Bch = Btmp; 
        fname_ch = fullfile(real_dir, [Bch.MEGinfo.MEGch_name{ch} '.tfr.mat']);
        Bch.pick  = Bch.pick(ch,:);
        Bch.Qpick = Bch.Qpick(ch,:);
        % Bch.MEGinfo.ActiveChannel = Btmp.MEGinfo.ActiveChannel(ch);
        Bch.MEGinfo.ActiveChannel = true;
        Bch.MEGinfo.MEGch_name    = Btmp.MEGinfo.MEGch_name(ch);
        Bch.MEGinfo.MEGch_id      = Btmp.MEGinfo.MEGch_id(ch);
        Bch.MEGinfo.Nchannel      = 1;
        % Bch.MEGinfo.ChannelInfo.Active = Btmp.MEGinfo.ChannelInfo.Active(ch);
        Bch.MEGinfo.ChannelInfo.Active = true;
        Bch.MEGinfo.ChannelInfo.ID     = Btmp.MEGinfo.ChannelInfo.ID(ch);
        Bch.MEGinfo.ChannelInfo.Name   = Btmp.MEGinfo.ChannelInfo.Name(ch);
        Bch.MEGinfo.ChannelInfo.Type   = Btmp.MEGinfo.ChannelInfo.Type(ch);
        % saveman is disabled in 1ch file
        Bch.MEGinfo.saveman.data_dir = [];
        if strcmp(Measurement, 'EEG')
            % Note: reconvert_meg2eeg removes additional tfr fields such as tfr, frequency, or tfr_parm
            Bch = reconvert_meg2eeg(Bch);
            if isfield(Bch.EEGinfo.ChannelInfo, 'PhysicalUnit')
                Bch.EEGinfo.ChannelInfo.PhysicalUnit = Bch.EEGinfo.ChannelInfo.PhysicalUnit(ch);
            end
        end
        % Append TFR data
        Bch.tfr       = tfr_1ch;
        Bch.frequency = frequency;
        Bch.tfr_parm  = tfr_parm_1ch;
        % Save ch file
        save(fname_ch, '-struct', 'Bch', save_ver, save_comp)
    end

    if mod(ch,10)==0 && ch~=num_ch
        disp([num2str(ch) '/' num2str(num_ch) ' processed...'])
    end
end

disp([num2str(num_ch) '/' num2str(num_ch) ' completed...'])

%% Process reference-channels
% Classify ExtraChannelInfo into 'ref' or 'ext' 
% (after ch removing)
[~,name_ext,name_ref] = inner_separate_read_ch_list(B.MEGinfo.ExtraChannelInfo.Channel_name, B.MEGinfo);
ix_ext = vb_util_get_index(B.MEGinfo.ExtraChannelInfo.Channel_name, name_ext);
ix_ref = vb_util_get_index(B.MEGinfo.ExtraChannelInfo.Channel_name, name_ref);

% Prepare template for ref by removing unnecessary fields
Btmp = inner_rmfield(B, {'bexp','refmg','bexp_ext'});
Btmp.pick  = [];
Btmp.Qpick = [];
Btmp.MEGinfo.sensor_weight = [];
Btmp.MEGinfo.ActiveChannel = [];
Btmp.MEGinfo.MEGch_name    = [];
Btmp.MEGinfo.MEGch_id      = [];
Btmp.MEGinfo.ChannelInfo.Active = [];
Btmp.MEGinfo.ChannelInfo.ID     = [];
Btmp.MEGinfo.ChannelInfo.Name   = [];
Btmp.MEGinfo.ChannelInfo.Type   = [];
Btmp.MEGinfo.Nchannel = 0; % It does not include refmg and bexp_ext

if tfr_parm.make_ref && ~isempty(B.refmg)
disp('Start processing reference channels...')
for ch=1:num_ch_ref
    data_1ch = zeros(num_time, num_trial);
    data_1ch(:,:) = B.refmg(ch,:,:); % It retains the shape (#time*#trial)
    % Convert data into TFR
    [tfr_1ch, frequency] = compute_tf(data_1ch, sfreq, tfr_parm);
    % Explicitly state the first dim is channel
    tfr_1ch = reshape(tfr_1ch, [1, size(tfr_1ch)]);
    % Save converted TFR
    if save_internal
        tfr_ref(ch,:,:,:) = tfr_1ch;
    else
        % 1ch TFR is saved in a file separately
        Bch = Btmp; 
        fname_ch = fullfile(real_dir, [Bch.MEGinfo.ExtraChannelInfo.Channel_name{ix_ref(ch)} '.tfr.mat']);
        Bch.ref_pick  = Bch.ref_pick(ch,:);
        Bch.ref_Qpick = Bch.ref_Qpick(ch,:);
        % Bch.MEGinfo.ExtraChannelInfo.Channel_active = Bch.MEGinfo.ExtraChannelInfo.Channel_active(ix_ref2(ch)); 
        Bch.MEGinfo.ExtraChannelInfo.Channel_active = true; 
        Bch.MEGinfo.ExtraChannelInfo.Channel_id     = Bch.MEGinfo.ExtraChannelInfo.Channel_id(ix_ref(ch)); 
        Bch.MEGinfo.ExtraChannelInfo.Channel_name   = Bch.MEGinfo.ExtraChannelInfo.Channel_name(ix_ref(ch)); 
        Bch.MEGinfo.ExtraChannelInfo.Channel_type   = Bch.MEGinfo.ExtraChannelInfo.Channel_type(ix_ref(ch)); 
        % saveman is disabled in 1ch file
        Bch.MEGinfo.saveman.data_dir = [];
        if strcmp(Measurement, 'EEG')
            % Note: reconvert_meg2eeg removes additional tfr fields such as tfr, frequency, or tfr_parm
            Bch = reconvert_meg2eeg(Bch);
        end
        % Append TFR data
        Bch.tfr_ref   = tfr_1ch;
        Bch.frequency = frequency;
        Bch.tfr_parm  = tfr_parm_1ch;
        % Save ch file
        save(fname_ch, '-struct', 'Bch', save_ver, save_comp)
    end

    if mod(ch,10)==0 && ch~=num_ch_ref
        disp([num2str(ch) '/' num2str(num_ch_ref) ' processed(ref)...'])
    end
end
disp([num2str(num_ch_ref) '/' num2str(num_ch_ref) ' completed(ref)...'])
end

%% Process extra-channels
% Prepare template for ext by removing unnecessary fields from tmp for ref
Btmp.ref_pick  = [];
Btmp.ref_Qpick = [];

if tfr_parm.make_ext && ~isempty(B.bexp_ext)
disp('Start processing extra channels...')
for ch=1:num_ch_ext
    data_1ch = zeros(num_time, num_trial);
    data_1ch(:,:) = B.bexp_ext(ch,:,:); % It retains the shape (#time*#trial)
    % Convert data into TFR
    [tfr_1ch, frequency] = compute_tf(data_1ch, sfreq, tfr_parm);
    % Explicitly state the first dim is channel
    tfr_1ch = reshape(tfr_1ch, [1, size(tfr_1ch)]);
    % Save converted TFR
    if save_internal
        tfr_ext(ch,:,:,:) = tfr_1ch;
    else
        % 1ch TFR is saved in a file separately
        Bch = Btmp; 
        fname_ch = fullfile(real_dir, [Bch.MEGinfo.ExtraChannelInfo.Channel_name{ix_ext(ch)} '.tfr.mat']);
        % Bch.MEGinfo.ExtraChannelInfo.Channel_active = Bch.MEGinfo.ExtraChannelInfo.Channel_active(ix_ext2(ch)); 
        Bch.MEGinfo.ExtraChannelInfo.Channel_active = true; 
        Bch.MEGinfo.ExtraChannelInfo.Channel_id     = Bch.MEGinfo.ExtraChannelInfo.Channel_id(ix_ext(ch)); 
        Bch.MEGinfo.ExtraChannelInfo.Channel_name   = Bch.MEGinfo.ExtraChannelInfo.Channel_name(ix_ext(ch)); 
        Bch.MEGinfo.ExtraChannelInfo.Channel_type   = Bch.MEGinfo.ExtraChannelInfo.Channel_type(ix_ext(ch)); 
        % saveman is disabled in 1ch file
        Bch.MEGinfo.saveman.data_dir = [];
        if strcmp(Measurement, 'EEG')
            % Note: reconvert_meg2eeg removes additional tfr fields such as tfr, frequency, or tfr_parm
            Bch = reconvert_meg2eeg(Bch);
        end
        % Append TFR data
        Bch.tfr_ext   = tfr_1ch;
        Bch.frequency = frequency;
        Bch.tfr_parm  = tfr_parm_1ch;
        % Save ch file
        save(fname_ch, '-struct', 'Bch', save_ver, save_comp)
    end

    if mod(ch,10)==0 && ch~=num_ch_ext
        disp([num2str(ch) '/' num2str(num_ch_ext) ' processed(ext)...'])
    end
end
disp([num2str(num_ch_ext) '/' num2str(num_ch_ext) ' completed(ext)...'])
end

%% Save consolidation file
% Prepare template by removing unnecessary fields
Btmp = inner_rmfield(B, {'bexp','refmg','bexp_ext'});
Btmp.MEGinfo.sensor_weight = [];

% Remove ref or ext info when make_ flag is false
ix_remove = [];
if ~tfr_parm.make_ref
    Btmp.ref_pick  = [];
    Btmp.ref_Qpick = [];
    ix_remove = [ix_remove; ix_ref];
end
if ~tfr_parm.make_ext
    ix_remove = [ix_remove; ix_ext];
end
Btmp.MEGinfo.ExtraChannelInfo.Channel_active(ix_remove) = [];
Btmp.MEGinfo.ExtraChannelInfo.Channel_id(ix_remove)     = [];
Btmp.MEGinfo.ExtraChannelInfo.Channel_name(ix_remove)   = [];
Btmp.MEGinfo.ExtraChannelInfo.Channel_type(ix_remove)   = [];
if strcmp(Measurement, 'EEG')
    % Note: reconvert_meg2eeg removes additional tfr fields such as tfr, frequency, and tfr_parm
    Btmp = reconvert_meg2eeg(Btmp);
end
% Append TFR data
Btmp.tfr       = tfr;
Btmp.tfr_ref   = tfr_ref;
Btmp.tfr_ext   = tfr_ext;
Btmp.frequency = frequency;
Btmp.tfr_parm  = tfr_parm;
% Save consolidation file
save(tfr_file, '-struct', 'Btmp', save_ver, save_comp)

disp(['TFR file is saved in: ' tfr_file])

end % End of the function

function S = inner_rmfield(S, rfields)
% rmfield with checking fields
for rf=1:length(rfields)
    if isfield(S, rfields{rf})
        S = rmfield(S, rfields{rf});
    else
        warning([rfields{rf} ' does not exist in the struct.'])
    end
end
end

function tfr_parm = inner_check_parm(tfr_parm)
% Check parm structure and set default values

% Method for converting MEG to TFR
% Select from {'wavelet', 'hilbert', 'bandpass'}
if ~isfield(tfr_parm, 'method') || isempty(tfr_parm.method)
    tfr_parm.method = 'wavelet';
end

% ------ Options for wavelet -------
if strcmp(tfr_parm.method, 'wavelet')
    % Freq options (required for wavelet)
    if ~isfield(tfr_parm, 'min_freq') || isempty(tfr_parm.min_freq)
        tfr_parm.min_freq = 1;
    end
    if ~isfield(tfr_parm, 'max_freq') || isempty(tfr_parm.max_freq)
        tfr_parm.max_freq = 100;
    end
    if ~isfield(tfr_parm, 'num_freq') || isempty(tfr_parm.num_freq)
        tfr_parm.num_freq = round(tfr_parm.max_freq-tfr_parm.min_freq+1); % Freq stepsize becomes 1Hz
    end
    if ~isfield(tfr_parm, 'central_freq') || isempty(tfr_parm.central_freq)
        tfr_parm.central_freq = 1;
    end
    if ~isfield(tfr_parm, 'fwhm') || isempty(tfr_parm.fwhm)
        tfr_parm.fwhm = 3;
    end
    if ~isfield(tfr_parm, 'freq_scale') || isempty(tfr_parm.freq_scale)
        tfr_parm.freq_scale = 'linear'; % or log
    end
end
% ------------------------------------

% ------ Options for hilbert or bandpass -------
if strcmp(tfr_parm.method, 'hilbert') || strcmp(tfr_parm.method, 'bandpass')
    if ~isfield(tfr_parm, 'freq_band_table') || isempty(tfr_parm.freq_band_table)
        % Set the default frequency bands
        tfr_parm.freq_band_table = get_freq_bands();
    end

    if ~isfield(tfr_parm, 'filter_type') || isempty(tfr_parm.filter_type)
        tfr_parm.filter_type = 'butter';
    end

    if strcmp(tfr_parm.filter_type, 'butter')
        if ~isfield(tfr_parm, 'filter_order') || isempty(tfr_parm.filter_order)
            % Filter order for butterworth
            tfr_parm.filter_order = 4;
        end
    end
end
% ------------------------------------

% ----- These are fixed as false -----
tfr_parm.baseline_long  = false;
tfr_parm.calc_power     = false;
tfr_parm.calc_magnitude = false;
tfr_parm.average_trial  = false;
% ------------------------------------

% Other options (valid for wavelet)
if ~isfield(tfr_parm, 'mask_coi') || isempty(tfr_parm.mask_coi)
    if strcmp(tfr_parm.method, 'wavelet')
        tfr_parm.mask_coi = true;
    else
        tfr_parm.mask_coi = false;
    end
end

% Channel options
if ~isfield(tfr_parm, 'make_ext') || isempty(tfr_parm.make_ext)
    tfr_parm.make_ext = true;
end
if ~isfield(tfr_parm, 'make_ref') || isempty(tfr_parm.make_ref)
    tfr_parm.make_ref = true;
end
if ~isfield(tfr_parm, 'remove_ch')
    tfr_parm.remove_ch = [];
end
if ~isfield(tfr_parm, 'remove_ch_ext')
    tfr_parm.remove_ch_ext = [];
end
if ~isfield(tfr_parm, 'remove_ch_ref')
    tfr_parm.remove_ch_ref = [];
end

% Save options
if ~isfield(tfr_parm, 'saveman') || isempty(tfr_parm.saveman)
    tfr_parm.saveman = struct;
end
if ~isfield(tfr_parm.saveman, 'data_dir')
    tfr_parm.saveman.data_dir = [];
end
if ~isfield(tfr_parm.saveman, 'compression')
    tfr_parm.saveman.compression = '-nocompression';
end
if ~isfield(tfr_parm.saveman, 'version')
    if strcmp(tfr_parm.saveman.compression, '-nocompression')
        tfr_parm.saveman.version = '-v7.3';
    else
        tfr_parm.saveman.version = '-v7';
    end
end
end

function [read_meg_name, read_ext_name, read_ref_name, ...
          read_meg_idx,  read_ext_idx,  read_ref_idx] = inner_separate_read_ch_list(read_ch_list, meginfo)
% Separate read_ch_list into meg, ext, and ref
% Return name and index of read channels

% Get channel labels
switch meginfo.Measurement
    case 'MEG'
        meg_labels = vb_meginfo_get_channel_label_meg(meginfo);
        ext_labels = vb_meginfo_get_channel_label_extra(meginfo);
        ref_labels = vb_meginfo_get_channel_label_refmg(meginfo);
    case 'EEG'
        meg_labels = vb_meginfo_get_channel_label_meg(meginfo); % Use meginfo func
        ext_labels = vb_eeginfo_get_channel_label_extra(meginfo);
        ref_labels = []; % There is no ref in EEGinfo
end

% Check MEG/EEG channels
if ~isempty(meg_labels)
  [read_meg_idx] = vb_util_get_index(meg_labels, read_ch_list);
  if ~isempty(read_meg_idx)
    read_meg_idx = vb_util_arrange_list(read_meg_idx, 0);
  end
else
  read_meg_idx = [];
end

% Check EXTRA channels
if ~isempty(ext_labels)
  [read_ext_idx] = vb_util_get_index(ext_labels, read_ch_list);
  if ~isempty(read_ext_idx)
    read_ext_idx = vb_util_arrange_list(read_ext_idx, 0);
  end
else
  read_ext_idx = [];
end

% Check REFERENCE channels
if ~isempty(ref_labels)
  [read_ref_idx] = vb_util_get_index(ref_labels, read_ch_list);
  if ~isempty(read_ref_idx)
    read_ref_idx = vb_util_arrange_list(read_ref_idx, 0);
  end
else
  read_ref_idx = [];
end

read_meg_name = meg_labels(read_meg_idx);
read_ext_name = ext_labels(read_ext_idx);
read_ref_name = ref_labels(read_ref_idx);
end % --- end of inner_separate_read_ch_list()