function vb_make_tfr_trial_data(tfr_file, proc_spec, new_tfr_file)
% Make trial data files by chopping continuous TFR data
% [usage]
%   [data, ch_info] = vb_msrmnt_make_trial_data(meeg_file, proc_spec, new_meeg_file)
% [input]
%    meeg_file : <required> <<file>>
%                Continuous format TFR data file(.tfr.mat)
%    proc_spec : <required> <<struct>>
%              :  struct defined process specifications which has fields 
%              :  as follows:
%              :
%      .trig_file : <conditionally required> <<file>>
%                 :  trigger file(.trig.mat)
%                 :  this file is expected to have data as follows:
%                 :   .trig [1 x Ntrial]
%                 :   .parm.data_file (without path) : base file
%                 :   .parm.Pretrigger_ms  : pretrigger [msec]
%                 :   .parm.Posttrigger_ms : posttrigger [msec]
%                 :   .parm.fsamp : sampling frequency [Hz]
%                 :  if this is invalid and .proc_parm field is not
%                 :  specified, it is error.
%      .internal  : <optional>
%                 : If internal is not specified, refer to saveman.data_dir
%                   = true  : single file format.
%                   = false : multiple file format(MAT+binary dir).
%      .proc_parm : <optional> <<struct>>
%                 :  .Pretrigger_ms  : Pretrigger period   [msec]
%                 :  .Posttrigger_ms : Posttrigger period  [msec]
%                 :  .sample_freq    : sampling frequency [Hz]
%                 :  .trigger        : list of trigger samples
%                 :                  : [1 x Ntrial]
%                 :    if these are specified, 
%                 :    they overwrite the setting values of trig_file
%   new_meeg_file : <required> <<file>>
%                   Trial format TFR data file(.tfr.mat)
%            
% [output]
%           : these are the same as the return values of vb_load_meg_data
%      data : chopped data [Nchannel x Nsample x Ntrial]
%   ch_info : channel information struct
%           :  .Active
%           :  .Name
%           :  .ID
%           :  .Type
% [note]
%
% [history]
%   2023-11-03 (k_suzuki) Made the first ver based on vb_msrmnt_make_trial_data.m
%
% Copyright (C) 2011, ATR All Rights Reserved.
% License : New BSD License(see VBMEG_LICENSE.txt)

% --- CHECK ARGUMENTS --- %
if ~exist('proc_spec', 'var'), proc_spec = []; end
if ~exist('tfr_file', 'var'), tfr_file = ''; end
if ~exist('new_tfr_file', 'var'), new_tfr_file = ''; end

[tfr_file, proc_spec, new_tfr_file] = ...
    inner_check_arguments(tfr_file, proc_spec, new_tfr_file);

% --- MAIN PROCEDURE --------------------------------------------------------- %
%

% default values
[base_file_def, trigger_def, cond_id_def, pretrigger_def, posttrigger_def, ...
  sample_freq_def] = inner_read_trigger_file(proc_spec.trig_file);

% substitute values
[base_file_sub, trigger_sub, cond_id_sub, pretrigger_sub, posttrigger_sub, ...
  sample_freq_sub] = inner_read_proc_parm(tfr_file, proc_spec.proc_parm);

if ~isempty(trigger_sub)
  trigger = trigger_sub;
else
  trigger = trigger_def;
end

if ~isempty(cond_id_sub)
  cond_id = cond_id_sub;
else
  cond_id = cond_id_def;
end

if ~isempty(pretrigger_sub)
  pretrigger_ms = pretrigger_sub;
else
  pretrigger_ms = pretrigger_def;
end

if ~isempty(posttrigger_sub)
  posttrigger_ms = posttrigger_sub;
else
  posttrigger_ms = posttrigger_def;
end

% sample frequency for triger file
if ~isempty(sample_freq_sub)
  sample_freq = sample_freq_sub;
else
  sample_freq = sample_freq_def;
end

% Set new_spec
[f_, p_, e_] = vb_get_file_parts(new_tfr_file);
new_spec.trigger = trigger;
new_spec.cond_id = cond_id;
new_spec.pretrigger_ms = pretrigger_ms;
new_spec.posttrigger_ms = posttrigger_ms;
new_spec.sample_freq = sample_freq;
new_spec.root_dir = f_;
new_spec.new_file = [p_, e_];
new_spec.new_bin_dir = proc_spec.new_bin_dir;
vb_tfrfile_make_trial_data(tfr_file, new_spec);

%
% --- END OF MAIN PROCEDURE -------------------------------------------------- %

% --- INNER FUNCTIONS -------------------------------------------------------- %
%
% --- inner_check_arguments()
%
function [tfr_file, proc_spec, new_tfr_file] = ...
    inner_check_arguments(tfr_file, proc_spec, new_tfr_file)
func_ = mfilename;

if isempty(tfr_file)
  error('(%s) tfr_file is a required parameter', func_);
end
if exist(tfr_file, 'file') ~= 2
  error('(%s) Specified tfr_file not found.', func_);
end    
if ~contains(tfr_file, '.tfr.mat') 
    error('(%s) tfr_file is not TFR(.tfr.mat) file.', func_);
end

if isempty(proc_spec)
  error('(%s) proc_spec is a required parameter', func_);
end

if isempty(new_tfr_file)
  error('(%s) new_tfr_file is a required parameter', func_);
end

% --- proc_spec
%  .trig_file
%  .internal
%  .proc_parm

if ~isfield(proc_spec, 'trig_file')
  proc_spec.trig_file = '';
end

% If internal is not specified, refer to data_dir in input file
if ~isfield(proc_spec, 'internal') || isempty(proc_spec.internal)
    meginfo = vb_tfrfile_load_meginfo(tfr_file);
    if ~isfield(meginfo.saveman,'data_dir') || isempty(meginfo.saveman.data_dir)
        % If internal is true, save all ch in a single file
        proc_spec.internal = true;
    else
        proc_spec.internal = false;
    end
end

if proc_spec.internal == false
  [p_, f_, e_] = vb_get_file_parts(new_tfr_file);
  new_bin_dir = f_(1:end-4); % f_ : -4 means '.tfr'
  proc_spec.new_bin_dir = ['./' new_bin_dir];
else
  proc_spec.new_bin_dir = '';
end

if ~isfield(proc_spec, 'proc_parm')
  proc_spec.proc_parm = [];
end

if isempty(proc_spec.trig_file) && isempty(proc_spec.proc_parm)
  error('(%s) either trig_file or proc_parm is required', func_);
end
return;
%
% --- end of inner_check_arguments()


% --- inner_read_trigger_file()
%
function [base_file, trigger, cond_id, pretrigger, posttrigger, sample_freq] = ...
  inner_read_trigger_file(trig_file)

func_ = mfilename;

base_file = '';
trigger = [];
cond_id = [];
pretrigger = [];
posttrigger = [];
sample_freq = 1024;

if isempty(trig_file)
  return;
end

if exist(trig_file, 'file') ~= 2
  error('(%s) cannot find trig_file : %s', func_, trig_file);
end

trig = load(trig_file);
if isfield(trig, 'trig') && ~isempty(trig.trig)
  trigger = trig.trig;
end
if isfield(trig, 'cond_id') && ~isempty(trig.cond_id)
  cond_id = trig.cond_id;
end

if isfield(trig, 'parm') && ~isempty(trig.parm)
  if isfield(trig.parm, 'data_file') && ~isempty(trig.parm.data_file)
    base_file = trig.parm.data_file;
  end

  if isfield(trig.parm, 'Pretrigger_ms')
    pretrigger = trig.parm.Pretrigger_ms;
  end

  if isfield(trig.parm, 'Posttrigger_ms')
    posttrigger = trig.parm.Posttrigger_ms;
  end
  
  if isfield(trig.parm, 'fsamp')
    sample_freq = trig.parm.fsamp;
  end
end
return;
%
% --- end of inner_read_trigger_file()


% --- inner_read_proc_parm()
%
function [base_file, trigger, cond_id, pretrigger, posttrigger, sample_freq] = ...
  inner_read_proc_parm(tfr_file, parm)

base_file = '';
trigger = [];
cond_id = [];
pretrigger = [];
posttrigger = [];
sample_freq = [];

if isempty(parm)
  return;
end

if isfield(parm, 'trigger') && ~isempty(parm.trigger)
  trigger = parm.trigger;
end
if isfield(parm, 'cond_id') && ~isempty(parm.cond_id)
  cond_id = parm.cond_id;
end

[p_, f_, e_] = vb_get_file_parts(tfr_file);
base_file = [f_, e_];

if isfield(parm, 'Pretrigger_ms')
  pretrigger = parm.Pretrigger_ms;
end

if isfield(parm, 'Posttrigger_ms')
  posttrigger = parm.Posttrigger_ms;
end

if isfield(parm, 'sample_freq')
  sample_freq = parm.sample_freq;
end
return;
%
% --- end of inner_read_proc_parm()
%
% --- END OF INNER FUNCTIONS ------------------------------------------------- %

%%% END OF FILE %%%
