function vb_tfrfile_make_trial_data(tfr_file, proc_spec)
% Make trial data files of TFR after extracting trials from continuous raw data
% [usage]
%   [tfr_data, ch_info] = vb_tfrfile_make_trial_data(tfr_file, proc_spec)
%
% [input]
%    tfr_file : <required> TFR-MAT file as base (with path)
%   proc_spec : <required> <<struct>> struct defined process specifications
%             :  fields are as follows:
%      .root_dir  : <optional> data directory for new meg file
%                 : if not specified, directory of tfr_file is used
%        .trigger : <required> list of sample of trigger [1 x Ntrial]
%  .pretrigger_ms : <required> pretrigger [msec]
% .posttrigger_ms : <required> posttrigger [msec]
%    .sample_freq : <requied> sampling frequency [Hz]
%       .new_file : <optional> new MEG file (without path)
%                 : this will be made at 'root_dir'
%                 : if this is not specified, data is not stored.
%    .new_bin_dir : <optional> external data directory where chopped data
%                 :  will be stored.
%                 :  This is relative path from '.new_file'
%                 :  if this is not specified, data is stored internally
%                 :  (tfr etc.)
%
% [output]
%   tfr_data : chopped data [Nchannel x Nsample x Ntrial]
%    ch_info : <<struct>> channel information of loaded data
%            :  .Active [Nchannel x 1]
%            :  .Name   [Nchannel x 1]
%            :  .ID     [Nchannel x 1]
%            :  .Type   [Nchannel x 1]
%
% [note]
%   @see vb_msrmnt_make_trial_data.m
%
% [history]
%   2023-11-03 (k_suzuki) Made the first ver based on vb_megfile_make_trial_data.m
%   2024-02-15 (k_suzuki) Fixed line 173-175
%
% Copyright (C) 2011, ATR All Rights Reserved.
% License : New BSD License(see VBtfr_LICENSE.txt)

% --- CHECK ARGUMENTS --- %
if ~exist('tfr_file', 'var'), tfr_file = ''; end
if ~exist('proc_spec', 'var'), proc_spec = []; end
[tfr_file, proc_spec] = inner_check_arguments(tfr_file, proc_spec);

% --- MAIN PROCEDURE --------------------------------------------------------- %
%
fs0  = proc_spec.sample_freq;

meginfo = vb_tfrfile_load_meginfo(tfr_file); % In case of EEG, load EEGinfo as MEGinfo
fs  = vb_meginfo_get_sampling_frequency(meginfo);

% Recalculate trial time index according to 'Pre/Posttrigger'
% if Sample Frequency is different from raw data
% Convert trigger time index for fs [Hz]
frate = (fs/fs0);

% --- initial trig is loaded from trig_file
trigger = round( proc_spec.trigger * frate );

cond_id = proc_spec.cond_id;

% Sample number for Pre/Post period in data file
pretrigger  = ceil(proc_spec.pretrigger_ms  * (fs/1000));
posttrigger = ceil(proc_spec.posttrigger_ms * (fs/1000));

load(tfr_file, 'frequency', 'tfr_parm', 'Measurement');

std_sample_list = (-pretrigger:posttrigger)';
num_sample = length(std_sample_list);
num_trial  = length(trigger);
num_freq   = length(frequency);

trig_info.Trigger = trigger;
trig_info.Pretrigger = pretrigger;
trig_info.Posttrigger = posttrigger;
[target_sample] = vb_loadspec_make_trial_sample(trig_info);

num_trial_new = size(target_sample,1);
num_time_new = target_sample(1,2)-target_sample(1,1)+1;

% ---
% --- If new binary directory is given, store chopped data as the new
% --- binary data externally.
% --- If it is not given, store chopped data internally (tfr)
% ---
if isempty(proc_spec.new_file)
    new_bin_dir = '';
    save_externally = false;
else
    if ~isfield(proc_spec, 'new_bin_dir') || isempty(proc_spec.new_bin_dir)
        new_bin_dir = '';
        save_externally = false;

    else
        save_externally = true;

        new_bin_dir = fullfile(proc_spec.root_dir, proc_spec.new_bin_dir);
        if exist(new_bin_dir, 'dir') ~= 7
            vb_mkdir(new_bin_dir);
        end
        %precision = vb_meginfo_get_precision(tfr_info);
        %precision = 'float64';
    end
end

if save_externally
    % Load all ch-type includeing inactive channels
    ch_labels = vb_meginfo_get_channel_label_whole(meginfo, false);
    num_ch    = size(ch_labels, 1);

    fprintf('\n...now on chopping data ...\n');
    LINE_LEN = 30;

    for i_ch = 1:num_ch
        load_spec = [];
        load_spec.ChannelName   = {ch_labels{i_ch}};
        load_spec.ChannelSwitch = true;
        load_spec.ChannelType   = 'ALL';
        load_spec.ActiveChannel = false;
        load_spec.ActiveTrial   = false;
        load_spec.Trigger       = trigger;
        load_spec.Pretrigger    = pretrigger;
        load_spec.Posttrigger   = posttrigger;

        new_file = fullfile(new_bin_dir, [ch_labels{i_ch} '.tfr.mat']);

        % chopped_data is saved as new_file
        vb_load_tfr_data(tfr_file, load_spec, new_file);

        % --- re-stored filtered data
        % dat format is not supported yet
        %     new_file = sprintf('%s/%s.ch.tfr.dat', new_bin_dir, ch_labels{i_ch});
        %     fid = fopen(new_file, 'wb');
        %     if fid == -1
        %       warning('(%s) cannot open file : %s\n', mfilename, new_file);
        %       continue;
        %     end
        %     fwrite(fid, chopped_data, precision);
        %     fclose(fid);

    end

    % --- Save consolidation file
    % Load original data as base template
    Btmp = load(tfr_file, '-regexp', '^(?!tfr$|tfr_ext$|tfr_ref$).');
    if strcmp(Measurement, 'EEG')
        % These are removed by converting
        parm_tmp = Btmp.tfr_parm;
        freq_tmp = Btmp.frequency;
        % EEG -> MEG format
        Btmp = convert_eeg2meg(Btmp);
    end
    % data_dir option is updated
    %Btmp.tfr_parm.saveman.data_dir = proc_spec.new_bin_dir;
    Btmp.MEGinfo.saveman.data_dir = proc_spec.new_bin_dir;
    % Update fields
    Btmp.MEGinfo.Nsample = num_time_new;
    Btmp.MEGinfo.Nrepeat = num_trial_new;
    Btmp.MEGinfo.Pretrigger  = load_spec.Pretrigger;
    Btmp.MEGinfo.ActiveTrial = true(num_trial_new,1);
    % Set cond_id
    if isfield(Btmp.MEGinfo, 'cond_id')
        cond_tmp = Btmp.MEGinfo.cond_id;
    else
        cond_tmp = 1;
    end
    Btmp.MEGinfo.cond_id = ones(1,num_trial_new)*cond_tmp;
    % Set Trial structure
    for tr=1:num_trial_new
        Btmp.MEGinfo.Trial(tr,1).Active = true;
        Btmp.MEGinfo.Trial(tr,1).number = tr;
        Btmp.MEGinfo.Trial(tr,1).sample = [target_sample(tr,1):target_sample(tr,2)]';
    end
    % In case of EEG, save Btmp as EEG format
    if strcmp(Measurement, 'EEG')
        % Note: reconvert_meg2eeg removes additional tfr fields such as tfr, frequency, and tfr_parm
        Btmp = reconvert_meg2eeg(Btmp);
        % Recover removed fields
        Btmp.tfr_parm  = parm_tmp;
        Btmp.frequency = freq_tmp;
    end
    % Append blank TFR data
    Btmp.tfr       = [];
    Btmp.tfr_ref   = [];
    Btmp.tfr_ext   = [];
    Btmp.frequency = frequency;
    Btmp.tfr_parm  = tfr_parm;

    % New file name
    new_file = fullfile(proc_spec.root_dir, proc_spec.new_file);

    % Save consolidation file
    save(new_file, '-struct', 'Btmp')

    disp(['TFR file is saved in: ' new_file])

else
    % save MEG data as internal variable

    fprintf('\n...now on processing TFR data ...\n');

    load_spec = [];
    load_spec.ChannelSwitch = true;
    load_spec.ChannelType   = 'ALL';
    load_spec.ActiveChannel = false;
    load_spec.ActiveTrial   = false;
    load_spec.Trigger       = trigger;
    load_spec.Pretrigger    = pretrigger;
    load_spec.Posttrigger   = posttrigger;

    new_file = fullfile(proc_spec.root_dir, proc_spec.new_file);

    % data is saved as new_file
    vb_load_tfr_data(tfr_file, load_spec, new_file);
end

end % --- END OF MAIN PROCEDURE -------------------------------------------------- %

% --- INNER FUNCTIONS -------------------------------------------------------- %
%
% --- inner_check_arguments()
%
function [tfr_file, proc_spec] = inner_check_arguments(tfr_file, proc_spec)
func_ = mfilename;

% --- check tfr_file
if isempty(tfr_file)
    error('(%s) tfr_file is a required parameter', func_);
end

if exist(tfr_file, 'file') ~= 2
    error('(%s) cannot find tfr_file : %s', func_, tfr_file);
end

% --- check proc_spec ---
%     .root_dir
%     .trigger
%     .pretrigger_ms
%     .posttrigger_ms
%     .sample_freq
%     .new_file
%     .new_bin_dir
% -----------------------
if isempty(proc_spec)
    error('(%s) proc_spec is a required parameter', func_);
end

% --- required fields

% ----- proc_spec.trigger
if ~isfield(proc_spec, 'trigger') || isempty(proc_spec.trigger)
    error('(%s) trigger is a required field of proc_spec', func_);
end

% ----- proc_spec.pretrigger_ms
if ~isfield(proc_spec, 'pretrigger_ms') || isempty(proc_spec.pretrigger_ms)
    error('(%s) pretrigger_ms is a required field of proc_spec', func_);
end

% ----- proc_spec.posttrigger_ms
if ~isfield(proc_spec, 'posttrigger_ms') || isempty(proc_spec.posttrigger_ms)
    error('(%s) posttrigger_ms is a required field of proc_spec', func_);
end

% ----- proc_spec.sample_freq
if ~isfield(proc_spec, 'sample_freq') || isempty(proc_spec.sample_freq)
    error('(%s) sample_freq is a required field of proc_spec', func_);
end


% --- optional fields

% ----- get path of tfr_file as a default path
[fpath] = vb_get_file_parts(tfr_file);

% ----- proc_spec.root_dir
if ~isfield(proc_spec, 'root_dir') || isempty(proc_spec.root_dir)
    proc_spec.root_dir = fpath;
end

% ----- proc_spec.new_file
if ~isfield(proc_spec, 'new_file')
    proc_spec.new_file = '';
end

% ----- proc_spec.new_bin_dir
if ~isfield(proc_spec, 'new_bin_dir')
    proc_spec.new_bin_dir = '';
end
end % --- end of inner_check_arguments()
%
% --- END OF INNER FUNCTIONS ------------------------------------------------- %

% --- END OF FILE --- %
