function [H, Pe, Hfilt, C, FE] = vbCSSP(B, S, Nem)
%--- Inputs ---
% B     : Measurement MEG signals (Nm*Nt)
%         Saturated signals should be excluded
% S     : Basis of the noise space (Nm*Nh)
% Nem   : Num of EM-iteration (optional, default=1000)

%--- Outputs ---
% H     : Estimated coefficients of the basis (Nh*Nt)
% Pe    : Projection matrix from B to epsilon (residual space of noise = brain signal)
% Hfilt : Converting matrix from B to Hbar (estimated H)
% C     : Estimated channel gain matrix (Nm*Nm)
% FE    : Free energy calculated when every E-step finished

%--- Other variables ---
% Lam   : Measurement noise precision
% Gamma : Posterior precision of h

% Num of iteration
if ~exist('Nem', 'var') || isempty(Nem)
    Nem = 1000;
end

% Set time length and num of sensor
nt = size(B,2);
nm = size(B,1);

% Total num of harmonics
nh = size(S,2); 

% Define prior covariance of C as identity matrix
% (L2-regularization)
covmat = eye(nm);
icovmat = covmat;

% Init rho and K (prior covariance of C)
pw = 0.5;
rho = (1-pw) / (pw*nt);
K = covmat .* rho;

%% initialization
% C is initialized using the svd of the data
% [U,S,~] = svd(B,'econ');
% sd = diag(S);
% CN = U(:,1:Nl)*diag(sd(1:Nl));

% This initialization means first Hbar is conventional HF
% But diagonal loading const is added
% Assuming C=eye
CS = S;
C2 = eye(nm,nm);

% The noise precision is initialized using the diagonal elements
% of the data covariance
Rbb = B*B'; % Sum{b(t)*b(t)'}
lam = 1./diag(Rbb/nt);
% Lam = diag(lam);
Lam = mean(lam).*eye(nm); % Use scalar lam

% lam = 0.1; % Simple case
% Lam = lam*eye(nm);

% Init free energy
FE = zeros(Nem, 1);
E1 = zeros(Nem, 1);
E2 = zeros(Nem, 1);
E3 = zeros(Nem, 1);

%% Start iteration
for iem=1:Nem
    %------- Start E-step (p(H|B)) -------
    Gamma = S'*(C2*Lam)*S + eye(nh,nh); % Posterior precision of H
    iGam = Gamma \ eye(size(Gamma));
    Hfilt = iGam * CS' * Lam;
    H = Hfilt * B; % Posterior mean

    % Calculate sufficient statistics for M-step
    HH = H*H';
    Rhh = nt*iGam + HH; % T*iGam + Sum{h(t)*h(t)'}
    Rhb = H * B';       % Sum{h(t)*b(t)'}
    %-------- End E-step --------

    % Store old Lam to calc evidence
    % This is because, we must use variables in the same state
    % Here, we use all vars in E-step when we calc evidence
    % (it is really hard to use all vars in M-step)
    Lam_old = Lam;

    %------- Start M-step (p(C|B)) -------
    % Calc inverse posterior precision iPhi
    % iPhi = pinv(diag(diag(S*Rhh*S'*Lam)) + pinv(K));
    % Calc inverse posterior precision iPhi using inverse lemma
    % It is equivalent to above one when K is invertible
    idNRN = diag( 1./diag(S*Rhh*S') );
    %iPhi = K - K * inv(idNRN+Lam*K) * Lam*K;
    iPhi = K - K / (idNRN+Lam*K) * Lam*K;

    Cd = iPhi * diag(S*Rhb*Lam); % Cdbar=<Cd>
    C = diag(Cd); % Cbar=<C>
    CS = C * S; % <C>S
    Cd2 = diag(iPhi) + Cd.^2; % <Cd.^2>=diag(iPhi+Cdbar*Cdbar')
    C2 = diag(Cd2); % <C^2>
    %-------- End M-step --------

    %------- Update Lambda -------
    ECdCd = iPhi+Cd*Cd'; % = <CdCd'>
    %      ilam = diag( Rbb - 2*CN*Rhb + ECdCd.*(S*Rhh*S') ) /nt;
    %      lam = 1 ./ ilam;
    %      Lam = diag(lam);

    BB = sum(dot(B,B)); % sum(b'*b)
    BCNH = sum(dot(B, CS*H)); % sum(b'*C*S*h)
    trNCNR = trace(S'*C2*S*Rhh); % sum( tr( S'*C2*S * (iGam+h*h') ) )
    ilam = (BB - 2*BCNH + trNCNR) / (nt*nm);
    lam = 1 ./ ilam;
    Lam = lam.*eye(nm);

    %      %--- sum(b'*b)
    %      % Def1
    %      BB = 0;
    %      for tt=1:nt
    %          BB = BB + B(:,tt)'*B(:,tt);
    %      end
    %      % Def2
    %      BB = B(:)'*B(:);
    %      % Def3
    %      BB = sum(dot(B,B));
    %
    %      %--- sum(b'*C*S*h)
    %      % Def1
    %      BCNH = 0;
    %      for tt=1:nt
    %          BCNH = BCNH+ B(:,tt)'*CN*H(:,tt);
    %      end
    %      % Def2
    %      CNH = CN * H;
    %      BCNH = B(:)'*CNH(:);
    %      % Def3
    %      BCNH = sum(dot(B, CN*H));
    %
    %      %--- sum( tr( S'*C2*S * (iGam+h*h') ) )
    %      % Def1
    %      trNCNR = 0;
    %      for tt=1:nt
    %          trNCNR = trNCNR + trace(S'*C2*S* (iGam + H(:,tt)*H(:,tt)') );
    %      end
    %      % Def2
    %      trNCNR = 0;
    %      for tt=1:nt
    %          trNCNR = trNCNR + S'*C2*S* (iGam + H(:,tt)*H(:,tt)');
    %      end
    %      trNCNR = trace(trNCNR);
    %      % Def3
    %      trNCNR = trace(S'*C2*S*(nt*iGam + HH));
    %      % Def4
    %      trNCNR = trace(S'*C2*S*Rhh);

    %-------- End Lambda --------

    %------- Update rho and K -------
    rho = trace(icovmat*ECdCd) / 2*nm;
    % rho = lam;
    % rho = trace(pinv(covmat)*ECdCd) / 2*nm;
    K = covmat .* rho;
    % Update pw for monitoring
    pw = 1 / (rho*nt+1);
    %-------- End rho and K --------

    %------ Start Free energy ------
    e1 = nt * (sum(log(diag(Lam_old))) - inner_logdet(Gamma)); % T log(|Lam|/|Gamma|)
    e2 = B.*repmat(diag(Lam_old),[1,nt]);
    e2 = -e2(:)' * B(:); % -sum(b'*Lam*b)

    Hlong1 = repmat(H, [nh,1]);
    Hlong1 = Hlong1(:); % [h(1)_1 h(1)_2 h(1)_3  h(1)_1 h(1)_2 h(1)_3  h(1)_1 h(1)_2 h(1)_3  h(2)_1 h(2)_2 h(2)_3 ...]
    Hlong2 = repmat(H(:), [1,nh])';
    Hlong2 = Hlong2(:); % [h(1)_1 h(1)_1 h(1)_1  h(1)_2 h(1)_2 h(1)_2  h(1)_3 h(1)_3 h(1)_3  h(2)_1 h(2)_1 h(2)_1 ...]
    Glong  = Gamma(:);
    Glong  = repmat(Glong, [nt,1]);% [g11 g21 g31  g12 g22 g32  g13 g23 g33 ...]

    e3 = sum( Hlong1 .* Glong .* Hlong2 ); % sum(hbar'*Gam*hbar)
    e4 = - trace(ECdCd) / rho; % -1/rho tr(iPhi+CdCd')
    e5 = inner_logdet(iPhi); % log|iPhi|
    e6 = -nm*log(rho); % -Mlog(rho)
    e7 = -(nm*nt + nh*nt + nm) * log(2*pi); % -(MT+LT+M) log(2pi)
    e8 = -nh*nt; % -LT

    FE(iem) = (e1+e2+e3+e4+e5+e6+e7+e8)/2;

    % For debug (set arbitorary term for checking)
    % E1(iem)=e1; E2(iem)=e2; E3(iem)=e3;

    if ~rem(iem, 10), disp(['Itr: ' num2str(iem) ', pw = ' num2str(pw) ', evidence = ' num2str(FE(iem))]); end

    %      Pe = eye(nm) - CN * iGam * CN' * Lam;
    %      subplot(1,2,1),imagesc(iGam)
    %      subplot(1,2,2),imagesc(Pe),colorbar,drawnow
    %      pause(0.1)

    % Debug
    % Etmp=0;
    % Etmp2=0;
    % for tt=1:Nt
    %     Etmp = Etmp+ B(:,tt)'*Lam*B(:,tt) + 3 + Hbar(:,tt)'*Gamma*Hbar(:,tt) - 2*Hbar(:,tt)'*Gamma*Hbar(:,tt);
    %     Etmp2 = Etmp2+ B(:,tt)'*Lam*B(:,tt) + 3 + Hbar(:,tt)'*Gamma*Hbar(:,tt) - 2*Hbar(:,tt)'*CN'*Lam*B(:,tt);
    % end
    % evidence(iem) = 0.5*Nt*log(det(Lam)) -0.5*Etmp2 - 0.5*Nt*log(det(Gamma));

    %------- End Evidence -------
end

% Compute matrices using the latest variables
%  Pe = eye(nm) - CS * iGam * CS' * Lam;
Hfilt = iGam * CS' * Lam; 
Pe = eye(nm) - CS * Hfilt;

% This plot is useful for debugging
% figure, subplot(4,1,1), plot(evidence); subplot(4,1,2), plot(E1); subplot(4,1,3), plot(E2); subplot(4,1,4), plot(E3);

% Re-standardization
% B = (B .* Bstd) + Bmean;

% Bclean = B - CN*Hbar;
% Bclean2 = Pe*B;
% figure, % These are equivalent
% subplot(2,1,1), plot(Bclean') % But when B is re-standardized, this is wrong.
% subplot(2,1,2), plot(Bclean2')

% figure,
% subplot(3,1,1), plot(B')
% subplot(3,1,2), plot(Bclean2')
% subplot(3,1,3), plot(Hbar')

% figure,imagesc(C), colorbar, colormap(my_parula)
end % End of function

function logdetX = inner_logdet(X)
[L, U, P] = lu(X);
du = diag(U);
c = det(P) * prod(sign(du));
logdetX = log(c) + sum(log(abs(du)));

end % End of function
