function [datarm, zref,ixt] = remove_by_temporalPCA(data, refdata, parm_denoise)
% Remove noise by using temporal PCA of reference sensor signal
% (Cheveigné and Simon, 2007)
%
% [Reference]
% Cheveigné and Simon. Denoising based on time-shift PCA. J Neurosci Methods. 
% 2007 Sep 30;165(2):297-305. doi: 10.1016/j.jneumeth.2007.06.003.
%
%   datarm = remove_by_temporalPCA(data,Twin_sec,smin)
% --- Input
%   data : data matrix you want to denoise [ch time] 
%   refdata : reference data to compute denoised components 
%   parm_denoise : parameter structure for tPCA
%          sampling_freq : sampling frequency (Hz) (Necessary)
%          Twin_sec = [Nbck , Nfwd] : time shift length for backward and forward direction [sec]
%                                     if Twin is scalar, Nbck = Nfwd = Twin is assumed
%          smin : >= 1 : number of components for threshould PCA (default 40)
%                 < 1  : contribution rate for threshould PCA)
% -- Output
%   datarm : denoised data [ch time]
%
% 2008-1-25 Masa-aki Sato
% 2015-4-20 Yusuke Fujiwara
% 2015-6-12 Yusuke Takeda (Modified for pipeline of VBMEG)
% 2021-5-12 Okito Yamashita new_meg_file is modified. 
% 2021/6/30 O.Yamashit
% 2021/11/5 K.Suzuki (Modified parameter option)



% Check parameters
if exist('parm_denoise', 'var')
    [Twin_sec, smin, sampling_freq] = inner_check_parm(parm_denoise);
else
    error('At least parameter sampling_freq is necessary')
end

% Calculate temporal PCA of reference sensor data
fprintf('Compute noise components \n')
Twin = round(Twin_sec * sampling_freq); 
[zref,ixt] = temporal_pca(refdata, Twin, smin);

% Start denoising
fprintf('Start denoising using reference sensor\n')
datarm = zeros(size(data));
for i_ch=1: size(data,1)
    
    datach = data(i_ch,:);
    
    datach_denoised = datach(ixt)-(datach(ixt)*zref')*zref;
    
    datarm(i_ch,ixt) = datach_denoised;
end

end % End of function

% Inner function to calculate temporal PCA
function [z,indxT,s] = temporal_pca(x, Twin, smin)
% Temporal PCA
% [z,indxT,s] = temporal_pca(x, Twin, smin)
% --- Input
% x : time series signal [NX x T]
% x(n,t) : n-th component signal at time t , t=1:T, n=1:NX
% Twin = [Nbck , Nfwd] : time shift length for backward and forward direction
%  if Twin is scalar, Nbck = Nfwd = Twin is assumed
%   smin : >= 1  : number of components for threshould PCA (muse be integer, default 40)
%           < 1  : contribution rate for threshould PCA
% --- Output
%  z : Time shifted signal after orthogonalization by PCA
%  indxT : time index for z
%  z(:,:) <-> x(:,indxT)
%  indxT = (1+Nbck):(T-Nfwd))
%  s : eigen value
%
% 2007-8-21 Masa-aki Sato
% 2015-4-20 Yusuke Fujiwara
% 2015-6-12 Yusuke Takeda

[NX,T] = size(x);

D  = sum(Twin) + 1;	% time shift length
TD = T - sum(Twin);	% Effective time length for process

if TD <= 0, error('Signal time length is too short'); end;

% subtract bias component
x = vb_repadd(x, - mean(x,2));

% --- Time shifted embedding index
%
%  z(:,t) = [ x(:,t-Nbck);
%                ...;
%             x(:,t);
%                ...;
%             x(:,t+Nfwd) ]

indxT = (1+Twin(1)):(T-Twin(2));
indxD = -Twin(1):Twin(2);
indx  = repmat( indxD(:),[1 TD]) + repmat(indxT,[D 1]);
indx  = indx(:);

X = reshape( x(:,indx), [NX*D TD]);

% --- SVD ---
% X = U * S * V , U:[N x N], S=diag(s):[N x N], V:[N x T]
%     U' * U = V * V' = eye(N)
% --- PCA ---
%  X * X'  = U * S^2 * U'
% (X * X') * U = U * S^2
%  V = S^(-2) * U' * X
% [U, D] = eig(A) : A * U = U * D , D = diag(d)

%[U,S,V] = svd( X * X' );

%[U,s,V]=csvd(X);

[U,S] = eig( X * X' );
s  = diag(S);
if smin<1
    ix = find( s/max(s) > smin );
else
    Npcs = length(s);
    ix = (Npcs - smin + 1):Npcs;
end

% --- Orthogonalize time shifted signal
z  = U(:,ix)' * X;

% Normalization
z  = vb_repmultiply( z, 1./sqrt(sum(z.^2,2)));

err1 = 1 - sum(s(ix))/sum(s);
err2 = max(max(abs( z*z' - eye(size(z,1)))));

fprintf('Dim of noise space = %d, ',length(ix))
fprintf('PCA err = %g, Ortho err = %g\n',err1,err2)
fprintf('PCA contribution ratio = %g \n', sum(s(ix))/sum(s));

end % End of function


function [Twin_sec, smin, sampling_freq] = inner_check_parm(parm)

if isempty(parm)
    error('At least parameter sampling_freq is necessary')
else
    % Extract fields
    field = {'Twin_sec', 'smin', 'sampling_freq'};
    exist_field = isfield(parm, field);
    
    % sampling_freq is necessary
    if ~exist_field(3)
        error('At least parameter sampling_freq is necessary')
    else
        sampling_freq = parm.sampling_freq;
        
        % If Twin_sec is empty, set default value
        if exist_field(1)
            Twin_sec = parm.Twin_sec;
            if length(Twin_sec)==1, Twin_sec = [Twin_sec, Twin_sec]; end;
            if length(Twin_sec)>2, error('Time shift parameter error'); end;
        else
            Twin_sec = [0.05 0.05];
        end
        
        % If smin is empty, set default value
        if exist_field(2)
            smin = parm.smin;
            if length(smin)>1, error('Noise dimension parameter error'); end;
        else
            smin = 40;
        end
    end
end
end % End of Function