function E = reconvert_meg2eeg(B)
% Re-convert the contents of meg.mat to eeg.mat.
% (Assume meg.mat was converted from eeg.mat using convert_eeg2meg)
% E : struct contains all of eeg.mat.
%     e.g., E = load(test.eeg.mat)
% B : struct contains all of meg.mat.
%     e.g., B = load(test.meg.mat) 
%
% Copyright (C) 2011, ATR All Rights Reserved.
% License : New BSD License(see VBMEG_LICENSE.txt)

E = [];

% Convert MEGinfo to EEGinfo
E.EEGinfo = inner_convert_MEGinfo2EEGinfo(B.MEGinfo);

% bexp and bexp_ext are merged into eeg_data 
% refmg is ignored
nch_eeg = B.MEGinfo.Nchannel+length(B.MEGinfo.ExtraChannelInfo.Channel_name);

ix_bexp = B.MEGinfo.ChannelInfo.ID;
ix_bexp_ext = B.MEGinfo.ExtraChannelInfo.Channel_id;

if isfield(B,'bexp') && isfield(B,'bexp_ext')
    E.eeg_data = zeros(nch_eeg, B.MEGinfo.Nsample, B.MEGinfo.Nrepeat);
    E.eeg_data(ix_bexp,:,:) = B.bexp;
    E.eeg_data(ix_bexp_ext,:,:) = B.bexp_ext;
end

% Append Coord and CoordType into EEGinfo
E.EEGinfo.Coord     = B.pick;
E.EEGinfo.CoordType = B.CoordType;

% Set Measurement as MEG
E.Measurement = 'EEG';

% --- INNER FUNCTIONS -------------------------------------------------------- %
%
% --- inner_convert_MEGinfo2EEGinfo()
%
%   returned fields of EEGinfo : 
%     Nchannel         --> Nchannel
%     Nsample          --> Nsample
%     Pretrigger       --> Pretrigger
%     SampleFreq       --> SampleFrequency
%     Nrepeat          --> Nrepeat
%     Trial            --> Trial
%     Vcenter          --> Vcenter
%     Vradius          --> Vradius
%     MRI_ID           --> MRI_ID
%     MEGch_name       --> ChannelName
%     MEGch_id         --> ChannelID
%     ActiveChannel    --> ActiveChannel
%     ActiveTrial      --> ActiveTrial
%     device           --> Device
%     Measurement      --> Measurement
%     ChannelInfo      --> ChannelInfo
%     ExtraChannelInfo --> ExtraChannelInfo
%     (saveman)        --> File & DataType
%
function EEGinfo = inner_convert_MEGinfo2EEGinfo(MEGinfo)
  if ~isfield(MEGinfo, 'Nchannel'),        MEGinfo.Nchannel = []; end
  if ~isfield(MEGinfo, 'Nsample'),         MEGinfo.Nsample = []; end
  if ~isfield(MEGinfo, 'Pretrigger'),      MEGinfo.Pretrigger = []; end
  if ~isfield(MEGinfo, 'SampleFreq'),      MEGinfo.SampleFreq = []; end
  if ~isfield(MEGinfo, 'Nrepeat'),         MEGinfo.Nrepeat = []; end
  if ~isfield(MEGinfo, 'Trial'),           MEGinfo.Trial = []; end
  if ~isfield(MEGinfo, 'Vcenter'),         MEGinfo.Vcenter = []; end
  if ~isfield(MEGinfo, 'Vradius'),         MEGinfo.Vradius = []; end
  if ~isfield(MEGinfo, 'MEGch_name'),      MEGinfo.MEGch_name = []; end
  if ~isfield(MEGinfo, 'MEGch_id'),        MEGinfo.MEGch_id = []; end
  if ~isfield(MEGinfo, 'ActiveChannel'),   MEGinfo.ActiveChannel = []; end
  if ~isfield(MEGinfo, 'ActiveTrial'),     MEGinfo.ActiveTrial = []; end
  if ~isfield(MEGinfo, 'device'),          MEGinfo.device = []; end
  if ~isfield(MEGinfo, 'Measurement'),     MEGinfo.Measurement = []; end
  if ~isfield(MEGinfo, 'MRI_ID'),          MEGinfo.MRI_ID = []; end
  if ~isfield(MEGinfo, 'ChannelInfo'),     MEGinfo.ChannelInfo = []; end
  if ~isfield(MEGinfo, 'ExtraChannelInfo'),MEGinfo.ExtraChannelInfo = []; end
  if ~isfield(MEGinfo, 'saveman'),         MEGinfo.saveman = []; end

  EEGinfo.Nchannel      = MEGinfo.Nchannel;
  EEGinfo.Nsample       = MEGinfo.Nsample;
  EEGinfo.Pretrigger    = MEGinfo.Pretrigger;
  EEGinfo.SampleFrequency  = MEGinfo.SampleFreq;
  EEGinfo.Nrepeat       = MEGinfo.Nrepeat;
  EEGinfo.Trial         = MEGinfo.Trial;
  if isfield(MEGinfo, 'cond_id')
      EEGinfo.cond_id   = MEGinfo.cond_id;
  end
  EEGinfo.Vcenter       = MEGinfo.Vcenter;
  EEGinfo.Vradius       = MEGinfo.Vradius;
  EEGinfo.ChannelName   = MEGinfo.MEGch_name;
  EEGinfo.ChannelID     = MEGinfo.MEGch_id;
  EEGinfo.ActiveChannel = MEGinfo.ActiveChannel;
  EEGinfo.ActiveTrial   = MEGinfo.ActiveTrial;
  EEGinfo.Device        = MEGinfo.device;
  EEGinfo.Measurement   = MEGinfo.Measurement;
  EEGinfo.MRI_ID        = MEGinfo.MRI_ID;
  EEGinfo.ChannelInfo   = MEGinfo.ChannelInfo;
  EEGinfo.ExtraChannelInfo = MEGinfo.ExtraChannelInfo;

  if isfield(MEGinfo.saveman, 'precision')
      EEGinfo.DataType = repmat(MEGinfo.saveman.precision, MEGinfo.Nchannel+length(MEGinfo.ExtraChannelInfo.Channel_name),1);
  else
      EEGinfo.DataType = [];
  end

  if isfield(MEGinfo.saveman, 'data_dir') && ~isempty(MEGinfo.saveman.data_dir)
      [odir,ddir] = fileparts(MEGinfo.saveman.data_dir);
      EEGinfo.File.OutputDir = odir;
      EEGinfo.File.DataDir   = ddir;
  else
      EEGinfo.File = [];
  end

  return;
%
% --- end of inner_convert_EEGinfo2MEGinfo()
