function check_pick(p)
% Show position of the channels
% It requires bias-corrected T1 (mSubject.nii)
%
% Copyright (C) 2011, ATR All Rights Reserved.
% License : New BSD License(see VBMEG_LICENSE.txt)

% Set figure properties
set_fig_property(3, 3, 10, 10);
angles = [-90 0; 90 0; 0 90];
titles = {'Left view', 'Right view', 'Top view'};

% Set modality-specific parameters
switch p.dirname.modality
    case 'opm-meg'
        extention = '.meg.mat';
        ch_type = 'MEG';
    case 'squid-meg'
        extention = '.meg.mat';
        ch_type = 'MEG';
    case 'eeg'
        extention = '.eeg.mat';
        ch_type = {};
end

%% Load pick and Qpick of all channels including inactive channels.
run = 1; % Assume pick and Qpick are common across run
file_name = sprintf('run%02d', run);
meg_file = fullfile(p.proj_root, p.dirname.modality, p.dirname.load, p.task, [file_name extention]);
active_swt = false; % Load inactive ch as well
[pick, Qpick, CoilWeight, Vcenter, result, channel_info] = vb_load_sensor(meg_file, ch_type, [], active_swt);

if isempty(CoilWeight)
    pos = pick;
    dir = Qpick;
else
    Nch = size(CoilWeight, 1);
    pos = zeros(Nch,3);
    dir = zeros(Nch,3);
    for n=1:Nch
        pos(n,:) = mean(pick(CoilWeight(n, :) ~= 0, :), 1);
        dir(n,:) = mean(Qpick(CoilWeight(n, :) ~= 0, :), 1);
    end
end

Nsensor = size(pos, 1);

%% Calc scalp model from MRI
analyze_file = fullfile(p.proj_root, p.dirname.t1, ['m' p.struct_name '.nii']);
[surf_face] = vb_job_face(analyze_file);
sV = surf_face.V;
sF = surf_face.F;

NsV = size(sV, 1);

%% Calc dist between scalp and sensors
dis = zeros(Nsensor, 1);
for sensor = 1:Nsensor
    d = sum((repmat(pos(sensor, :), NsV, 1)-sV).^2, 2).^0.5;
    dis(sensor) = min(d);
end

% Range of figure
x_max = max(abs([sV(:,1); pos(:,1)]))+0.015;
y_max = max(abs([sV(:,2); pos(:,2)]))+0.015;
z_max = max(abs([sV(:,3); pos(:,3)]))+0.015;

%% Show pick and Qpick
lg = cell(4,1); % for customized legend
h = figure;
for an = 1:3
    subplot(2, 2, an), set_axis
    vb_plot_surf(sV, sF)
    alpha(0.5)
    hold on
    xlabel(''), ylabel(''), zlabel('')
    for sensor = 1:Nsensor
        % Sensor name and axis
        switch p.dirname.modality
            case 'opm-meg'
                sensor_name = regexprep(channel_info.Name{sensor}, '(_x|_y|_z)', '');
                [~,ixaxis] = regexp(channel_info.Name{sensor}, '(_x|_y|_z)');
                sensor_axis = channel_info.Name{sensor}(ixaxis);
                font_size = 0;
                pos_color = [0 0.4470 0.7410];
                switch sensor_axis
                    case 'x'
                        dir_coef = 1;
                        dir_color = [0.9290 0.6940 0.1250]; % yellow
                        lg{2} = plot3(nan,nan,nan, 'color', dir_color); % dummy plot for legend
                    case 'y'
                        dir_coef = 1;
                        dir_color = [0.8500 0.3250 0.0980]; % orange
                        lg{3} = plot3(nan,nan,nan, 'color', dir_color); % dummy plot for legend
                    case 'z'
                        % Flip direction in case of z-axis
                        dir_coef = -1;
                        dir_color = [0.4660 0.6740 0.1880]; % green
                        lg{4} = plot3(nan,nan,nan, 'color', dir_color); % dummy plot for legend
                end
            case 'squid-meg'
                sensor_name = channel_info.Name{sensor};
                sensor_type = channel_info.Type(sensor);
                font_size = 0;
                switch sensor_type
                    case 2 % AXIAL
                        pos_color = [0 0.4470 0.7410];
                        dir_coef = 1;
                        dir_color = [0.4660 0.6740 0.1880]; % green
                    case 3 % PLANAR
                        pos_color = [0.8500 0.3250 0.0980];
                        dir_coef = 0; % No dir plot for PLANAR
                        dir_color = [];
                        lg{2} = plot3(nan,nan,nan, 'o', 'markeredgecolor', [0.8500 0.3250 0.0980], 'markerfacecolor', [0.8500 0.3250 0.0980]); % To avoid deleting object(why?)
                end
            case 'eeg'
                sensor_name = channel_info.Name{sensor};
                pos_color = [0 0.4470 0.7410];
                dir_coef = 0;
                dir_color = []; 
                font_size = 0;
        end

        % Plot sensor position
        plot3(pos(sensor, 1), pos(sensor, 2), pos(sensor, 3), 'o', ...
            'markeredgecolor', pos_color, 'markerfacecolor', pos_color)
        hold on

        if dir_coef~=0
            % Plot direction
            plot3([pos(sensor, 1) pos(sensor, 1)+dir(sensor, 1)/50*dir_coef], ...
                [pos(sensor, 2) pos(sensor, 2)+dir(sensor, 2)/50*dir_coef], ...
                [pos(sensor, 3) pos(sensor, 3)+dir(sensor, 3)/50*dir_coef], ...
                'color', dir_color)
        end

        if font_size~=0
            % Plot sensor name
            text(pos(sensor, 1), pos(sensor, 2)-0.005, pos(sensor, 3), sensor_name, 'fontsize', font_size)
        end
    end
    view(angles(an, :))
    axis([-x_max x_max -y_max y_max -z_max z_max])
    % Add labels
    xlabel('X [m]')
    ylabel('Y [m]')
    zlabel('Z [m]')
    title(titles{an})
    set_axis
end

subplot(2, 2, 4)
% Show customized legend
switch p.dirname.modality
    case 'opm-meg'
        lg{1} = plot3(nan,nan,nan, 'o', 'markeredgecolor', pos_color, 'markerfacecolor', pos_color);
        lg_label = {'Cell', '+X axis', '+Y axis', '-Z axis'};
        legend([lg{:}], lg_label(~cellfun(@isempty, lg)), 'Location','north')
    case 'squid-meg'
        lg{1} = plot3(nan,nan,nan, 'o', 'markeredgecolor', [0 0.4470 0.7410], 'markerfacecolor', [0 0.4470 0.7410]);
        lg_label = {'AXIAL', 'PLANAR'};
        legend([lg{:}], lg_label(~cellfun(@isempty, lg)), 'Location','north')   
    case 'eeg'
        lg{1} = plot3(nan,nan,nan, 'o', 'markeredgecolor', pos_color, 'markerfacecolor', pos_color);
        lg_label = {'Sensor'};
        legend([lg{:}], lg_label(~cellfun(@isempty, lg)), 'Location','north')
end

axis off
title(['Mean distance from scalp = ' num2str(mean(dis*1000)) ' mm'])

%% Save figure
fig_file = fullfile(p.fig_root, mfilename, p.dirname.modality, p.sub);
vb_savefig_as_shown(h, fig_file)
disp([fig_file '.png was saved.'])
close all