function compare_preprocess(p, preproc1, preproc2, freq_of_interest)
% Comparing the preprocesseed data.
%
% Copyright (C) 2011, ATR All Rights Reserved.
% License : New BSD License(see VBMEG_LICENSE.txt)

disp(mfilename);

% Set directories
preproc1_dir = fullfile(p.proj_root, p.dirname.modality, preproc1, p.task);
preproc2_dir = fullfile(p.proj_root, p.dirname.modality, preproc2, p.task);

if ~exist('freq_of_interest','var')
    freq_of_interest = []; % Show all frequency
end

if isempty(freq_of_interest)
    sfoi = [];
else
    sfoi = ['(' num2str(freq_of_interest(1)) '-' num2str(freq_of_interest(2)) ')'];
end

% Set modality-specific parameters
switch p.dirname.modality
    case 'opm-meg'
        chtype = 'MEG';
        extention = '.meg.mat';
        plot_unit = 'T';
        plot_chtype = [5, 6]; % y and z
        plot_label = {'y', 'z'}; % label of saved fig
    case 'squid-meg'
        chtype = 'MEG';
        extention = '.meg.mat';
        plot_unit = 'T';
        plot_chtype = [2, 3]; % axial or planar
        plot_label = {'axial', 'planar'}; % label of saved fig
    case 'eeg'
        chtype = 'EEG';
        extention = '.eeg.mat';
        plot_unit = 'V';
        plot_chtype = 1; % eeg channel
        plot_label = {'eeg'}; % label of saved fig
end

for run = 1:p.num_run
    file_name = [sprintf('run%02d', run), extention];
    preproc1_file = fullfile(preproc1_dir, file_name);
    preproc2_file = fullfile(preproc2_dir, file_name);

    loadspec = [];
    loadspec.ActiveChannel = false; % Load all ch

    % Load MEG/EEG data of preprocess1
    [bexp1, ch_info1, time_info1] = vb_load_meg_data(preproc1_file, loadspec);
    time1 = time_info1.time;
    sfreq1 = time_info1.sample_frequency;

    % Load MEG/EEG data of preprocess2
    [bexp2, ch_info2, time_info2] = vb_load_meg_data(preproc2_file, loadspec);
    time2 = time_info2.time;
    sfreq2 = time_info2.sample_frequency;

    % If bexp is already epoched, make it flat
    if size(bexp1,3)~=1
        bexp1 = reshape(bexp1, [size(bexp1,1), size(bexp1,2)*size(bexp1,3)]);
        dt = 1/sfreq1;
        time1 = 0:dt:(size(bexp1,2)-1)*dt; % Make long time
    end
    if size(bexp2,3)~=1
        bexp2 = reshape(bexp2, [size(bexp2,1), size(bexp2,2)*size(bexp2,3)]);
        dt = 1/sfreq2;
        time2 = 0:dt:(size(bexp2,2)-1)*dt; % Make long time
    end

    % Make sampling freqs consistent
    if sfreq1==sfreq2
        disp('Sampling freqs are matched')
        time = time1;
        sfreq = sfreq1;
    elseif sfreq1>sfreq2
        disp(['Down sampling ' preproc1])
        % tindex = dsearchn(time1.time', time2.time'); % takes minutes...
        % bexp1 = bexp1(tindex);
        bexp1 = vb_convert_freq(bexp1, sfreq1, sfreq2);
        time = time2;
        sfreq = sfreq2;
    else
        disp(['Down sampling ' preproc2])
        % tindex = dsearchn(time2.time', time1.time'); % takes minutes...
        % bexp2 = bexp2(tindex);
        bexp2 = vb_convert_freq(bexp2, sfreq2, sfreq1); 
        time = time1;
        sfreq = sfreq1;
    end

    [nch, ntime] = size(bexp1);

    % Make inactive channels to 0
    bexp1(~ch_info1.Active,:) = zeros(sum(~ch_info1.Active), ntime);
    bexp2(~ch_info2.Active,:) = zeros(sum(~ch_info2.Active), ntime);

    % Demean
    % bexp1 = bsxfun(@minus, bexp1, mean(bexp1,2));
    % bexp2 = bsxfun(@minus, bexp2, mean(bexp2,2));

    % Load positions of all channels
    [pos, ~] = vb_load_channel(preproc1_file, false);
    % Show results of denoising
    set_fig_property(5, 3, 10, 10);

    % Loop for ch type (in the case of OPM, each axis will be shown seperately)
    for cc = 1:length(plot_chtype)
        chtype = plot_chtype(cc);
        chlabel = plot_label{cc};
        h = inner_plot(bexp1, preproc1, bexp2, preproc2, pos, ch_info1, time, sfreq, plot_unit, chtype, freq_of_interest);
        fig_file = fullfile(p.fig_root, mfilename, p.task, p.dirname.modality, [preproc1 '-' preproc2], [p.sub '_run' num2str(run) '_' chlabel sfoi '.png']);
        vb_savefig_as_shown(h, fig_file)
        close(h)
    end
    
end

end % End of function

function h = inner_plot(y1, y1_title, y2, y2_title, pos, channel_info, time, sampling_freq, unit_disp, channel_type, foi)
% Comparing two preprocessed data
h = figure; hold on

% Extract data with specific ch type
ix_ch = find(channel_info.Type == channel_type);
pos = pos(ix_ch, :);
y1 = y1(ix_ch, :);
y2 = y2(ix_ch, :);

% Compute PSD
[freq, psdx1] = calc_psd(y1, sampling_freq, @hanning, 2000, 'psds');
[~,    psdx2] = calc_psd(y2, sampling_freq, @hanning, 2000, 'psds');
% Difference of PSD
% dpsdx = psdx1 - psdx2;
[~,    dpsdx] = calc_psd(y1-y2, sampling_freq, @hanning, 2000, 'psds');

if ~isempty(foi)
    % Apply filtering on time-series
    [parm_filter] = struct(...
        'bandpass_freq', foi,...
        'bandpass_filtertype' , 'butter',...
        'bandpass_filterorder', 4);
    y1 = filter2d(y1, sampling_freq, parm_filter, 0);
    y2 = filter2d(y2, sampling_freq, parm_filter, 0);

    % Get index of freqency
    [~, ixfoi(1)] = min(abs(freq-foi(1)));
    [~, ixfoi(2)] = min(abs(freq-foi(2)));
    title_add = [newline '(' num2str(foi(1)) '-' num2str(foi(2)) 'Hz)'];
else
    ixfoi(1) = 1;
    ixfoi(2) = length(freq);
    foi = [freq(ixfoi(1)) freq(ixfoi(2))];
    title_add = [''];
end

% Difference of (filtered) time-series
dy = y1 - y2;

% Calculate power
% p1 = mean(y1.^2, 2);
% p2 = mean(y2.^2, 2);
p1 = mean(psdx1(ixfoi(1):ixfoi(2),:), 1)';
p2 = mean(psdx2(ixfoi(1):ixfoi(2),:), 1)';
% Difference of power
dp = p1-p2;

%% Set parameters of plots
% Options of PSD
opt = [];
opt.const = [];
opt.dunit = unit_disp;
opt.sunit = 'psds';
opt.funit = '';

% Get minimum and maximum values of power
ma = max([p1(:); p2(:)]);
mi = min([p1(:); p2(:)]);

% Prepare a tiledlayout
t = tiledlayout(6,10, 'TileSpacing','loose', 'Padding','compact');

%% Plot data1
nexttile([2,1])
text(0,0,[y1_title title_add],'FontSize',14)
ylim([-1 1])
ax = gca;
ax.XAxis.Visible = 'off';
ax.YAxis.Visible = 'off';

nexttile([2,4])
plot(time, y1')
xlim([min(time) max(time)]), ylim([min(y1(:)) max(y1(:))])
ylabel(unit_disp)
set_axis
title('Time-series')

nexttile([2,3])
plot_psd(freq, psdx1, opt)
xlim(foi)
set_axis
title('PSD')

nexttile([2,2])
vb_plot_sensor_2d(pos, p1, [mi ma], true);
axis square off
% colorbar
colormap(my_parula())
title('Topography')

%% Plot data2
nexttile([2,1])
text(0,0,[y2_title title_add],'FontSize',14)
ylim([-1 1])
ax = gca;
ax.XAxis.Visible = 'off';
ax.YAxis.Visible = 'off';

nexttile([2,4])
plot(time, y2')
xlim([min(time) max(time)]), ylim([min(y2(:)) max(y2(:))])
ylabel(unit_disp)
set_axis

nexttile([2,3])
plot_psd(freq, psdx2, opt)
xlim(foi)
set_axis

nexttile([2,2])
vb_plot_sensor_2d(pos, p2, [mi ma], true);
axis square off
% colorbar
colormap(my_parula())

%% Plot data1-data2
nexttile([2,1])
text(0,0,'Diff', 'FontSize',14)
ylim([-1 1])
ax = gca;
ax.XAxis.Visible = 'off';
ax.YAxis.Visible = 'off';

nexttile([2,4])
plot(time, dy')
xlim([min(time) max(time)]), ylim([min(dy(:)) max(dy(:))])
ylabel(unit_disp)
set_axis

nexttile([2,3])
plot_psd(freq, dpsdx, opt)
xlim(foi)
set_axis

nexttile([2,2])
vb_plot_sensor_2d(pos, dp, [mi ma], true);
axis square off
% colorbar
colormap(my_parula())

% Append colorbar
cb = colorbar;
cb.Layout.Tile = 'east';

end % End of function
