function compare_processed_psd(p, INPUTS, opt_ref, opt_active)
% Show processed results
%
% Copyright (C) 2011, ATR All Rights Reserved.
% License : New BSD License(see VBMEG_LICENSE.txt)

disp(mfilename);

% Set figure properties
set_fig_property(5, 3, 10, 10);

% Option for plotting a reference channels
%   'WITH_REF' : Plot the MEG channels and reference channels
%   'NO_REF'   : Plot the MEG channels only
%   'ONLY_REF' : Plot the reference channels only
% Defalt: 'NO_REF'
if ~exist('opt_ref', 'var') || isempty(opt_ref)
    opt_ref = 'NO_REF';
end

% Option for plotting inactive channels
%   true  : Plot active channels only
%   false : Plot inactive channels as well
% Default: true
if ~exist('opt_active', 'var') || isempty(opt_active)
    opt_active = true;
end

% Set modality-wise parameters
switch p.dirname.modality
    case 'opm-meg'
        chtype = 'MEG';
        extention = '.meg.mat';
        unit_coef = 10^15; % T -> fT
        unit_disp = 'fT';
        noise_floor = 15; % In case of OPM, noise floor is 15 [fT/sqrt(HZ)]
    case 'squid-meg'
        chtype = 'MEG';
        extention = '.meg.mat';
        unit_coef = 10^15; % T -> fT
        unit_disp = 'fT';
        noise_floor = [];
    case 'eeg'
        chtype = 'EEG';
        extention = '.eeg.mat';
        unit_coef = 10^3; % V -> mV
        unit_disp = 'mV';
        noise_floor = [];
end

Nmethod = length(INPUTS);
line_colors = my_lines(Nmethod); % Get line colors

for run = 1:p.num_run
    h = figure;

    for mm = 1:Nmethod
        dir_input = INPUTS{mm};
        file_name = sprintf('run%02d', run);
        data_file = fullfile(p.proj_root, p.dirname.modality, dir_input, p.task, [file_name extention]);
    
        %% Load data
        loadspec = [];
        loadspec.ActiveChannel = opt_active;
        [y, channel_info, time_info] = vb_load_meg_data(data_file, loadspec);
        chnames = channel_info.Name;
        time = time_info.time;
        fs = time_info.sample_frequency;

        % Load reference
        if strcmp(chtype, 'MEG')
            loadspec_ref.ChannelType = 'REFERENCE';
            [y_ref, channel_info_ref, ~] = vb_load_meg_data(data_file,loadspec_ref);
        else
            % EEG has no ref
            y_ref = [];
            channel_info_ref = [];
        end
        if isempty(channel_info_ref)
            chnames_ref = [];
        else
            chnames_ref = channel_info_ref.Name;
        end
        
        switch opt_ref
            case 'WITH_REF'
                y_all = [y; y_ref];
                chnames_all = [chnames; chnames_ref];
            case 'NO_REF'
                y_all = y;
                chnames_all = chnames;
            case 'ONLY_REF'
                y_all = y_ref;
                chnames_all = chnames_ref;
            otherwise
                y_all = y;
                chnames_all = chnames;
        end

        % Unit conversion
        y_all = y_all.*unit_coef;

        %% Plot averaged power spectrum
        % Set unit
        data_unit = unit_disp;  % Unit of original data
        spect_unit = 'psds'; % Unit of resulted PSD (see calc_psd)

        % Set window length [ms]
        % If data has multiple trials (dim3~=1), len_win_ms is ignored
        len_win_ms = 2000;

        % Set window function
        % Format of window functon: func(window_length, options)
        func_win = @hanning;

        % Constant line draw in the figure
        constant = noise_floor;

        % ch time trial
        [freq, psdx] = calc_psd(y_all, fs, func_win, len_win_ms, spect_unit);
        
        % Average across ch
        psdx_mean = mean(psdx,2);
        
        % Plot params
        opt = [];
        if mm==Nmethod
            opt.const = constant;
        end
        opt.dunit = data_unit;
        opt.sunit = spect_unit;
        opt.line_color = line_colors(mm,:);

        % Plot 
        % For lower freq
        subplot(1, 9, [1,2])
        opt.funit = '';
        plot_psd(freq, psdx_mean, opt)
        % For higher freq
        subplot(1, 9, [4,5,6,7,8,9])
        opt.sunit = '';
        opt.funit = 'Hz';
        plot_psd(freq, psdx_mean, opt)

    end
    % For lower freq
    subplot(1, 9, [1,2])
    foi = [0; 3];
    xlim(foi)
    xticks([0 1 2 3])
    xticklabels({'0' '1' '2' '3'})
    set_axis

    % For higher freq
    subplot(1, 9, [4,5,6,7,8,9])
    foi = [3; 70];
    xlim(foi)
    xticks([3 10 20 30 40 50 60 70])
    xticklabels({'3' '10' '20' '30' '40' '50' '60' '70'})
    set_axis
    
    for mm=1:Nmethod
        qw{mm} = plot(nan, 'LineWidth',2, 'Color', line_colors(mm,:));
    end
    if isempty(noise_floor)
        lgn = INPUTS;
    else
        lgn = [INPUTS 'noise floor'];
        qw{Nmethod+1} = plot(nan, '--k', 'LineWidth',2);
    end
    legend([qw{:}], lgn , 'location', 'northeastoutside')

    %% Save figure
    fig_file = fullfile(p.fig_root, mfilename, p.dirname.modality, p.task, [p.sub '_run' num2str(run)]);
    %vb_savefig_as_shown(h, fig_file, '-depsc')
    vb_savefig_as_shown(h, fig_file, '-dpng')
    disp([fig_file '.png was saved.'])

end