function show_processed_data(p, dir_input, opt_ref, opt_active, foi)
% Show processed results
%
% Copyright (C) 2011, ATR All Rights Reserved.
% License : New BSD License(see VBMEG_LICENSE.txt)

disp(mfilename);

% Set figure properties
set_fig_property(3, 2, 10, 10);

% Option for plotting a reference channels
%   'WITH_REF' : Plot the MEG channels and reference channels
%   'NO_REF'   : Plot the MEG channels only
%   'ONLY_REF' : Plot the reference channels only
% Defalt: 'NO_REF'
if ~exist('opt_ref', 'var') || isempty(opt_ref)
    opt_ref = 'NO_REF';
end

% Option for plotting inactive channels
%   true  : Plot active channels only
%   false : Plot inactive channels as well
% Default: true
if ~exist('opt_active', 'var') || isempty(opt_active)
    opt_active = true;
end

% Set freq of interests
if ~exist('foi', 'var') || isempty(foi)
    foi = [1; 100];
end

% Set modality-wise parameters
switch p.dirname.modality
    case 'opm-meg'
        chtype = 'MEG';
        extention = '.meg.mat';
        unit_coef = 10^15; % T -> fT
        unit_disp = 'fT';
        noise_floor = 15; % In case of OPM, noise floor is 15 [fT/sqrt(HZ)]
    case 'squid-meg'
        chtype = 'MEG';
        extention = '.meg.mat';
        unit_coef = 10^15; % T -> fT
        unit_disp = 'fT';
        noise_floor = [];
    case 'eeg'
        chtype = 'EEG';
        extention = '.eeg.mat';
        unit_coef = 10^3; % V -> mV
        unit_disp = 'mV';
        noise_floor = [];
end

for run = 1:p.num_run
    file_name = sprintf('run%02d', run);
    data_file = fullfile(p.proj_root, p.dirname.modality, dir_input, p.task, [file_name extention]);
    
    %% Load data
    loadspec = [];
    loadspec.ActiveChannel = opt_active;
    [y, channel_info, time_info] = vb_load_meg_data(data_file, loadspec);
    chnames = channel_info.Name;
    time = time_info.time;
    fs = time_info.sample_frequency;

    % Load reference
    if strcmp(chtype, 'MEG')
        loadspec_ref.ChannelType = 'REFERENCE';
        [y_ref, channel_info_ref, ~] = vb_load_meg_data(data_file,loadspec_ref);
    else
        % EEG has no ref
        y_ref = [];
        channel_info_ref = [];
    end
    if isempty(channel_info_ref)
        chnames_ref = [];
    else
        chnames_ref = channel_info_ref.Name;
    end
    
    switch opt_ref
        case 'WITH_REF'
            y_all = [y; y_ref];
            chnames_all = [chnames; chnames_ref];
        case 'NO_REF'
            y_all = y;
            chnames_all = chnames;
        case 'ONLY_REF'
            y_all = y_ref;
            chnames_all = chnames_ref;
        otherwise
            y_all = y;
            chnames_all = chnames;
    end

    [nch, ntime, ntrial] = size(y_all);
    % If data is already epoched, make it flat
    if ntrial~=1
        y_all = reshape(y_all, [nch, ntime*ntrial]);
        dt = 1/fs;
        time = 0:dt:(ntime*ntrial-1)*dt; % Make long time
    end
    
    line_colors = my_lines(nch); % Get line colors
    line_colors = min(line_colors+0.0, 1); % Make color lighter with upper bound=1
    
    % Unit conversion
    y_all = y_all.*unit_coef;

    %% Plot time series
    h = figure;
    subplot(2,1,1), hold on
    max_y = max(y_all(:));
    min_y = min(y_all(:));
    for nn=1:nch
        plot(time, y_all(nn,:), 'LineWidth',0.1, 'Color',line_colors(nn,:))
    end
    plot(time, mean(y_all), 'LineWidth',0.5, 'Color','k')
    xlim([time(1) time(end)])
    xlabel('Time [s]')
    ylabel(['Magnetic field [' unit_disp ']'])
    title([dir_input ' Run-' num2str(run)])
    set_axis
    grid on
    
    %% Plot power spectrum
    % Set unit
    data_unit = unit_disp;  % Unit of original data
    spect_unit = 'psds'; % Unit of resulted PSD (see calc_psd)
    
    % Set window length [ms]
    % If data has multiple trials (dim3~=1), len_win_ms is ignored
    len_win_ms = 2000;
    
    % Set window function
    % Format of window functon: func(window_length, options)
    func_win = @hanning;
    
    % Constant line draw in the figure
    constant = noise_floor;

    % ch time trial
    [freq, psdx] = calc_psd(y_all, fs, func_win, len_win_ms, spect_unit);
    
    % Plot params
    opt = [];
    opt.const = constant;
    opt.dunit = data_unit;
    opt.sunit = spect_unit;
    
    % Plot with Freq of interests
    subplot(2,1,2),
    plot_psd(freq, psdx, opt)
    xlim(foi)
    set_axis
    
    %% Save figure
    fig_file = fullfile(p.fig_root, mfilename, p.dirname.modality, p.task, [p.sub '_' dir_input '_run' num2str(run)]);
    vb_savefig_as_shown(h, fig_file, '-dpng')
    disp([fig_file '.png was saved.'])
    
end
