function show_processed_data_row(p, dir_input, opt_ref, opt_active, foi)
% Show processed results
%
% Copyright (C) 2011, ATR All Rights Reserved.
% License : New BSD License(see VBMEG_LICENSE.txt)

disp(mfilename);

% Set figure properties
set_fig_property(4, 3, 5, 5);

% Option for plotting a reference channels
%   'WITH_REF' : Plot the MEG channels and reference channels
%   'NO_REF'   : Plot the MEG channels only
%   'ONLY_REF' : Plot the reference channels only
% Defalt: 'NO_REF'
if ~exist('opt_ref', 'var') || isempty(opt_ref)
    opt_ref = 'NO_REF';
end

% Option for plotting inactive channels
%   true  : Plot active channels only
%   false : Plot inactive channels as well
% Default: true
if ~exist('opt_active', 'var') || isempty(opt_active)
    opt_active = true;
end

% Set freq of interests
if ~exist('foi', 'var') || isempty(foi)
    foi = [1; 100];
end

% Set modality-wise parameters
switch p.dirname.modality
    case 'opm-meg'
        chtype = 'MEG';
        extention = '.meg.mat';
        unit_coef = 10^15; % T -> fT
        unit_disp = 'fT';
        noise_floor = 15; % In case of OPM, noise floor is 15 [fT/sqrt(HZ)]
    case 'squid-meg'
        chtype = 'MEG';
        extention = '.meg.mat';
        unit_coef = 10^15; % T -> fT
        unit_disp = 'fT';
        noise_floor = [];
    case 'eeg'
        chtype = 'EEG';
        extention = '.eeg.mat';
        unit_coef = 10^3; % V -> mV
        unit_disp = 'mV';
        noise_floor = [];
end

for run = 1:p.num_run
    file_name = sprintf('run%02d', run);
    data_file = fullfile(p.proj_root, p.dirname.modality, dir_input, p.task, [file_name extention]);
    
    %% Load data
    loadspec = [];
    loadspec.ActiveChannel = opt_active;
    [y, channel_info, time_info] = vb_load_meg_data(data_file, loadspec);
    chnames = channel_info.Name;
    time = time_info.time;
    fs = time_info.sample_frequency;

    % Load reference
    if strcmp(chtype, 'MEG')
        loadspec_ref.ChannelType = 'REFERENCE';
        [y_ref, channel_info_ref, ~] = vb_load_meg_data(data_file,loadspec_ref);
    else
        % EEG has no ref
        y_ref = [];
        channel_info_ref = [];
    end
    if isempty(channel_info_ref)
        chnames_ref = [];
    else
        chnames_ref = channel_info_ref.Name;
    end
    
    switch opt_ref
        case 'WITH_REF'
            y_all = [y; y_ref];
            chnames_all = [chnames; chnames_ref];
        case 'NO_REF'
            y_all = y;
            chnames_all = chnames;
        case 'ONLY_REF'
            y_all = y_ref;
            chnames_all = chnames_ref;
        otherwise
            y_all = y;
            chnames_all = chnames;
    end

    [nch, ntime, ntrial] = size(y_all);
    % If data is already epoched, make it flat
    if ntrial~=1
        y_all = reshape(y_all, [nch, ntime*ntrial]);
        dt = 1/fs;
        time = 0:dt:(ntime*ntrial-1)*dt; % Make long time
    end
    
    % Unit conversion
    y_all = y_all.*unit_coef;

    %% Plot time series
    twin_sec = [50 51]; % [sec]
    [~,tix(1)] = min(abs(time-twin_sec(1)));
    [~,tix(2)] = min(abs(time-twin_sec(2)));
    tix = tix(1):tix(2);

    h = figure;
    y_plot = y_all(:,tix);
    plot_rows(y_plot, chnames_all);
    ax=gca;
    ax.FontUnits = 'normalized';
    ax.FontSize = 1/nch;
    set(gca, 'XTick', [0 tix(end)-tix(1)+1], 'XTickLabel',{num2str(twin_sec(1)),num2str(twin_sec(2))}, 'tickdir', 'out')
    xlabel('Time[sec]')
    title([dir_input ' (1sec)'])

    %% Save figure
    fig_file = fullfile(p.fig_root, mfilename, p.dirname.modality, p.task, [p.sub '_' dir_input '_run' num2str(run)]);
    vb_savefig_as_shown(h, fig_file, '-dpng')
    disp([fig_file '.png was saved.'])
end
