function show_source_current_SOLCO(p, METHOD)
% Show source current estimated from MEG data
% METHOD : Method can be chosen from 
%          {'wmn', 'lcmv', 'mc', 'loreta'}
%
% Copyright (C) 2011, ATR All Rights Reserved.
% License : New BSD License(see VBMEG_LICENSE.txt)

disp(mfilename);
close all

% Set default method
if ~exist('METHOD', 'var') || isempty(METHOD)
    METHOD = {'lcmv'};
end
if ~iscell(METHOD)
    METHOD = {METHOD};
end

% Set figure properties
set_fig_property(4, 2, 10, 10);

% For projecting Jmap on restricted area to whole cortex
brain_file = fullfile(p.proj_root, p.dirname.brain, [p.struct_name, '.brain.mat']);
[V] = vb_load_cortex(brain_file, 'Inflate');
nvertex =  size(V,1); % All vertices of whole cortex

% Angles to see brain
angle = [-90 0; 90 0; 0 90];

% Set dir of currents
current_dir = fullfile(p.proj_root, p.dirname.modality, p.dirname.current);

% Loop for each method
for me=1:length(METHOD)

method = METHOD{me};

% Set dir and prefix of current file
prefix_fileid = [p.task '_' method '_'];

% Find current files having specified fileid in current_dir
% current_files is struct
current_files = dir(fullfile(current_dir, [prefix_fileid '*' '.curr.mat']));

for ff=1:length(current_files)
filename = current_files(ff).name;
current_file = fullfile(current_dir, filename);

% Load temporal peak
load(current_file, 'time_win_sec')

% Set time window [sec]
twin_show_sec   = p.time_show_sec;
twin_signal_sec = time_win_sec;
twin_noise_sec  = p.time_noise_sec;

% Load inverse filter 
J = load(current_file);
inv_filt = J.Model.IF;

% Load MEG/EEG data
if isfield(J, 'datafile')
    % Case of EEG or MEG estimation
    [bexp, ~, time_info] = vb_load_meg_data(J.datafile);
    % Trial-averaging
    bexp_ave = mean(bexp, 3);

else
    % Case of MEEG estimation
    area_file  = fullfile(p.proj_root, p.dirname.brain, [p.struct_name, '.area.mat']);
    temp_parm = [];
    temp_parm.brainfile = brain_file;
    temp_parm.areafile  = area_file;
    temp_parm.area_key  = 'Cortex';
    temp_parm.reduce    = 1;
    temp_parm.Rfilt     = 0;
    temp_parm.patch_norm = ON;

    % Parepare MEG
    temp_parm.basisfile = J.megbasisfile;
    [B, ~, time_info] = vb_load_meg_data(J.megfile);
    Gb = vb_focal_leadfield_preparation(temp_parm,1); % without leadfield scaling
    % Prepare EEG
    temp_parm.basisfile = J.eegbasisfile;
    E = vb_load_meg_data(J.eegfile);
    Ge = vb_focal_leadfield_preparation(temp_parm,1); % without leadfield scaling

    % Trial-averaging
    Bave = mean(B, 3);
    Eave = mean(E, 3);

    % --- Combining MEG/EEG data with normalization ---
    bsnorm_meg = sqrt(mean(mean(Gb{1}.^2,2)));
    bsnorm_eeg = sqrt(mean(mean(Ge{1}.^2,2)));
    bexp_ave = [Bave./bsnorm_meg; Eave./bsnorm_eeg]; % Normalized the scale of data
end

time_sec = time_info.time;

% Convert sec to index
[~, from_show_ix]   = min(abs(time_sec-twin_show_sec(1)));
[~, to_show_ix]     = min(abs(time_sec-twin_show_sec(2)));
[~, from_signal_ix] = min(abs(time_sec-twin_signal_sec(1)));
[~, to_signal_ix]   = min(abs(time_sec-twin_signal_sec(2)));
[~, from_noise_ix]  = min(abs(time_sec-twin_noise_sec(1)));
[~, to_noise_ix]    = min(abs(time_sec-twin_noise_sec(2)));

% Extract bexp and time for showing
bexp_show = bexp_ave(:, from_show_ix:to_show_ix);
time_show_sec = time_sec(from_show_ix:to_show_ix);

% Current reconstruction using inv filter
cur = inv_filt * bexp_show;

% Project current time-series to whole cortex
ntime = size(cur, 2);
Jtime = zeros(nvertex, ntime);
Jtime(J.ix_act,:) = cur; 

% Calc noise normalized current map
switch method
    case {'wmn', 'loreta', 'mc', 'lcmv'}
        Varact = zeros(nvertex,1);
        Varact(J.ix_act) = J.Varact;
        Jtime = Jtime ./ repmat( sqrt(Varact(:)), [1,ntime] );
        % Extract ToI
        Jtime_toi = Jtime(:,from_signal_ix:to_signal_ix);
        Jmap = mean(abs(Jtime_toi),2);
        % Resact = zeros(nvertex,1);
        % Resact(J.ix_act) = J.Resact; % Keep resolution kernel for plotting sLORETA
    case 'sloreta' 
        error('sloreta is not supported yet')
end

%% Plot
fig_prefix = [method '_'];
fig_suffix = ['_' num2str(ff)];
toi = twin_signal_sec;
disp_thresh = max(Jmap)*0.3;
label_time = '';
label_map = [method ' map (>' num2str(disp_thresh,'%.2e') ')'];

% Plot for temporal-averaged data (within ToI)
fig_file = fullfile(p.fig_root, mfilename, p.dirname.modality, p.task, [fig_prefix p.sub fig_suffix]);
inner_plot_and_save_figure(brain_file, angle, time_show_sec, Jtime, Jmap, disp_thresh, toi, label_time, label_map, fig_file)

end % end of peak loop

end % End of method loop

end % end of function

function inner_plot_and_save_figure(brain_file, angle, time_show, ytime, ymap, disp_thresh, time_of_interest, label_time, label_map, fig_file)
% Plot time series of averaged MEG data
h = figure;
max_y = max(ytime(:));
min_y = min(ytime(:));
if time_of_interest(1)==time_of_interest(2)
    time_of_interest = time_of_interest(1);
end

subplot(2, 1, 1)
if length(time_of_interest)==2
    % Add filled region
    ar=area([time_of_interest(1) time_of_interest(2)],[max_y max_y], min_y);
    set(ar,'FaceColor',[.7 .7 .7]);
    set(ar,'LineStyle','none');
    set(gca,'layer','top');
end          
hold on
% Plot time-series
plot(time_show, ytime, 'LineWidth',0.05)
%plot(time_show, ytime, 'LineWidth',0.05, 'Color',[0.7 0.7 0.7])
xlabel('Time [s]')
ylabel(label_time)
title(['Stimulus-triggered average of current'])
axis([time_show(1) time_show(end) min_y max_y])
grid on
if length(time_of_interest)==1
    % Add yline since filled region is not plotted
    plot([time_of_interest time_of_interest], [min_y max_y], 'k-', 'LineWidth',1.0)
end
set_axis

% Load brain model on which current is shown
[V, F, xx, inf_C] = vb_load_cortex(brain_file, 'Inflate');
max_xyz = max(abs(V), [], 1);
plot_parm = vb_set_plot_parm;

% Plot spatial pattern of absolute value of current
for an = 1:size(angle, 1)
    subplot(2, size(angle, 1), size(angle, 1)+an)
    vb_plot_cortex(plot_parm, V, F, inf_C, ymap, disp_thresh, max(ymap));
    axis([-max_xyz(1) max_xyz(1) -max_xyz(2) max_xyz(2) -max_xyz(3) max_xyz(3)])
    axis equal off
    view(angle(an, :))
    %zoom(1.5)
    if an==2
        if length(time_of_interest)==2
            title([label_map ' (' num2str(time_of_interest(1)) ' - ' num2str(time_of_interest(2)) ' sec)'])
        else
            title([label_map ' (' num2str(time_of_interest(1)) ' sec)'])
        end
    end
    if an==3
        colorbar
    end
end
colormap(my_parula())

% Save figure
vb_savefig_as_shown(h, fig_file)
disp([fig_file '.png was saved.'])

end % end of function