function show_source_current(p, normalize_method)
% Show source current estimated from MEG data
%
% normalize_method : Method for normalization.
%                    Multiple methods are selectable at once.
%                    0 : No normalization
%
% Copyright (C) 2011, ATR All Rights Reserved.
% License : New BSD License(see VBMEG_LICENSE.txt)

disp(mfilename);
close all

if ~exist('normalize_method','var') || isempty(normalize_method)
    normalize_method = 0;
end

% Set figure properties
set_fig_property(4, 2, 10, 10);

% Set brain file
brain_file = fullfile(p.proj_root, p.dirname.brain, [p.struct_name, '.brain.mat']);

% Angles to look brain from
angle = [-90 0; 90 0; 0 90];

% Set dir and prefix of current file
prefix_fileid = [p.task '_' p.prior strrep(num2str(p.prior_weight),'.','') '_'];
current_dir = fullfile(p.proj_root, p.dirname.modality, p.dirname.current);

% Find current files having specified fileid in current_dir
% current_files is struct
current_files = dir(fullfile(current_dir, [prefix_fileid '*' '.curr.mat']));

for ff=1:length(current_files)
filename = current_files(ff).name;
fileid = [p.prior strrep(num2str(p.prior_weight),'.','') '_' num2str(ff)];

current_file = fullfile(current_dir, filename);

% Load temporal peak
load(current_file, 'time_win_sec')

% Set time window [sec]
twin_show_sec   = p.time_show_sec;
twin_signal_sec = time_win_sec;
twin_noise_sec  = p.time_noise_sec;

% Load time information
Jinfo = vb_load_current(current_file);
time_sec = Jinfo.Tmsec/1000; % All time-series of current data [sec]

% Convert sec to index
[~, from_show_ix]   = min(abs(time_sec-twin_show_sec(1)));
[~, to_show_ix]     = min(abs(time_sec-twin_show_sec(2)));
[~, from_signal_ix] = min(abs(time_sec-twin_signal_sec(1)));
[~, to_signal_ix]   = min(abs(time_sec-twin_signal_sec(2)));
[~, from_noise_ix]  = min(abs(time_sec-twin_noise_sec(1)));
[~, to_noise_ix]    = min(abs(time_sec-twin_noise_sec(2)));

time_show = time_sec(from_show_ix:to_show_ix); % Time-series of showing interval

% Load currents
% In case of method is only 0 and/or 1, load trial-averaged currents for saving the time 
if all(ismember(normalize_method, [0,1]))
    [~, current] = vb_load_current(current_file, 2, ON);
else
    [~, current] = vb_load_current(current_file, 2, OFF); % It takes time due to trial_average=OFF
end

% Separate currens into signal and noise interval
y  = current(:, from_show_ix:to_show_ix, :); % All currents
ys = current(:, from_signal_ix:to_signal_ix, :); % Signal currents in ToI
yn = current(:, from_noise_ix:to_noise_ix, :); % Noise currents before onset

% Average source currents across trials
y_ave  = mean(y, 3); % Trial-ave all currents
ys_ave = mean(ys, 3); % Trial-ave for signal
yn_ave = mean(yn, 3); % Trial-ave for noise

% |yave|
y_ave_abs  = abs(y_ave);
ys_ave_abs = abs(ys_ave);
yn_ave_abs = abs(yn_ave);

N  = size(y_ave_abs, 2);
Ns = size(ys_ave_abs, 2);
Nn = size(yn_ave_abs, 2);

%% Plot data with specified normalize_method
for nn=1:length(normalize_method)
    method = normalize_method(nn);
    switch method
        case 0 % Absolute-mean value of trial-averaged current
            yys = ys_ave_abs; % for peak plot
            ymap = mean(yys, 2); % absolute current map (averaged in ToI)
            % Plot
            fig_prefix = [];
            fig_suffix = ['_' fileid];
            toi = twin_signal_sec;
            ytime = y_ave;
            disp_thresh = max(ymap)*0.3;
            label_time = 'Current [Am/m^2]';
            label_map = ['Absolute averaged current (>' num2str(disp_thresh,'%.2e') ')'];

        case 1 % Normalization using std of trial-averaged noise
            yn_ave_std = std(yn_ave, 0, 2);
            % Normalize ys_ave_abs using yn_ave_std
            yys = ys_ave_abs ./ repmat(yn_ave_std, [1,Ns]);
            ymap = mean(yys, 2);
            % For timeseries plot
            yy = y_ave_abs ./ repmat(yn_ave_std, [1,N]);
            % Plot
            fig_prefix = ['z1_'];
            fig_suffix = ['_' fileid];
            toi = twin_signal_sec;
            ytime = yy;
            disp_thresh = max(ymap)*0.3;
            label_time = 'Normalized current [a.u.]';
            label_map = ['Normalized averaged current (>' num2str(disp_thresh,'%.2e') ')'];

        case 2 % Normalization using trial-variance of averaged-noise
            % Average noise currents for temporal direction
            yn_tave = squeeze(mean(yn, 2)); % Temporal-ave for noise
            % Calc std across trials
            yn_tave_std = std(yn_tave, 0, 2);
            % Normalize ys_ave_abs using yn_tave_std
            yys = ys_ave_abs ./ repmat(yn_tave_std, [1,Ns]);
            ymap = mean(yys, 2);
            % For timeseries plot
            yy = y_ave_abs ./ repmat(yn_tave_std, [1,N]);
            % Plot
            fig_prefix = ['z2_'];
            fig_suffix = ['_' fileid];
            toi = twin_signal_sec;
            ytime = yy;
            disp_thresh = max(ymap)*0.3;
            label_time = 'Normalized current [a.u.]';
            label_map = ['Normalized averaged current (>' num2str(disp_thresh,'%.2e') ')'];

        case 3 % Normalization using variance of concat noise
            % Concat trial and time for each vertex
            yn_long = reshape(yn, [size(yn,1), size(yn,2)*size(yn,3)]);
            % Calc std of yn_long
            yn_long_std = std(yn_long, 0, 2);
            % Normalize ys_ave_abs using yn_long_std
            yys = ys_ave_abs ./ repmat(yn_long_std, [1,Ns]);
            ymap = mean(yys, 2);
            % For timeseries plot
            yy = y_ave_abs ./ repmat(yn_long_std, [1,N]);
            % Plot
            fig_prefix = ['z3_'];
            fig_suffix = ['_' fileid];
            toi = twin_signal_sec;
            ytime = yy;
            disp_thresh = max(ymap)*0.3;
            label_time = 'Normalized current [a.u.]';
            label_map = ['Normalized averaged current (>' num2str(disp_thresh,'%.2e') ')'];
    end

    % Plot for temporal-averaged data (within ToI)
    fig_file = fullfile(p.fig_root, mfilename, p.dirname.modality, p.task, [fig_prefix p.sub fig_suffix]);
    inner_plot_and_save_figure(brain_file, angle, time_show, ytime, ymap, disp_thresh, toi, label_time, label_map, fig_file)
end
end

end % end of function

function inner_plot_and_save_figure(brain_file, angle, time_show, ytime, ymap, disp_thresh, time_of_interest, label_time, label_map, fig_file)
% Plot time-series of averaged MEG data
h = figure;
max_y = max(ytime(:));
min_y = min(ytime(:));
if time_of_interest(1)==time_of_interest(2)
    time_of_interest = time_of_interest(1);
end

subplot(2, 1, 1)
if length(time_of_interest)==2
    % Add filled region
    ar=area([time_of_interest(1) time_of_interest(2)],[max_y max_y], min_y);
    set(ar,'FaceColor',[.7 .7 .7]);
    set(ar,'LineStyle','none');
    set(gca,'layer','top');
end
hold on
% Plot time-series
plot(time_show, ytime, 'LineWidth',0.05)
%plot(time_show, ytime, 'LineWidth',0.05, 'Color',[0.7 0.7 0.7])
xlabel('Time [s]')
ylabel(label_time)
title(['Stimulus-triggered average of current'])
axis([time_show(1) time_show(end) min_y max_y])
grid on
if length(time_of_interest)==1
    % Add yline since filled region is not plotted
    plot([time_of_interest time_of_interest], [min_y max_y], 'k-', 'LineWidth',1.0)
end
set_axis

% Load brain model on which current is shown
[V, F, xx, inf_C] = vb_load_cortex(brain_file, 'Inflate');
max_xyz = max(abs(V), [], 1);
plot_parm = vb_set_plot_parm;

% Plot spatial pattern of absolute value of current
for an = 1:size(angle, 1)
    subplot(2, size(angle, 1), size(angle, 1)+an)
    vb_plot_cortex(plot_parm, V, F, inf_C, ymap, disp_thresh, max(ymap));
    axis([-max_xyz(1) max_xyz(1) -max_xyz(2) max_xyz(2) -max_xyz(3) max_xyz(3)])
    axis equal off
    view(angle(an, :))
    %zoom(1.5)
    if an==2
        if length(time_of_interest)==2
            title([label_map ' (' num2str(time_of_interest(1)) ' - ' num2str(time_of_interest(2)) ' sec)'])
        else
            title([label_map ' (' num2str(time_of_interest(1)) ' sec)'])
        end
    end
    if an==3
        colorbar
    end
end
colormap(my_parula())

% Save figure
vb_savefig_as_shown(h, fig_file)
disp([fig_file '.png was saved.'])

end % end of function