function show_source_current_dipole(p)
% Show source current estimated from MEG data by dipole-fitting
%
% Copyright (C) 2011, ATR All Rights Reserved.
% License : New BSD License(see VBMEG_LICENSE.txt)

disp(mfilename);
close all

% Set figure properties
set_fig_property(4, 2, 10, 10);

% For projecting dipoles to whole cortex
brain_file = fullfile(p.proj_root, p.dirname.brain, [p.struct_name, '.brain.mat']);
[V] = vb_load_cortex(brain_file, 'Inflate');
nvertex =  size(V,1); % All vertices of whole cortex

% Angles to see brain
angle = [-90 0; 90 0; 0 90];

% Set dir of currents
current_dir = fullfile(p.proj_root, p.dirname.modality, p.dirname.current);

% Single dipole on cortex and double dipole on both hemisphere
method1 = 'dipole1';
method2 = 'dipole2';

% Set dir and prefix of current file
prefix_fileid1 = [p.task '_' method1 '_'];
prefix_fileid2 = [p.task '_' method2 '_'];

% Find current files having specified fileid in current_dir
% current_files is struct
current_files1 = dir(fullfile(current_dir, [prefix_fileid1 '*' '.curr.mat']));
current_files2 = dir(fullfile(current_dir, [prefix_fileid2 '*' '.curr.mat']));

for ff=1:length(current_files1)
filename = current_files1(ff).name;
current_file1 = fullfile(current_dir, filename);
filename = current_files2(ff).name;
current_file2 = fullfile(current_dir, filename);

% Load temporal peak
load(current_file1, 'time_win_sec')

% Set time window [sec]
twin_show_sec   = p.time_show_sec;
twin_signal_sec = time_win_sec;
twin_noise_sec  = p.time_noise_sec;

% Load files
J1 = load(current_file1);
J2 = load(current_file2);

% Inverse filter is for estimated dipoles
inv_filt1 = J1.IF;
inv_filt2 = J2.IF;

% Set index of dipoles
dipole_ix1 = J1.Dindex;
dipole_ix2 = J2.Dindex;

% Observation data is common to single and double dipole
% bexp and time_info is saved in estimated current file
bexp = J1.bexp;
time_info = J1.time_info;
time_sec = time_info.time();

% Convert sec to index
[~, from_show_ix]   = min(abs(time_sec-twin_show_sec(1)));
[~, to_show_ix]     = min(abs(time_sec-twin_show_sec(2)));
[~, from_signal_ix] = min(abs(time_sec-twin_signal_sec(1)));
[~, to_signal_ix]   = min(abs(time_sec-twin_signal_sec(2)));
[~, from_noise_ix]  = min(abs(time_sec-twin_noise_sec(1)));
[~, to_noise_ix]    = min(abs(time_sec-twin_noise_sec(2)));

time_show_sec = time_sec(from_show_ix:to_show_ix);

% Trial-averaging
bexp_ave = mean(bexp, 3);

% Current reconstruction using inv filter
cur1 = inv_filt1 * bexp_ave;
cur2 = inv_filt2 * bexp_ave;

% Project time-series of dipoles to whole cortex
ntime = length(time_show_sec);
Jtime1 = zeros(nvertex, ntime);
Jtime2 = zeros(nvertex, ntime);
Jtime1(dipole_ix1,:) = cur1; 
Jtime2(dipole_ix2,:) = cur2; 

% Calc current map of dipoles after onset
Jtime1_toi = Jtime1(:,from_signal_ix:to_signal_ix);
Jtime2_toi = Jtime2(:,from_signal_ix:to_signal_ix);
Jmap1 = mean(abs(Jtime1_toi),2);
Jmap2 = mean(abs(Jtime2_toi),2);

%% Plot dipoles
% --- Single dipole ---
fig_prefix = [method1 '_'];
fig_suffix = ['_' num2str(ff)];
toi = twin_signal_sec;
disp_thresh = max(Jmap1)*0.000001;
label_time = 'Current [Am/m^2]';
label_map = ['Single dipole'];

% Plot for temporal-averaged data (within ToI)
fig_file = fullfile(p.fig_root, mfilename, p.dirname.modality, p.task, [fig_prefix p.sub fig_suffix]);
inner_plot_and_save_figure(brain_file, angle, time_show_sec, Jtime1, Jmap1, disp_thresh, toi, label_time, label_map, fig_file)

% --- Double dipoles ---
fig_prefix = [method2 '_'];
fig_suffix = ['_' num2str(ff)];
toi = twin_signal_sec;
disp_thresh = max(Jmap2)*0.000001;
label_time = 'Current [Am/m^2]';
label_map = ['Double dipoles'];

% Plot for temporal-averaged data (within ToI)
fig_file = fullfile(p.fig_root, mfilename, p.dirname.modality, p.task, [fig_prefix p.sub fig_suffix]);
inner_plot_and_save_figure(brain_file, angle, time_show_sec, Jtime2, Jmap2, disp_thresh, toi, label_time, label_map, fig_file)

%% Visualize Goodness of Fit for all vertices
% Set GoF
GoF = J1.GoF; % J1.GoF = J2.GoF

% Dummy time series
Jtime = Jtime1;

% --- Single dipole ---
fig_prefix = ['GoF_'];
fig_suffix = ['_' num2str(ff)];
toi = twin_signal_sec;
disp_thresh = 0;
label_map = ['Goodness of Fits'];

fig_file = fullfile(p.fig_root, mfilename, p.dirname.modality, p.task, [fig_prefix p.sub fig_suffix]);

% Load brain model on which current is shown
[V, F, xx, inf_C] = vb_load_cortex(brain_file, 'Inflate');
max_xyz = max(abs(V), [], 1);
plot_parm = vb_set_plot_parm;

h = figure;

% Plot spatial pattern of GoF (Left cortex)
for an = 1:size(angle, 1)
    plot_parm.LRflag = 'L';
    if an==3
        plot_parm.LRflag = 'LR';
    end
    subplot(2, size(angle, 1), an)
    vb_plot_cortex(plot_parm, V, F, inf_C, GoF, disp_thresh, max(GoF));
    axis([-max_xyz(1) max_xyz(1) -max_xyz(2) max_xyz(2) -max_xyz(3) max_xyz(3)])
    axis equal off
    view(angle(an, :))
    if an==2
        if length(toi)==2
            title([label_map ' (' num2str(toi(1)) ' - ' num2str(toi(2)) ' sec)'])
        else
            title([label_map ' (' num2str(toi(1)) ' sec)'])
        end
    end
    if an==3
        colorbar
    end
end

% Plot spatial pattern of GoF (Right cortex)
for an = 1:2
    plot_parm.LRflag = 'R';
    subplot(2, size(angle, 1), size(angle, 1)+an)
    vb_plot_cortex(plot_parm, V, F, inf_C, GoF, disp_thresh, max(GoF));
    axis([-max_xyz(1) max_xyz(1) -max_xyz(2) max_xyz(2) -max_xyz(3) max_xyz(3)])
    axis equal off
    view(angle(an, :))
end
colormap(my_parula())

% Save figure
vb_savefig_as_shown(h, fig_file)
disp([fig_file '.png was saved.'])

end % end of peak loop

end % end of function

function inner_plot_and_save_figure(brain_file, angle, time_show, ytime, ymap, disp_thresh, time_of_interest, label_time, label_map, fig_file)
% Plot time series of averaged MEG data
h = figure;
max_y = max(ytime(:));
min_y = min(ytime(:));
if time_of_interest(1)==time_of_interest(2)
    time_of_interest = time_of_interest(1);
end

subplot(2, 1, 1)
if length(time_of_interest)==2
    % Add filled region
    ar=area([time_of_interest(1) time_of_interest(2)],[max_y max_y], min_y);
    set(ar,'FaceColor',[.7 .7 .7]);
    set(ar,'LineStyle','none');
    set(gca,'layer','top');
end
hold on
% Plot time-series
plot(time_show, ytime, 'LineWidth',0.05)
%plot(time_show, ytime, 'LineWidth',0.05, 'Color',[0.7 0.7 0.7])
xlabel('Time [s]')
ylabel(label_time)
title(['Stimulus-triggered average of current'])
axis([time_show(1) time_show(end) min_y max_y])
grid on
if length(time_of_interest)==1
    % Add yline since filled region is not plotted
    plot([time_of_interest time_of_interest], [min_y max_y], 'k-', 'LineWidth',1.0)
end
set_axis

% Load brain model on which current is shown
[V, F, xx, inf_C] = vb_load_cortex(brain_file, 'Inflate');
max_xyz = max(abs(V), [], 1);
plot_parm = vb_set_plot_parm;

% Plot spatial pattern of absolute value of current
for an = 1:size(angle, 1)
    subplot(2, size(angle, 1), size(angle, 1)+an)
    vb_plot_cortex(plot_parm, V, F, inf_C, ymap, disp_thresh, max(ymap));
    axis([-max_xyz(1) max_xyz(1) -max_xyz(2) max_xyz(2) -max_xyz(3) max_xyz(3)])
    axis equal off
    view(angle(an, :))
    %zoom(1.5)
    if an==2
        if length(time_of_interest)==2
            title([label_map ' (' num2str(time_of_interest(1)) ' - ' num2str(time_of_interest(2)) ' sec)'])
        else
            title([label_map ' (' num2str(time_of_interest(1)) ' sec)'])
        end
    end
    if an==3
        colorbar
    end
end
colormap(my_parula())

% Save figure
vb_savefig_as_shown(h, fig_file)
disp([fig_file '.png was saved.'])

end % end of function