function  show_trial_average(p, prefix_in)
% Show trial-averaged MEG data
% If 'time_peak_win_sec' are saved in data_file, 
% it will be used instead of detected one.
%
% Copyright (C) 2011, ATR All Rights Reserved.
% License : New BSD License(see VBMEG_LICENSE.txt)

disp(mfilename);

% No additional prefix

% Set prefix of input file
if isempty(prefix_in)
    prefix_in_ = [];
else
    prefix_in_ = [prefix_in '_'];
end

% Set figure properties
set_fig_property(3, 2, 10, 10);

% Set parameters
twin_to_show_sec = p.time_show_sec; % sec
twin_of_interest_sec = p.time_of_interest_sec; % sec

% Set input file
data_file = fullfile(p.proj_root, p.dirname.modality, p.dirname.trial, [prefix_in_ p.task '.info.mat']);

% Load time information
[~, ~, time_info] = vb_load_meg_data(data_file);

% sec -> index
[~, from_show_ix] = min(abs(time_info.time-twin_to_show_sec(1)));
[~, to_show_ix] = min(abs(time_info.time-twin_to_show_sec(2)));
time_show = time_info.time(from_show_ix: to_show_ix);
% TOI index within time for showing
[~, from_toi_ix] = min(abs(time_show-twin_of_interest_sec(1)));
[~, to_toi_ix] = min(abs(time_show-twin_of_interest_sec(2)));

% Load temporal peaks detected in find_temporal_peak.m
load(data_file, 'time_peak_sec', 'time_peak_win_sec')

% Load positions of channels
[pos, channel_info] = vb_load_channel(data_file);

% Set modality-specific parameters
switch p.dirname.modality
    case 'opm-meg'
        % Set Ylabel of figure
        label_y = 'Magnetic field [T]';
        % Extract z and y ch
        chix_z = channel_info.Type==6;
        chix_y = channel_info.Type==5;
        % Define num rows of subplot
        if any(chix_y) && any(chix_z)
            naxis = 2;
            CHIX = {find(chix_z), find(chix_y)};
            TITLES_AX = {'-Z', 'Y'};
        elseif any(chix_z)
            naxis = 1;
            CHIX = {find(chix_z)};
            TITLES_AX = {'Z'};
        elseif any(chix_y)
            naxis = 1;
            CHIX = {find(chix_y)};
            TITLES_AX = {'Y'};
        end    

    case 'squid-meg'
        % Set Ylabel of figure
        label_y = 'Magnetic field [T]';
        % Extract axial ch
        chix_axial = channel_info.Type==2;
        % Define num rows of subplot
        naxis = 1;
        CHIX = {find(chix_axial)};
        TITLES_AX = {'Axial'};

    case 'eeg'
        % Set Ylabel of figure
        label_y = 'Potential [V]';
        % Extract axial ch
        chix_eeg = channel_info.Type==1;
        % Define num rows of subplot
        naxis = 1;
        CHIX = {find(chix_eeg)};
        TITLES_AX = {'EEG'};
end

% Load MEG data
bexp = vb_load_meg_data(data_file);

% Average MEG data across trials
data = mean(bexp(:, from_show_ix:to_show_ix, :), 3);

% To show the temporal peaks, run find_temporal_peak.m before this function
% time_peak_win_sec is detected in find_temporal_peak.m
if exist('time_peak_win_sec','var') && ~isempty(time_peak_win_sec)
    % Get index corresponds to the time for showing
    time_peak_win_ix = zeros(size(time_peak_win_sec));
    for pp=1:size(time_peak_win_sec,1)
        % From
        [~, time_peak_win_ix(pp,1)] = min(abs(time_show-time_peak_win_sec(pp,1)));
        % To
        [~, time_peak_win_ix(pp,2)] = min(abs(time_show-time_peak_win_sec(pp,2)));
    end
else
    time_peak_win_ix = [];
end

%% Show trial-averaged MEG data
% Make a figure for averaged data within ToIs
h = figure;
inner_plot(time_show, data, pos, [from_toi_ix,to_toi_ix], label_y, naxis, CHIX, TITLES_AX)
% Save figure
fig_file = fullfile(p.fig_root, mfilename, p.dirname.modality, p.task, [p.sub]);
vb_savefig_as_shown(h, fig_file)
disp([fig_file '.png was saved.'])

% Make figures with peaks
if ~isempty(time_peak_win_ix)
    for peak = 1:size(time_peak_win_ix,1)
        h = figure;
        toi = time_peak_win_ix(peak,:);
        inner_plot(time_show, data, pos, toi, label_y, naxis, CHIX, TITLES_AX)
        % Save figure
        fig_file = fullfile(p.fig_root, mfilename, p.dirname.modality, p.task, [p.sub '_' num2str(peak)]);
        vb_savefig_as_shown(h, fig_file)
        disp([fig_file '.png was saved.'])
    end
end

function inner_plot(time, data, pos, ix_peak, label_y, naxis, CHIX, TITLES_AX)
max_y = max(data(:));
min_y = min(data(:));
meg_peak = time(ix_peak);
if meg_peak(1)==meg_peak(2)
    meg_peak = meg_peak(1);
end

%--- Plot time series of averaged MEG data ---
subplot(2, 1, 1)
if length(meg_peak)==2
    % Add filled region
    ar=area([meg_peak(1) meg_peak(2)],[max_y max_y], min_y);
    set(ar,'FaceColor',[.7 .7 .7]);
    set(ar,'LineStyle','none');
    set(gca,'layer','top');
end
hold on
% Plot data
plot(time, data)
if length(meg_peak)==1
    % Add yline since filled region is not plotted
    plot([meg_peak meg_peak], [min_y max_y], 'k-', 'LineWidth',1.0)
end
xlabel('Time [s]')
ylabel(label_y)
title(['Stimulus-triggered average'])
axis([time(1) time(end) min_y max_y])
grid on
set_axis

%--- Plot spatial patterns at peaks ---
%ma = max(abs(y(:)));
data_map = mean(data(:,ix_peak(1):ix_peak(end)),2);
ma = max(abs(data_map));
for ax_plt = 1:naxis
    % Separate data
    chix = CHIX{ax_plt};
    data_plt = data_map(chix);
    pos_plt = pos(chix,:);
    title_ax = TITLES_AX{ax_plt};

    % Invert the map when plotting axis is specified to a negative direction
    if strcmp(title_ax(1), '-') 
        data_plt = -data_plt;
    end

    subplot(2, naxis, ax_plt+naxis)
    vb_plot_sensor_2d(pos_plt, data_plt, [-ma ma], true);
    axis square off
    if length(meg_peak)==2
        title([num2str(meg_peak(1)) ':' num2str(meg_peak(2)) ' s (' title_ax ')'])
    else
        title([num2str(meg_peak) ' s (' title_ax ')'])
    end
    colormap(my_parula())
    colorbar
end